@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,16 +16,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 624 $
+ * $Date: 2013-04-08 23:04:29 +0200 (Mon, 08 Apr 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -110,12 +110,27 @@ void ArticleDownloader::SetInfoName(const char * v)
 	m_szInfoName = strdup(v);
 }
 
-void ArticleDownloader::SetStatus(EStatus eStatus)
-{
-	m_eStatus = eStatus;
-	Notify(NULL);
-}
+/*
+ * How server management (for one particular article) works:
+	- there is a list of failed servers which is initially empty;
+	- level is initially 0;
 
+	<loop>
+		- request a connection from server pool for current level;
+		  Exception: this step is skipped for the very first download attempt, because a
+		  level-0 connection is initially passed from queue manager;
+		- try to download from server;
+		- if connection to server cannot be established or download fails due to interrupted connection,
+		  try again (as many times as needed without limit) the same server until connection is OK;
+		- if download fails with error "Not-Found" (article or group not found) or with CRC error,
+		  add the server to failed server list;
+		- if download fails with general failure error (article incomplete, other unknown error
+		  codes), try the same server again as many times as defined by option <Retries>; if all attempts
+		  fail, add the server to failed server list;
+		- if all servers from current level were tried, increase level;
+		- if all servers from all levels were tried, break the loop with failure status.
+	<end-loop>
+*/
 void ArticleDownloader::Run()
 {
 	debug("Entering ArticleDownloader-loop");
@@ -129,59 +144,44 @@ void ArticleDownloader::Run()
 		{
 			// file exists from previous program's start
 			detail("Article %s already downloaded, skipping", m_szInfoName);
-			SetStatus(adFinished);
 			FreeConnection(true);
+			SetStatus(adFinished);
+			Notify(NULL);
 			return;
 		}
 	}
 
-	int iRemainedDownloadRetries = g_pOptions->GetRetries() > 0 ? g_pOptions->GetRetries() : 1;
-
-#ifdef THREADCONNECT_WORKAROUND
-	// NOTE: about iRemainedConnectRetries:
-	// Sometimes connections just do not want to work in a particular thread,
-	// regardless of retry count. However they work in other threads.
-	// If ArticleDownloader can't start download after many attempts, it terminates
-	// and let QueueCoordinator retry the article in a new thread.
-	// It wasn't confirmed that this workaround actually helps.
-	// Therefore it is disabled by default. Define symbol "THREADCONNECT_WORKAROUND"
-	// to activate the workaround.
-	int iRemainedConnectRetries = iRemainedDownloadRetries > 5 ? iRemainedDownloadRetries * 2 : 10;
-#endif
-
 	EStatus Status = adFailed;
-	int iMaxLevel = g_pServerPool->GetMaxLevel();
-	int* LevelStatus = (int*)malloc((iMaxLevel + 1) * sizeof(int));
-	for (int i = 0; i <= iMaxLevel; i++)
-	{
-		LevelStatus[i] = 0;
-	}
-	int level = 0;
 
-	while (!IsStopped() && iRemainedDownloadRetries > 0)
-	{
-		SetLastUpdateTimeNow();
+	int iRetries = g_pOptions->GetRetries() > 0 ? g_pOptions->GetRetries() : 1;
+	int iRemainedRetries = iRetries;
+	ServerPool::Servers failedServers;
+	failedServers.reserve(g_pServerPool->GetServers()->size());
+	NewsServer* pWantServer = NULL;
+	NewsServer* pLastServer = NULL;
+	int iLevel = 0;
 
+	while (!IsStopped())
+	{
 		Status = adFailed;
 
+		SetStatus(adWaiting);
 		while (!IsStopped() && !m_pConnection)
 		{
-			m_pConnection = g_pServerPool->GetConnection(level);
+			m_pConnection = g_pServerPool->GetConnection(iLevel, pWantServer, &failedServers);
 			usleep(5 * 1000);
 		}
+		SetLastUpdateTimeNow();
+		SetStatus(adRunning);
 
-		if (IsStopped())
-		{
-			Status = adFailed;
-			break;
-		}
-
-		if (g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
+		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
 		{
 			Status = adRetry;
 			break;
 		}
 
+		pLastServer = m_pConnection->GetNewsServer();
+
 		m_pConnection->SetSuppressErrors(false);
 
 		// test connection
@@ -189,7 +189,8 @@ void ArticleDownloader::Run()
 		if (bConnected && !IsStopped())
 		{
 			// Okay, we got a Connection. Now start downloading.
-			detail("Downloading %s @ %s", m_szInfoName, m_pConnection->GetServer()->GetHost());
+			detail("Downloading %s @ server%i (%s)", m_szInfoName,
+				m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost());
 			Status = Download();
 		}
 
@@ -200,9 +201,6 @@ void ArticleDownloader::Run()
 				m_pConnection->Disconnect();
 				bConnected = false;
 				Status = adFailed;
-#ifdef THREADCONNECT_WORKAROUND
-				iRemainedConnectRetries--;
-#endif
 			}
 			else
 			{
@@ -212,89 +210,103 @@ void ArticleDownloader::Run()
 				// free the connection, to prevent starting of thousands of threads 
 				// (cause each of them will also free it's connection after the 
 				// same connect-error).
-				FreeConnection(Status == adFinished);
+				FreeConnection(Status == adFinished || Status == adNotFound);
 			}
 		}
-#ifdef THREADCONNECT_WORKAROUND
-		else
-		{
-			iRemainedConnectRetries--;
-		}
 
-		if (iRemainedConnectRetries == 0)
+		if (Status == adFinished || Status == adFatalError)
 		{
-			debug("Can't connect from this thread, retry later from another");
-			Status = adRetry;
-			break;
-		}
-#endif
-
-		if (g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
-		{
-			Status = adRetry;
 			break;
 		}
 
-		if (((Status == adFailed) || (Status == adCrcError && g_pOptions->GetRetryOnCrcError())) && 
-			(iRemainedDownloadRetries > 1 || !bConnected) && !IsStopped())
+		pWantServer = NULL;
+
+		if (bConnected && Status == adFailed)
+		{
+			iRemainedRetries--;
+		}
+
+		if (!bConnected || (Status == adFailed && iRemainedRetries > 0))
+		{
+			pWantServer = pLastServer;
+		}
+
+		if (pWantServer && !IsStopped() &&
+			!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
 		{
 			detail("Waiting %i sec to retry", g_pOptions->GetRetryInterval());
+			SetStatus(adWaiting);
 			int msec = 0;
-			while (!IsStopped() && (msec < g_pOptions->GetRetryInterval() * 1000))
+			while (!IsStopped() && (msec < g_pOptions->GetRetryInterval() * 1000) && 
+				!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
 			{
 				usleep(100 * 1000);
 				msec += 100;
 			}
+			SetLastUpdateTimeNow();
+			SetStatus(adRunning);
 		}
 
-		if (IsStopped())
-		{
-			Status = adFailed;
-			break;
-		}
-	 
-		if ((Status == adFinished) || (Status == adFatalError) ||
-			(Status == adCrcError && !g_pOptions->GetRetryOnCrcError()))
+		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
 		{
+			Status = adRetry;
 			break;
 		}
 
-		LevelStatus[level] = Status;
+		if (!pWantServer)
+		{
+			failedServers.push_back(pLastServer);
 
-		bool bAllLevelNotFound = true;
-		for (int lev = 0; lev <= iMaxLevel; lev++)
-		{
-			if (LevelStatus[lev] != adNotFound)
-			{
-				bAllLevelNotFound = false;
-				break;
-			}
-		}
-		if (bAllLevelNotFound)
-		{
-			if (iMaxLevel > 0)
-			{
-				warn("Article %s @ all servers failed: Article not found", m_szInfoName);
-			}
-			break;
-		}
+			// if all servers from current level were tried, increase level
+			// if all servers from all levels were tried, break the loop with failure status
 
-		// do not count connect-errors, only article- and group-errors
-		if (bConnected)
-		{
-			level++;
-			if (level > iMaxLevel)
+			bool bAllServersOnLevelFailed = true;
+			for (ServerPool::Servers::iterator it = g_pServerPool->GetServers()->begin(); it != g_pServerPool->GetServers()->end(); it++)
 			{
-				level = 0;
+				NewsServer* pCandidateServer = *it;
+				if (pCandidateServer->GetLevel() == iLevel)
+				{
+					bool bServerFailed = false;
+					for (ServerPool::Servers::iterator it = failedServers.begin(); it != failedServers.end(); it++)
+					{
+						NewsServer* pIgnoreServer = *it;
+						if (pIgnoreServer == pCandidateServer ||
+							(pIgnoreServer->GetGroup() > 0 && pIgnoreServer->GetGroup() == pCandidateServer->GetGroup() &&
+							 pIgnoreServer->GetLevel() == pCandidateServer->GetLevel()))
+						{
+							bServerFailed = true;
+							break;
+						}					
+					}
+					if (!bServerFailed)
+					{
+						bAllServersOnLevelFailed = false;
+						break;
+					}
+				}
 			}
-			iRemainedDownloadRetries--;
+
+			if (bAllServersOnLevelFailed)
+			{
+				if (iLevel < g_pServerPool->GetMaxLevel())
+				{
+					detail("Article %s @ all level %i servers failed, increasing level", m_szInfoName, iLevel);
+					iLevel++;
+				}
+				else
+				{
+					warn("Article %s @ all servers failed", m_szInfoName);
+					Status = adFailed;
+					break;
+				}
+			}
+			
+			iRemainedRetries = iRetries;
 		}
 	}
 
 	FreeConnection(Status == adFinished);
 
-	free(LevelStatus);
-
 	if (m_bDuplicate)
 	{
 		Status = adFinished;
@@ -305,19 +317,19 @@ void ArticleDownloader::Run()
 		Status = adFailed;
 	}
 
+	if (IsStopped())
+	{
+		detail("Download %s cancelled", m_szInfoName);
+		Status = adRetry;
+	}
+
 	if (Status == adFailed)
 	{
-		if (IsStopped())
-		{
-			detail("Download %s cancelled", m_szInfoName);
-		}
-		else
-		{
-			warn("Download %s failed", m_szInfoName);
-		}
+		warn("Download %s failed", m_szInfoName);
 	}
 
 	SetStatus(Status);
+	Notify(NULL);
 
 	debug("Exiting ArticleDownloader-loop");
 }
@@ -405,7 +417,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 		{
 			if (!IsStopped())
 			{
-				warn("Article %s @ %s failed: Unexpected end of article", m_szInfoName, m_pConnection->GetServer()->GetHost());
+				warn("Article %s @ server%i (%s) failed: Unexpected end of article", m_szInfoName,
+					m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost());
 			}
 			Status = adFailed;
 			break;
@@ -439,7 +452,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 				if (strncmp(p, m_pArticleInfo->GetMessageID(), strlen(m_pArticleInfo->GetMessageID())))
 				{
 					if (char* e = strrchr(p, '\r')) *e = '\0'; // remove trailing CR-character
-					warn("Article %s @ %s failed: Wrong message-id, expected %s, returned %s", m_szInfoName, m_pConnection->GetServer()->GetHost(), m_pArticleInfo->GetMessageID(), p);
+					warn("Article %s @ server%i (%s) failed: Wrong message-id, expected %s, returned %s", m_szInfoName,
+						m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), m_pArticleInfo->GetMessageID(), p);
 					Status = adFailed;
 					break;
 				}
@@ -467,7 +481,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 
 	if (!bEnd && Status == adRunning && !IsStopped())
 	{
-		warn("Article %s @ %s failed: article incomplete", m_szInfoName, m_pConnection->GetServer()->GetHost());
+		warn("Article %s @ server%i (%s) failed: article incomplete", m_szInfoName,
+			m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost());
 		Status = adFailed;
 	}
 
@@ -494,18 +509,21 @@ ArticleDownloader::EStatus ArticleDownloader::CheckResponse(const char* szRespon
 	{
 		if (!IsStopped())
 		{
-			warn("Article %s @ %s failed, %s: Connection closed by remote host", m_szInfoName, m_pConnection->GetServer()->GetHost(), szComment);
+			warn("Article %s @ server%i (%s) failed, %s: Connection closed by remote host", m_szInfoName, 
+				m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment);
 		}
 		return adConnectError;
 	}
 	else if (m_pConnection->GetAuthError() || !strncmp(szResponse, "400", 3) || !strncmp(szResponse, "499", 3))
 	{
-		warn("Article %s @ %s failed, %s: %s", m_szInfoName, m_pConnection->GetServer()->GetHost(), szComment, szResponse);
+		warn("Article %s @ server%i (%s) failed, %s: %s", m_szInfoName,
+			 m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment, szResponse);
 		return adConnectError;
 	}
 	else if (!strncmp(szResponse, "41", 2) || !strncmp(szResponse, "42", 2) || !strncmp(szResponse, "43", 2))
 	{
-		warn("Article %s @ %s failed, %s: %s", m_szInfoName, m_pConnection->GetServer()->GetHost(), szComment, szResponse);
+		warn("Article %s @ server%i (%s) failed, %s: %s", m_szInfoName,
+			 m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment, szResponse);
 		return adNotFound;
 	}
 	else if (!strncmp(szResponse, "2", 1))
@@ -516,7 +534,8 @@ ArticleDownloader::EStatus ArticleDownloader::CheckResponse(const char* szRespon
 	else 
 	{
 		// unknown error, no special handling
-		warn("Article %s @ %s failed, %s: %s", m_szInfoName, m_pConnection->GetServer()->GetHost(), szComment, szResponse);
+		warn("Article %s @ server%i (%s) failed, %s: %s", m_szInfoName,
+			 m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment, szResponse);
 		return adFailed;
 	}
 }
@@ -607,9 +626,8 @@ bool ArticleDownloader::PrepareFile(char* szLine)
 					{
 						pb += 6; //=strlen(" size=")
 						long iArticleFilesize = atol(pb);
-						if (!Util::SetFileSize(m_szOutputFilename, iArticleFilesize))
+						if (!CreateOutputFile(iArticleFilesize))
 						{
-							error("Could not create file %s!", m_szOutputFilename);
 							return false;
 						}
 						m_pFileInfo->SetOutputInitialized(true);
@@ -652,6 +670,41 @@ bool ArticleDownloader::PrepareFile(char* szLine)
 	return true;
 }
 
+/* creates output file and subdirectores */
+bool ArticleDownloader::CreateOutputFile(int iSize)
+{
+	if (g_pOptions->GetDirectWrite() && Util::FileExists(m_szOutputFilename) &&
+		Util::FileSize(m_szOutputFilename) == iSize)
+	{
+		// keep existing old file from previous program session
+		return true;
+	}
+		
+	// delete eventually existing old file from previous program session
+	remove(m_szOutputFilename);
+
+	// ensure the directory exist
+	char szDestDir[1024];
+	int iMaxlen = Util::BaseFileName(m_szOutputFilename) - m_szOutputFilename;
+	if (iMaxlen > 1024-1) iMaxlen = 1024-1;
+	strncpy(szDestDir, m_szOutputFilename, iMaxlen);
+	szDestDir[iMaxlen] = '\0';
+
+	if (!Util::ForceDirectories(szDestDir))
+	{
+		error("Could not create directory %s! Errcode: %i", szDestDir, errno);
+		return false;
+	}
+
+	if (!Util::CreateSparseFile(m_szOutputFilename, iSize))
+	{
+		error("Could not create file %s!", m_szOutputFilename);
+		return false;
+	}
+
+	return true;
+}
+
 ArticleDownloader::EStatus ArticleDownloader::DecodeCheck()
 {
 	bool bDirectWrite = g_pOptions->GetDirectWrite() && m_eFormat == Decoder::efYenc;
@@ -823,11 +876,18 @@ void ArticleDownloader::CompleteFileParts()
 
 	bool bDirectWrite = g_pOptions->GetDirectWrite() && m_pFileInfo->GetOutputInitialized();
 
-	char szNZBNiceName[1024];
-	m_pFileInfo->GetNZBInfo()->GetNiceNZBName(szNZBNiceName, 1024);
+	char szNZBName[1024];
+	char szNZBDestDir[1024];
+	// the locking is needed for accessing the memebers of NZBInfo
+	g_pDownloadQueueHolder->LockQueue();
+	strncpy(szNZBName, m_pFileInfo->GetNZBInfo()->GetName(), 1024);
+	strncpy(szNZBDestDir, m_pFileInfo->GetNZBInfo()->GetDestDir(), 1024);
+	g_pDownloadQueueHolder->UnlockQueue();
+	szNZBName[1024-1] = '\0';
+	szNZBDestDir[1024-1] = '\0';
 	
 	char InfoFilename[1024];
-	snprintf(InfoFilename, 1024, "%s%c%s", szNZBNiceName, (int)PATH_SEPARATOR, m_pFileInfo->GetFilename());
+	snprintf(InfoFilename, 1024, "%s%c%s", szNZBName, (int)PATH_SEPARATOR, m_pFileInfo->GetFilename());
 	InfoFilename[1024-1] = '\0';
 
 	if (!g_pOptions->GetDecode())
@@ -843,13 +903,6 @@ void ArticleDownloader::CompleteFileParts()
 		detail("Joining articles for %s", InfoFilename);
 	}
 
-	char szNZBDestDir[1024];
-	// the locking is needed for accessing the memebers of NZBInfo
-	g_pDownloadQueueHolder->LockQueue();
-	strncpy(szNZBDestDir, m_pFileInfo->GetNZBInfo()->GetDestDir(), 1024);
-	g_pDownloadQueueHolder->UnlockQueue();
-	szNZBDestDir[1024-1] = '\0';
-
 	// Ensure the DstDir is created
 	if (!Util::ForceDirectories(szNZBDestDir))
 	{
@@ -983,6 +1036,24 @@ void ArticleDownloader::CompleteFileParts()
 		{
 			error("Could not move file %s to %s! Errcode: %i", m_szOutputFilename, ofn, errno);
 		}
+
+		// if destination directory was changed delete the old directory (if empty)
+		int iLen = strlen(szNZBDestDir);
+		if (!(!strncmp(szNZBDestDir, m_szOutputFilename, iLen) && 
+			(m_szOutputFilename[iLen] == PATH_SEPARATOR || m_szOutputFilename[iLen] == ALT_PATH_SEPARATOR)))
+		{
+			debug("Checking old dir for: %s", m_szOutputFilename);
+			char szOldDestDir[1024];
+			int iMaxlen = Util::BaseFileName(m_szOutputFilename) - m_szOutputFilename;
+			if (iMaxlen > 1024-1) iMaxlen = 1024-1;
+			strncpy(szOldDestDir, m_szOutputFilename, iMaxlen);
+			szOldDestDir[iMaxlen] = '\0';
+			if (Util::DirEmpty(szOldDestDir))
+			{
+				debug("Deleting old dir: %s", szOldDestDir);
+				rmdir(szOldDestDir);
+			}
+		}
 	}
 
 	if (!bDirectWrite || g_pOptions->GetContinuePartial())
