@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2011 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 567 $
+ * $Date: 2013-02-15 09:04:23 +0100 (Fri, 15 Feb 2013) $
  *
  */
 
@@ -48,19 +48,23 @@
 #include "Log.h"
 #include "Options.h"
 #include "QueueCoordinator.h"
+#include "UrlCoordinator.h"
 #include "QueueEditor.h"
 #include "PrePostProcessor.h"
 #include "Util.h"
+#include "DownloadInfo.h"
 
 extern Options* g_pOptions;
 extern QueueCoordinator* g_pQueueCoordinator;
+extern UrlCoordinator* g_pUrlCoordinator;
 extern PrePostProcessor* g_pPrePostProcessor;
 extern void ExitProc();
+extern void Reload();
 
 const char* g_szMessageRequestNames[] =
     { "N/A", "Download", "Pause/Unpause", "List", "Set download rate", "Dump debug", 
-		"Edit queue", "Log", "Quit", "Version", "Post-queue", "Write log", "Scan", 
-		"Pause/Unpause postprocessor", "History" };
+		"Edit queue", "Log", "Quit", "Reload", "Version", "Post-queue", "Write log", "Scan", 
+		"Pause/Unpause postprocessor", "History", "Download URL", "URL-queue" };
 
 const unsigned int g_iMessageRequestSizes[] =
     { 0,
@@ -72,39 +76,40 @@ const unsigned int g_iMessageRequestSizes[] =
 		sizeof(SNZBEditQueueRequest),
 		sizeof(SNZBLogRequest),
 		sizeof(SNZBShutdownRequest),
+		sizeof(SNZBReloadRequest),
 		sizeof(SNZBVersionRequest),
 		sizeof(SNZBPostQueueRequest),
 		sizeof(SNZBWriteLogRequest),
 		sizeof(SNZBScanRequest),
-		sizeof(SNZBHistoryRequest)
+		sizeof(SNZBHistoryRequest),
+		sizeof(SNZBDownloadUrlRequest),
+		sizeof(SNZBUrlQueueRequest)
     };
 
 //*****************************************************************
 // BinProcessor
 
+BinRpcProcessor::BinRpcProcessor()
+{
+	m_MessageBase.m_iSignature = (int)NZBMESSAGE_SIGNATURE;
+}
+
 void BinRpcProcessor::Execute()
 {
 	// Read the first package which needs to be a request
-	int iBytesReceived = recv(m_iSocket, ((char*)&m_MessageBase) + sizeof(m_MessageBase.m_iSignature), sizeof(m_MessageBase) - sizeof(m_MessageBase.m_iSignature), 0);
-	if (iBytesReceived < 0)
+	if (!m_pConnection->Recv(((char*)&m_MessageBase) + sizeof(m_MessageBase.m_iSignature), sizeof(m_MessageBase) - sizeof(m_MessageBase.m_iSignature)))
 	{
+		warn("Non-nzbget request received on port %i from %s", g_pOptions->GetControlPort(), m_pConnection->GetRemoteAddr());
 		return;
 	}
 
-	// Make sure this is a nzbget request from a client
-	if ((int)ntohl(m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE)
+	if (strcmp(m_MessageBase.m_szPassword, g_pOptions->GetControlPassword()))
 	{
-		warn("Non-nzbget request received on port %i from %s", g_pOptions->GetServerPort(), m_szClientIP);
+		warn("nzbget request received on port %i from %s, but password invalid", g_pOptions->GetControlPort(), m_pConnection->GetRemoteAddr());
 		return;
 	}
 
-	if (strcmp(m_MessageBase.m_szPassword, g_pOptions->GetServerPassword()))
-	{
-		warn("nzbget request received on port %i from %s, but password invalid", g_pOptions->GetServerPort(), m_szClientIP);
-		return;
-	}
-
-	debug("%s request received from %s", g_szMessageRequestNames[ntohl(m_MessageBase.m_iType)], m_szClientIP);
+	debug("%s request received from %s", g_szMessageRequestNames[ntohl(m_MessageBase.m_iType)], m_pConnection->GetRemoteAddr());
 
 	Dispatch();
 }
@@ -156,6 +161,10 @@ void BinRpcProcessor::Dispatch()
 			command = new ShutdownBinCommand();
 			break;
 
+		case eRemoteRequestReload:
+			command = new ReloadBinCommand();
+			break;
+
 		case eRemoteRequestVersion:
 			command = new VersionBinCommand();
 			break;
@@ -176,6 +185,14 @@ void BinRpcProcessor::Dispatch()
 			command = new HistoryBinCommand();
 			break;
 
+		case eRemoteRequestDownloadUrl:
+			command = new DownloadUrlBinCommand();
+			break;
+
+		case eRemoteRequestUrlQueue:
+			command = new UrlQueueBinCommand();
+			break;
+
 		default:
 			error("Received unsupported request %i", ntohl(m_MessageBase.m_iType));
 			break;
@@ -183,7 +200,7 @@ void BinRpcProcessor::Dispatch()
 
 	if (command)
 	{
-		command->SetSocket(m_iSocket);
+		command->SetConnection(m_pConnection);
 		command->SetMessageBase(&m_MessageBase);
 		command->Execute();
 		delete command;
@@ -205,8 +222,8 @@ void BinCommand::SendBoolResponse(bool bSuccess, const char* szText)
 	BoolResponse.m_iTrailingDataLength = htonl(iTextLen);
 
 	// Send the request answer
-	send(m_iSocket, (char*) &BoolResponse, sizeof(BoolResponse), 0);
-	send(m_iSocket, (char*)szText, iTextLen, 0);
+	m_pConnection->Send((char*) &BoolResponse, sizeof(BoolResponse));
+	m_pConnection->Send((char*)szText, iTextLen);
 }
 
 bool BinCommand::ReceiveRequest(void* pBuffer, int iSize)
@@ -215,8 +232,7 @@ bool BinCommand::ReceiveRequest(void* pBuffer, int iSize)
 	iSize -= sizeof(SNZBRequestBase);
 	if (iSize > 0)
 	{
-		int iBytesReceived = recv(m_iSocket, ((char*)pBuffer) + sizeof(SNZBRequestBase), iSize, 0);
-		if (iBytesReceived != iSize)
+		if (!m_pConnection->Recv(((char*)pBuffer) + sizeof(SNZBRequestBase), iSize))
 		{
 			error("invalid request");
 			return false;
@@ -233,6 +249,8 @@ void PauseUnpauseBinCommand::Execute()
 		return;
 	}
 
+	g_pOptions->SetResumeTime(0);
+
 	switch (ntohl(PauseUnpauseRequest.m_iAction))
 	{
 		case eRemotePauseUnpauseActionDownload:
@@ -263,7 +281,7 @@ void SetDownloadRateBinCommand::Execute()
 		return;
 	}
 
-	g_pOptions->SetDownloadRate(ntohl(SetDownloadRequest.m_iDownloadRate) / 1024.0f);
+	g_pOptions->SetDownloadRate(ntohl(SetDownloadRequest.m_iDownloadRate));
 	SendBoolResponse(true, "Rate-Command completed successfully");
 }
 
@@ -276,6 +294,7 @@ void DumpDebugBinCommand::Execute()
 	}
 
 	g_pQueueCoordinator->LogDebugInfo();
+	g_pUrlCoordinator->LogDebugInfo();
 	SendBoolResponse(true, "Debug-Command completed successfully");
 }
 
@@ -291,6 +310,18 @@ void ShutdownBinCommand::Execute()
 	ExitProc();
 }
 
+void ReloadBinCommand::Execute()
+{
+	SNZBReloadRequest ReloadRequest;
+	if (!ReceiveRequest(&ReloadRequest, sizeof(ReloadRequest)))
+	{
+		return;
+	}
+
+	SendBoolResponse(true, "Reloading server");
+	Reload();
+}
+
 void VersionBinCommand::Execute()
 {
 	SNZBVersionRequest VersionRequest;
@@ -311,46 +342,44 @@ void DownloadBinCommand::Execute()
 	}
 
 	char* pRecvBuffer = (char*)malloc(ntohl(DownloadRequest.m_iTrailingDataLength) + 1);
-	char* pBufPtr = pRecvBuffer;
 
-	// Read from the socket until nothing remains
-	int iResult = 0;
-	int NeedBytes = ntohl(DownloadRequest.m_iTrailingDataLength);
-	while (NeedBytes > 0)
+	if (!m_pConnection->Recv(pRecvBuffer, ntohl(DownloadRequest.m_iTrailingDataLength)))
 	{
-		iResult = recv(m_iSocket, pBufPtr, NeedBytes, 0);
-		// Did the recv succeed?
-		if (iResult <= 0)
-		{
-			error("invalid request");
-			break;
-		}
-		pBufPtr += iResult;
-		NeedBytes -= iResult;
+		error("invalid request");
+		free(pRecvBuffer);
+		return;
 	}
-
-	if (NeedBytes == 0)
+	
+	int iPriority = ntohl(DownloadRequest.m_iPriority);
+	bool bAddPaused = ntohl(DownloadRequest.m_bAddPaused);
+	
+	NZBFile* pNZBFile = NZBFile::CreateFromBuffer(DownloadRequest.m_szFilename, DownloadRequest.m_szCategory, pRecvBuffer, ntohl(DownloadRequest.m_iTrailingDataLength));
+	
+	if (pNZBFile)
 	{
-		NZBFile* pNZBFile = NZBFile::CreateFromBuffer(DownloadRequest.m_szFilename, DownloadRequest.m_szCategory, pRecvBuffer, ntohl(DownloadRequest.m_iTrailingDataLength));
-
-		if (pNZBFile)
+		info("Request: Queue collection %s", DownloadRequest.m_szFilename);
+		
+		for (NZBFile::FileInfos::iterator it = pNZBFile->GetFileInfos()->begin(); it != pNZBFile->GetFileInfos()->end(); it++)
 		{
-			info("Request: Queue collection %s", DownloadRequest.m_szFilename);
-			g_pQueueCoordinator->AddNZBFileToQueue(pNZBFile, ntohl(DownloadRequest.m_bAddFirst));
-			delete pNZBFile;
-
-			char tmp[1024];
-			snprintf(tmp, 1024, "Collection %s added to queue", Util::BaseFileName(DownloadRequest.m_szFilename));
-			tmp[1024-1] = '\0';
-			SendBoolResponse(true, tmp);
-		}
-		else
-		{
-			char tmp[1024];
-			snprintf(tmp, 1024, "Download Request failed for %s", Util::BaseFileName(DownloadRequest.m_szFilename));
-			tmp[1024-1] = '\0';
-			SendBoolResponse(false, tmp);
+			FileInfo* pFileInfo = *it;
+			pFileInfo->SetPriority(iPriority);
+			pFileInfo->SetPaused(bAddPaused);
 		}
+		
+		g_pQueueCoordinator->AddNZBFileToQueue(pNZBFile, ntohl(DownloadRequest.m_bAddFirst));
+		delete pNZBFile;
+		
+		char tmp[1024];
+		snprintf(tmp, 1024, "Collection %s added to queue", Util::BaseFileName(DownloadRequest.m_szFilename));
+		tmp[1024-1] = '\0';
+		SendBoolResponse(true, tmp);
+	}
+	else
+	{
+		char tmp[1024];
+		snprintf(tmp, 1024, "Download Request failed for %s", Util::BaseFileName(DownloadRequest.m_szFilename));
+		tmp[1024-1] = '\0';
+		SendBoolResponse(false, tmp);
 	}
 
 	free(pRecvBuffer);
@@ -369,12 +398,24 @@ void ListBinCommand::Execute()
 	ListResponse.m_MessageBase.m_iSignature = htonl(NZBMESSAGE_SIGNATURE);
 	ListResponse.m_MessageBase.m_iStructSize = htonl(sizeof(ListResponse));
 	ListResponse.m_iEntrySize = htonl(sizeof(SNZBListResponseFileEntry));
+	ListResponse.m_bRegExValid = 0;
 
 	char* buf = NULL;
 	int bufsize = 0;
 
 	if (ntohl(ListRequest.m_bFileList))
 	{
+		eRemoteMatchMode eMatchMode = (eRemoteMatchMode)ntohl(ListRequest.m_iMatchMode);
+		bool bMatchGroup = ntohl(ListRequest.m_bMatchGroup);
+		const char* szPattern = ListRequest.m_szPattern;
+
+		RegEx *pRegEx = NULL;
+		if (eMatchMode == eRemoteMatchModeRegEx)
+		{
+			pRegEx = new RegEx(szPattern);
+			ListResponse.m_bRegExValid = pRegEx->IsValid();
+		}
+
 		// Make a data structure and copy all the elements of the list into it
 		DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
 
@@ -386,6 +427,7 @@ void ListBinCommand::Execute()
 		{
 			NZBInfo* pNZBInfo = *it;
 			bufsize += strlen(pNZBInfo->GetFilename()) + 1;
+			bufsize += strlen(pNZBInfo->GetName()) + 1;
 			bufsize += strlen(pNZBInfo->GetDestDir()) + 1;
 			bufsize += strlen(pNZBInfo->GetCategory()) + 1;
 			bufsize += strlen(pNZBInfo->GetQueuedFilename()) + 1;
@@ -429,13 +471,17 @@ void ListBinCommand::Execute()
 			Util::SplitInt64(pNZBInfo->GetSize(), &iSizeHi, &iSizeLo);
 			pListAnswer->m_iSizeLo				= htonl(iSizeLo);
 			pListAnswer->m_iSizeHi				= htonl(iSizeHi);
+			pListAnswer->m_bMatch				= htonl(bMatchGroup && (!pRegEx || pRegEx->Match(pNZBInfo->GetName())));
 			pListAnswer->m_iFilenameLen			= htonl(strlen(pNZBInfo->GetFilename()) + 1);
+			pListAnswer->m_iNameLen				= htonl(strlen(pNZBInfo->GetName()) + 1);
 			pListAnswer->m_iDestDirLen			= htonl(strlen(pNZBInfo->GetDestDir()) + 1);
 			pListAnswer->m_iCategoryLen			= htonl(strlen(pNZBInfo->GetCategory()) + 1);
 			pListAnswer->m_iQueuedFilenameLen	= htonl(strlen(pNZBInfo->GetQueuedFilename()) + 1);
 			bufptr += sizeof(SNZBListResponseNZBEntry);
 			strcpy(bufptr, pNZBInfo->GetFilename());
 			bufptr += ntohl(pListAnswer->m_iFilenameLen);
+			strcpy(bufptr, pNZBInfo->GetName());
+			bufptr += ntohl(pListAnswer->m_iNameLen);
 			strcpy(bufptr, pNZBInfo->GetDestDir());
 			bufptr += ntohl(pListAnswer->m_iDestDirLen);
 			strcpy(bufptr, pNZBInfo->GetCategory());
@@ -484,7 +530,7 @@ void ListBinCommand::Execute()
 			unsigned long iSizeHi, iSizeLo;
 			FileInfo* pFileInfo = *it;
 			SNZBListResponseFileEntry* pListAnswer = (SNZBListResponseFileEntry*) bufptr;
-			pListAnswer->m_iID				= htonl(pFileInfo->GetID());
+			pListAnswer->m_iID = htonl(pFileInfo->GetID());
 
 			int iNZBIndex = 0;
 			for (unsigned int i = 0; i < pDownloadQueue->GetNZBInfoList()->size(); i++)
@@ -497,6 +543,13 @@ void ListBinCommand::Execute()
 			}
 			pListAnswer->m_iNZBIndex		= htonl(iNZBIndex);
 
+			if (pRegEx && !bMatchGroup)
+			{
+				char szFilename[MAX_PATH];
+				snprintf(szFilename, sizeof(szFilename) - 1, "%s/%s", pFileInfo->GetNZBInfo()->GetName(), Util::BaseFileName(pFileInfo->GetFilename()));
+				pListAnswer->m_bMatch = htonl(pRegEx->Match(szFilename));
+			}
+
 			Util::SplitInt64(pFileInfo->GetSize(), &iSizeHi, &iSizeLo);
 			pListAnswer->m_iFileSizeLo		= htonl(iSizeLo);
 			pListAnswer->m_iFileSizeHi		= htonl(iSizeHi);
@@ -505,6 +558,8 @@ void ListBinCommand::Execute()
 			pListAnswer->m_iRemainingSizeHi	= htonl(iSizeHi);
 			pListAnswer->m_bFilenameConfirmed = htonl(pFileInfo->GetFilenameConfirmed());
 			pListAnswer->m_bPaused			= htonl(pFileInfo->GetPaused());
+			pListAnswer->m_iActiveDownloads	= htonl(pFileInfo->GetActiveDownloads());
+			pListAnswer->m_iPriority		= htonl(pFileInfo->GetPriority());
 			pListAnswer->m_iSubjectLen		= htonl(strlen(pFileInfo->GetSubject()) + 1);
 			pListAnswer->m_iFilenameLen		= htonl(strlen(pFileInfo->GetFilename()) + 1);
 			bufptr += sizeof(SNZBListResponseFileEntry);
@@ -523,6 +578,11 @@ void ListBinCommand::Execute()
 
 		g_pQueueCoordinator->UnlockQueue();
 
+		if (pRegEx)
+		{
+			delete pRegEx;
+		}
+
 		ListResponse.m_iNrTrailingNZBEntries = htonl(iNrNZBEntries);
 		ListResponse.m_iNrTrailingPPPEntries = htonl(iNrPPPEntries);
 		ListResponse.m_iNrTrailingFileEntries = htonl(iNrFileEntries);
@@ -532,11 +592,11 @@ void ListBinCommand::Execute()
 	if (htonl(ListRequest.m_bServerState))
 	{
 		unsigned long iSizeHi, iSizeLo;
-		ListResponse.m_iDownloadRate = htonl((int)(g_pQueueCoordinator->CalcCurrentDownloadSpeed() * 1024));
+		ListResponse.m_iDownloadRate = htonl(g_pQueueCoordinator->CalcCurrentDownloadSpeed());
 		Util::SplitInt64(g_pQueueCoordinator->CalcRemainingSize(), &iSizeHi, &iSizeLo);
 		ListResponse.m_iRemainingSizeHi = htonl(iSizeHi);
 		ListResponse.m_iRemainingSizeLo = htonl(iSizeLo);
-		ListResponse.m_iDownloadLimit = htonl((int)(g_pOptions->GetDownloadRate() * 1024));
+		ListResponse.m_iDownloadLimit = htonl(g_pOptions->GetDownloadRate());
 		ListResponse.m_bDownloadPaused = htonl(g_pOptions->GetPauseDownload());
 		ListResponse.m_bDownload2Paused = htonl(g_pOptions->GetPauseDownload2());
 		ListResponse.m_bPostPaused = htonl(g_pOptions->GetPausePostProcess());
@@ -559,12 +619,12 @@ void ListBinCommand::Execute()
 	}
 
 	// Send the request answer
-	send(m_iSocket, (char*) &ListResponse, sizeof(ListResponse), 0);
+	m_pConnection->Send((char*) &ListResponse, sizeof(ListResponse));
 
 	// Send the data
 	if (bufsize > 0)
 	{
-		send(m_iSocket, buf, bufsize, 0);
+		m_pConnection->Send(buf, bufsize);
 	}
 
 	if (buf)
@@ -650,12 +710,12 @@ void LogBinCommand::Execute()
 	LogResponse.m_iTrailingDataLength = htonl(bufsize);
 
 	// Send the request answer
-	send(m_iSocket, (char*) &LogResponse, sizeof(LogResponse), 0);
+	m_pConnection->Send((char*) &LogResponse, sizeof(LogResponse));
 
 	// Send the data
 	if (bufsize > 0)
 	{
-		send(m_iSocket, buf, bufsize, 0);
+		m_pConnection->Send(buf, bufsize);
 	}
 
 	free(buf);
@@ -669,63 +729,71 @@ void EditQueueBinCommand::Execute()
 		return;
 	}
 
-	int iNrEntries = ntohl(EditQueueRequest.m_iNrTrailingEntries);
+	int iNrIDEntries = ntohl(EditQueueRequest.m_iNrTrailingIDEntries);
+	int iNrNameEntries = ntohl(EditQueueRequest.m_iNrTrailingNameEntries);
+	int iNameEntriesLen = ntohl(EditQueueRequest.m_iTrailingNameEntriesLen);
 	int iAction = ntohl(EditQueueRequest.m_iAction);
+	int iMatchMode = ntohl(EditQueueRequest.m_iMatchMode);
 	int iOffset = ntohl(EditQueueRequest.m_iOffset);
 	int iTextLen = ntohl(EditQueueRequest.m_iTextLen);
 	bool bSmartOrder = ntohl(EditQueueRequest.m_bSmartOrder);
 	unsigned int iBufLength = ntohl(EditQueueRequest.m_iTrailingDataLength);
 
-	if (iNrEntries * sizeof(int32_t) + iTextLen != iBufLength)
+	if (iNrIDEntries * sizeof(int32_t) + iTextLen + iNameEntriesLen != iBufLength)
 	{
 		error("Invalid struct size");
 		return;
 	}
 
-	if (iNrEntries <= 0)
+	char* pBuf = (char*)malloc(iBufLength);
+	
+	if (!m_pConnection->Recv(pBuf, iBufLength))
 	{
-		SendBoolResponse(false, "Edit-Command failed: no IDs specified");
+		error("invalid request");
+		free(pBuf);
 		return;
 	}
 
-	char* pBuf = (char*)malloc(iBufLength);
-	char* szText = NULL;
-	int32_t* pIDs = NULL;
-
-	// Read from the socket until nothing remains
-	char* pBufPtr = pBuf;
-	int NeedBytes = iBufLength;
-	int iResult = 0;
-	while (NeedBytes > 0)
+	if (iNrIDEntries <= 0 && iNrNameEntries <= 0)
 	{
-		iResult = recv(m_iSocket, pBufPtr, NeedBytes, 0);
-		// Did the recv succeed?
-		if (iResult <= 0)
-		{
-			error("invalid request");
-			break;
-		}
-		pBufPtr += iResult;
-		NeedBytes -= iResult;
+		SendBoolResponse(false, "Edit-Command failed: no IDs/Names specified");
+		return;
 	}
-	bool bOK = NeedBytes == 0;
 
-	if (bOK)
-	{
-		szText = iTextLen > 0 ? pBuf : NULL;
-		pIDs = (int32_t*)(pBuf + iTextLen);
-	}
+	char* szText = iTextLen > 0 ? pBuf : NULL;
+	int32_t* pIDs = (int32_t*)(pBuf + iTextLen);
+	char* pNames = (pBuf + iTextLen + iNrIDEntries * sizeof(int32_t));
 
 	IDList cIDList;
-	cIDList.reserve(iNrEntries);
-	for (int i = 0; i < iNrEntries; i++)
+	NameList cNameList;
+
+	if (iNrIDEntries > 0)
 	{
-		cIDList.push_back(ntohl(pIDs[i]));
+		cIDList.reserve(iNrIDEntries);
+		for (int i = 0; i < iNrIDEntries; i++)
+		{
+			cIDList.push_back(ntohl(pIDs[i]));
+		}
 	}
 
+	if (iNrNameEntries > 0)
+	{
+		cNameList.reserve(iNrNameEntries);
+		for (int i = 0; i < iNrNameEntries; i++)
+		{
+			cNameList.push_back(pNames);
+			pNames += strlen(pNames) + 1;
+		}
+	}
+
+	bool bOK = false;
+	
 	if (iAction < eRemoteEditActionPostMoveOffset)
 	{
-		bOK = g_pQueueCoordinator->GetQueueEditor()->EditList(&cIDList, bSmartOrder, (QueueEditor::EEditAction)iAction, iOffset, szText);
+		bOK = g_pQueueCoordinator->GetQueueEditor()->EditList(
+			iNrIDEntries > 0 ? &cIDList : NULL,
+			iNrNameEntries > 0 ? &cNameList : NULL,
+			(QueueEditor::EMatchMode)iMatchMode, bSmartOrder, (QueueEditor::EEditAction)iAction, iOffset, szText);
 	}
 	else
 	{
@@ -740,6 +808,13 @@ void EditQueueBinCommand::Execute()
 	}
 	else
 	{
+#ifndef HAVE_REGEX_H
+		if ((QueueEditor::EMatchMode)iMatchMode == QueueEditor::mmRegEx)
+		{
+			SendBoolResponse(false, "Edit-Command failed: the program was compiled without RegEx-support");
+			return;
+		}
+#endif
 		SendBoolResponse(false, "Edit-Command failed");
 	}
 }
@@ -825,12 +900,12 @@ void PostQueueBinCommand::Execute()
 	PostQueueResponse.m_iTrailingDataLength = htonl(bufsize);
 
 	// Send the request answer
-	send(m_iSocket, (char*) &PostQueueResponse, sizeof(PostQueueResponse), 0);
+	m_pConnection->Send((char*) &PostQueueResponse, sizeof(PostQueueResponse));
 
 	// Send the data
 	if (bufsize > 0)
 	{
-		send(m_iSocket, buf, bufsize, 0);
+		m_pConnection->Send(buf, bufsize);
 	}
 
 	free(buf);
@@ -845,50 +920,36 @@ void WriteLogBinCommand::Execute()
 	}
 
 	char* pRecvBuffer = (char*)malloc(ntohl(WriteLogRequest.m_iTrailingDataLength) + 1);
-	char* pBufPtr = pRecvBuffer;
-
-	// Read from the socket until nothing remains
-	int iResult = 0;
-	int NeedBytes = ntohl(WriteLogRequest.m_iTrailingDataLength);
-	pRecvBuffer[NeedBytes] = '\0';
-	while (NeedBytes > 0)
+	
+	if (!m_pConnection->Recv(pRecvBuffer, ntohl(WriteLogRequest.m_iTrailingDataLength)))
 	{
-		iResult = recv(m_iSocket, pBufPtr, NeedBytes, 0);
-		// Did the recv succeed?
-		if (iResult <= 0)
-		{
-			error("invalid request");
+		error("invalid request");
+		free(pRecvBuffer);
+		return;
+	}
+	
+	bool OK = true;
+	switch ((Message::EKind)ntohl(WriteLogRequest.m_iKind))
+	{
+		case Message::mkDetail:
+			detail(pRecvBuffer);
 			break;
-		}
-		pBufPtr += iResult;
-		NeedBytes -= iResult;
-	}
-
-	if (NeedBytes == 0)
-	{
-		bool OK = true;
-		switch ((Message::EKind)ntohl(WriteLogRequest.m_iKind))
-		{
-			case Message::mkDetail:
-				detail(pRecvBuffer);
-				break;
-			case Message::mkInfo:
-				info(pRecvBuffer);
-				break;
-			case Message::mkWarning:
-				warn(pRecvBuffer);
-				break;
-			case Message::mkError:
-				error(pRecvBuffer);
-				break;
-			case Message::mkDebug:
-				debug(pRecvBuffer);
-				break;
-			default:
-				OK = false;
-		}
-		SendBoolResponse(OK, OK ? "Message added to log" : "Invalid message-kind");
+		case Message::mkInfo:
+			info(pRecvBuffer);
+			break;
+		case Message::mkWarning:
+			warn(pRecvBuffer);
+			break;
+		case Message::mkError:
+			error(pRecvBuffer);
+			break;
+		case Message::mkDebug:
+			debug(pRecvBuffer);
+			break;
+		default:
+			OK = false;
 	}
+	SendBoolResponse(OK, OK ? "Message added to log" : "Invalid message-kind");
 
 	free(pRecvBuffer);
 }
@@ -901,8 +962,10 @@ void ScanBinCommand::Execute()
 		return;
 	}
 
-	g_pPrePostProcessor->ScanNZBDir();
-	SendBoolResponse(true, "Scan-Command scheduled successfully");
+	bool bSyncMode = ntohl(ScanRequest.m_bSyncMode);
+
+	g_pPrePostProcessor->ScanNZBDir(bSyncMode);
+	SendBoolResponse(true, bSyncMode ? "Scan-Command completed" : "Scan-Command scheduled successfully");
 }
 
 void HistoryBinCommand::Execute()
@@ -926,15 +989,14 @@ void HistoryBinCommand::Execute()
 	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
 
 	// calculate required buffer size for nzbs
-	int iNrNZBEntries = pDownloadQueue->GetHistoryList()->size();
-	bufsize += iNrNZBEntries * sizeof(SNZBHistoryResponseEntry);
+	int iNrEntries = pDownloadQueue->GetHistoryList()->size();
+	bufsize += iNrEntries * sizeof(SNZBHistoryResponseEntry);
 	for (HistoryList::iterator it = pDownloadQueue->GetHistoryList()->begin(); it != pDownloadQueue->GetHistoryList()->end(); it++)
 	{
-		NZBInfo* pNZBInfo = *it;
-		bufsize += strlen(pNZBInfo->GetFilename()) + 1;
-		bufsize += strlen(pNZBInfo->GetDestDir()) + 1;
-		bufsize += strlen(pNZBInfo->GetCategory()) + 1;
-		bufsize += strlen(pNZBInfo->GetQueuedFilename()) + 1;
+		HistoryInfo* pHistoryInfo = *it;
+		char szNicename[1024];
+		pHistoryInfo->GetName(szNicename, sizeof(szNicename));
+		bufsize += strlen(szNicename) + 1;
 		// align struct to 4-bytes, needed by ARM-processor (and may be others)
 		bufsize += bufsize % 4 > 0 ? 4 - bufsize % 4 : 0;
 	}
@@ -943,36 +1005,42 @@ void HistoryBinCommand::Execute()
 	char* bufptr = buf;
 
 	// write nzb entries
-	for (NZBInfoList::iterator it = pDownloadQueue->GetHistoryList()->begin(); it != pDownloadQueue->GetHistoryList()->end(); it++)
+	for (HistoryList::iterator it = pDownloadQueue->GetHistoryList()->begin(); it != pDownloadQueue->GetHistoryList()->end(); it++)
 	{
-		unsigned long iSizeHi, iSizeLo;
-		NZBInfo* pNZBInfo = *it;
+		HistoryInfo* pHistoryInfo = *it;
 		SNZBHistoryResponseEntry* pListAnswer = (SNZBHistoryResponseEntry*) bufptr;
-		Util::SplitInt64(pNZBInfo->GetSize(), &iSizeHi, &iSizeLo);
-		pListAnswer->m_iID					= htonl(pNZBInfo->GetID());
-		pListAnswer->m_tTime				= htonl((int)pNZBInfo->GetHistoryTime());
-		pListAnswer->m_iSizeLo				= htonl(iSizeLo);
-		pListAnswer->m_iSizeHi				= htonl(iSizeHi);
-		pListAnswer->m_iFileCount			= htonl(pNZBInfo->GetFileCount());
-		pListAnswer->m_iParStatus			= htonl(pNZBInfo->GetParStatus());
-		pListAnswer->m_iScriptStatus		= htonl(pNZBInfo->GetScriptStatus());
-		pListAnswer->m_iFilenameLen			= htonl(strlen(pNZBInfo->GetFilename()) + 1);
-		pListAnswer->m_iDestDirLen			= htonl(strlen(pNZBInfo->GetDestDir()) + 1);
-		pListAnswer->m_iCategoryLen			= htonl(strlen(pNZBInfo->GetCategory()) + 1);
-		pListAnswer->m_iQueuedFilenameLen	= htonl(strlen(pNZBInfo->GetQueuedFilename()) + 1);
+		pListAnswer->m_iID					= htonl(pHistoryInfo->GetID());
+		pListAnswer->m_iKind				= htonl((int)pHistoryInfo->GetKind());
+		pListAnswer->m_tTime				= htonl((int)pHistoryInfo->GetTime());
+
+		char szNicename[1024];
+		pHistoryInfo->GetName(szNicename, sizeof(szNicename));
+		pListAnswer->m_iNicenameLen			= htonl(strlen(szNicename) + 1);
+
+		if (pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo)
+		{
+			NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
+			unsigned long iSizeHi, iSizeLo;
+			Util::SplitInt64(pNZBInfo->GetSize(), &iSizeHi, &iSizeLo);
+			pListAnswer->m_iSizeLo				= htonl(iSizeLo);
+			pListAnswer->m_iSizeHi				= htonl(iSizeHi);
+			pListAnswer->m_iFileCount			= htonl(pNZBInfo->GetFileCount());
+			pListAnswer->m_iParStatus			= htonl(pNZBInfo->GetParStatus());
+			pListAnswer->m_iScriptStatus		= htonl(pNZBInfo->GetScriptStatus());
+		}
+		else if (pHistoryInfo->GetKind() == HistoryInfo::hkUrlInfo)
+		{
+			UrlInfo* pUrlInfo = pHistoryInfo->GetUrlInfo();
+			pListAnswer->m_iUrlStatus			= htonl(pUrlInfo->GetStatus());
+		}
+
 		bufptr += sizeof(SNZBHistoryResponseEntry);
-		strcpy(bufptr, pNZBInfo->GetFilename());
-		bufptr += ntohl(pListAnswer->m_iFilenameLen);
-		strcpy(bufptr, pNZBInfo->GetDestDir());
-		bufptr += ntohl(pListAnswer->m_iDestDirLen);
-		strcpy(bufptr, pNZBInfo->GetCategory());
-		bufptr += ntohl(pListAnswer->m_iCategoryLen);
-		strcpy(bufptr, pNZBInfo->GetQueuedFilename());
-		bufptr += ntohl(pListAnswer->m_iQueuedFilenameLen);
+		strcpy(bufptr, szNicename);
+		bufptr += ntohl(pListAnswer->m_iNicenameLen);
 		// align struct to 4-bytes, needed by ARM-processor (and may be others)
 		if ((size_t)bufptr % 4 > 0)
 		{
-			pListAnswer->m_iQueuedFilenameLen = htonl(ntohl(pListAnswer->m_iQueuedFilenameLen) + 4 - (size_t)bufptr % 4);
+			pListAnswer->m_iNicenameLen = htonl(ntohl(pListAnswer->m_iNicenameLen) + 4 - (size_t)bufptr % 4);
 			memset(bufptr, 0, 4 - (size_t)bufptr % 4); //suppress valgrind warning "uninitialized data"
 			bufptr += 4 - (size_t)bufptr % 4;
 		}
@@ -980,16 +1048,126 @@ void HistoryBinCommand::Execute()
 
 	g_pQueueCoordinator->UnlockQueue();
 
-	HistoryResponse.m_iNrTrailingEntries = htonl(iNrNZBEntries);
+	HistoryResponse.m_iNrTrailingEntries = htonl(iNrEntries);
 	HistoryResponse.m_iTrailingDataLength = htonl(bufsize);
 
 	// Send the request answer
-	send(m_iSocket, (char*) &HistoryResponse, sizeof(HistoryResponse), 0);
+	m_pConnection->Send((char*) &HistoryResponse, sizeof(HistoryResponse));
 
 	// Send the data
 	if (bufsize > 0)
 	{
-		send(m_iSocket, buf, bufsize, 0);
+		m_pConnection->Send(buf, bufsize);
+	}
+
+	free(buf);
+}
+
+void DownloadUrlBinCommand::Execute()
+{
+	SNZBDownloadUrlRequest DownloadUrlRequest;
+	if (!ReceiveRequest(&DownloadUrlRequest, sizeof(DownloadUrlRequest)))
+	{
+		return;
+	}
+
+	URL url(DownloadUrlRequest.m_szURL);
+	if (!url.IsValid())
+	{
+		char tmp[1024];
+		snprintf(tmp, 1024, "Url %s is not valid", DownloadUrlRequest.m_szURL);
+		tmp[1024-1] = '\0';
+		SendBoolResponse(true, tmp);
+		return;
+	}
+
+	UrlInfo* pUrlInfo = new UrlInfo();
+	pUrlInfo->SetURL(DownloadUrlRequest.m_szURL);
+	pUrlInfo->SetNZBFilename(DownloadUrlRequest.m_szNZBFilename);
+	pUrlInfo->SetCategory(DownloadUrlRequest.m_szCategory);
+	pUrlInfo->SetPriority(ntohl(DownloadUrlRequest.m_iPriority));
+	pUrlInfo->SetAddTop(ntohl(DownloadUrlRequest.m_bAddFirst));
+	pUrlInfo->SetAddPaused(ntohl(DownloadUrlRequest.m_bAddPaused));
+
+	g_pUrlCoordinator->AddUrlToQueue(pUrlInfo, ntohl(DownloadUrlRequest.m_bAddFirst));
+
+	info("Request: Queue url %s", DownloadUrlRequest.m_szURL);
+
+	char tmp[1024];
+	snprintf(tmp, 1024, "Url %s added to queue", DownloadUrlRequest.m_szURL);
+	tmp[1024-1] = '\0';
+	SendBoolResponse(true, tmp);
+}
+
+void UrlQueueBinCommand::Execute()
+{
+	SNZBUrlQueueRequest UrlQueueRequest;
+	if (!ReceiveRequest(&UrlQueueRequest, sizeof(UrlQueueRequest)))
+	{
+		return;
+	}
+
+	SNZBUrlQueueResponse UrlQueueResponse;
+	memset(&UrlQueueResponse, 0, sizeof(UrlQueueResponse));
+	UrlQueueResponse.m_MessageBase.m_iSignature = htonl(NZBMESSAGE_SIGNATURE);
+	UrlQueueResponse.m_MessageBase.m_iStructSize = htonl(sizeof(UrlQueueResponse));
+	UrlQueueResponse.m_iEntrySize = htonl(sizeof(SNZBUrlQueueResponseEntry));
+
+	char* buf = NULL;
+	int bufsize = 0;
+
+	// Make a data structure and copy all the elements of the list into it
+	UrlQueue* pUrlQueue = g_pQueueCoordinator->LockQueue()->GetUrlQueue();
+
+	int NrEntries = pUrlQueue->size();
+
+	// calculate required buffer size
+	bufsize = NrEntries * sizeof(SNZBUrlQueueResponseEntry);
+	for (UrlQueue::iterator it = pUrlQueue->begin(); it != pUrlQueue->end(); it++)
+	{
+		UrlInfo* pUrlInfo = *it;
+		bufsize += strlen(pUrlInfo->GetURL()) + 1;
+		bufsize += strlen(pUrlInfo->GetNZBFilename()) + 1;
+		// align struct to 4-bytes, needed by ARM-processor (and may be others)
+		bufsize += bufsize % 4 > 0 ? 4 - bufsize % 4 : 0;
+	}
+
+	buf = (char*) malloc(bufsize);
+	char* bufptr = buf;
+
+	for (UrlQueue::iterator it = pUrlQueue->begin(); it != pUrlQueue->end(); it++)
+	{
+		UrlInfo* pUrlInfo = *it;
+		SNZBUrlQueueResponseEntry* pUrlQueueAnswer = (SNZBUrlQueueResponseEntry*) bufptr;
+		pUrlQueueAnswer->m_iID				= htonl(pUrlInfo->GetID());
+		pUrlQueueAnswer->m_iURLLen			= htonl(strlen(pUrlInfo->GetURL()) + 1);
+		pUrlQueueAnswer->m_iNZBFilenameLen		= htonl(strlen(pUrlInfo->GetNZBFilename()) + 1);
+		bufptr += sizeof(SNZBUrlQueueResponseEntry);
+		strcpy(bufptr, pUrlInfo->GetURL());
+		bufptr += ntohl(pUrlQueueAnswer->m_iURLLen);
+		strcpy(bufptr, pUrlInfo->GetNZBFilename());
+		bufptr += ntohl(pUrlQueueAnswer->m_iNZBFilenameLen);
+		// align struct to 4-bytes, needed by ARM-processor (and may be others)
+		if ((size_t)bufptr % 4 > 0)
+		{
+			pUrlQueueAnswer->m_iNZBFilenameLen = htonl(ntohl(pUrlQueueAnswer->m_iNZBFilenameLen) + 4 - (size_t)bufptr % 4);
+			memset(bufptr, 0, 4 - (size_t)bufptr % 4); //suppress valgrind warning "uninitialized data"
+			bufptr += 4 - (size_t)bufptr % 4;
+		}
+	}
+
+	g_pQueueCoordinator->UnlockQueue();
+
+	UrlQueueResponse.m_iNrTrailingEntries = htonl(NrEntries);
+	UrlQueueResponse.m_iTrailingDataLength = htonl(bufsize);
+
+	// Send the request answer
+	m_pConnection->Send((char*) &UrlQueueResponse, sizeof(UrlQueueResponse));
+
+	// Send the data
+	if (bufsize > 0)
+	{
+		m_pConnection->Send(buf, bufsize);
 	}
 
 	free(buf);
