@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2009 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,16 +16,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 332 $
- * $Date: 2009-06-07 18:53:32 +0200 (Sun, 07 Jun 2009) $
+ * $Revision: 575 $
+ * $Date: 2013-03-04 23:01:14 +0100 (Mon, 04 Mar 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -54,19 +54,16 @@
 #include "nzbget.h"
 #include "Connection.h"
 #include "Log.h"
-#include "TLS.h"
 
 static const int CONNECTION_READBUFFER_SIZE = 1024;
-#ifndef DISABLE_TLS
-bool Connection::bTLSLibInitialized = false;
-#endif
 #ifndef HAVE_GETADDRINFO
 #ifndef HAVE_GETHOSTBYNAME_R
 Mutex* Connection::m_pMutexGetHostByName = NULL;
 #endif
 #endif
 
-void Connection::Init(bool bTLS)
+
+void Connection::Init()
 {
 	debug("Initializing global connection data");
 
@@ -87,21 +84,7 @@ void Connection::Init(bool bTLS)
 #endif
 
 #ifndef DISABLE_TLS
-	if (bTLS)
-	{
-		debug("Initializing TLS library");
-		char* szErrStr;
-		int iRes = tls_lib_init(&szErrStr);
-		bTLSLibInitialized = iRes == TLS_EOK;
-		if (!bTLSLibInitialized)
-		{
-			error("Could not initialize TLS library: %s", szErrStr ? szErrStr : "unknown error");
-			if (szErrStr)
-			{
-				free(szErrStr);
-			}
-		}
-	}
+	TLSSocket::Init();
 #endif
 
 #ifndef HAVE_GETADDRINFO
@@ -120,11 +103,7 @@ void Connection::Final()
 #endif
 
 #ifndef DISABLE_TLS
-	if (bTLSLibInitialized)
-	{
-		debug("Finalizing TLS library");
-		tls_lib_deinit();
-	}
+	TLSSocket::Final();
 #endif
 
 #ifndef HAVE_GETADDRINFO
@@ -134,38 +113,48 @@ void Connection::Final()
 #endif
 }
 
-Connection::Connection(NetAddress* pNetAddress)
+Connection::Connection(const char* szHost, int iPort, bool bTLS)
 {
 	debug("Creating Connection");
 
-	m_pNetAddress		= pNetAddress;
+	m_szHost			= NULL;
+	m_iPort				= iPort;
+	m_bTLS				= bTLS;
+	m_szCipher			= NULL;
 	m_eStatus			= csDisconnected;
 	m_iSocket			= INVALID_SOCKET;
 	m_iBufAvail			= 0;
 	m_iTimeout			= 60;
 	m_bSuppressErrors	= true;
 	m_szReadBuf			= (char*)malloc(CONNECTION_READBUFFER_SIZE + 1);
-	m_bAutoClose		= true;
 #ifndef DISABLE_TLS
-	m_pTLS				= NULL;
+	m_pTLSSocket		= NULL;
 	m_bTLSError			= false;
 #endif
+
+	if (szHost)
+	{
+		m_szHost = strdup(szHost);
+	}
 }
 
-Connection::Connection(SOCKET iSocket, bool bAutoClose)
+Connection::Connection(SOCKET iSocket, bool bTLS)
 {
 	debug("Creating Connection");
 
-	m_pNetAddress		= NULL;
+	m_szHost			= NULL;
+	m_iPort				= 0;
+	m_bTLS				= bTLS;
+	m_szCipher			= NULL;
 	m_eStatus			= csConnected;
 	m_iSocket			= iSocket;
 	m_iBufAvail			= 0;
 	m_iTimeout			= 60;
 	m_bSuppressErrors	= true;
 	m_szReadBuf			= (char*)malloc(CONNECTION_READBUFFER_SIZE + 1);
-	m_bAutoClose		= bAutoClose;
 #ifndef DISABLE_TLS
-	m_pTLS				= NULL;
+	m_pTLSSocket		= NULL;
+	m_bTLSError			= false;
 #endif
 }
 
@@ -173,32 +162,65 @@ Connection::~Connection()
 {
 	debug("Destroying Connection");
 
-	if (m_eStatus == csConnected && m_bAutoClose)
+	Disconnect();
+
+	if (m_szHost)
 	{
-		Disconnect();
+		free(m_szHost);
 	}
+	if (m_szCipher)
+	{
+		free(m_szCipher);
+	}
+
 	free(m_szReadBuf);
 #ifndef DISABLE_TLS
-	if (m_pTLS)
+	if (m_pTLSSocket)
 	{
-		free(m_pTLS);
+		delete m_pTLSSocket;
 	}
 #endif
 }
 
+void Connection::SetSuppressErrors(bool bSuppressErrors)
+{
+	m_bSuppressErrors = bSuppressErrors;
+#ifndef DISABLE_TLS
+	if (m_pTLSSocket)
+	{
+		m_pTLSSocket->SetSuppressErrors(bSuppressErrors);
+	}
+#endif
+}
+
+void Connection::SetCipher(const char* szCipher)
+{
+	if (m_szCipher)
+	{
+		free(m_szCipher);
+	}
+	m_szCipher = szCipher ? strdup(szCipher) : NULL;
+}
+
 bool Connection::Connect()
 {
 	debug("Connecting");
 
 	if (m_eStatus == csConnected)
+	{
 		return true;
+	}
 
 	bool bRes = DoConnect();
 
 	if (bRes)
+	{
 		m_eStatus = csConnected;
+	}
 	else
+	{
 		Connection::DoDisconnect();
+	}
 
 	return bRes;
 }
@@ -208,7 +230,9 @@ bool Connection::Disconnect()
 	debug("Disconnecting");
 
 	if (m_eStatus == csDisconnected)
+	{
 		return true;
+	}
 
 	bool bRes = DoDisconnect();
 
@@ -252,18 +276,27 @@ int Connection::WriteLine(const char* pBuffer)
 	return iRes;
 }
 
-int Connection::Send(const char* pBuffer, int iSize)
+bool Connection::Send(const char* pBuffer, int iSize)
 {
 	debug("Sending data");
 
 	if (m_eStatus != csConnected)
 	{
-		return -1;
+		return false;
 	}
 
-	int iRes = send(m_iSocket, pBuffer, iSize, 0);
+	int iBytesSent = 0;
+	while (iBytesSent < iSize)
+	{
+		int iRes = send(m_iSocket, pBuffer + iBytesSent, iSize-iBytesSent, 0);
+		if (iRes <= 0)
+		{
+			return false;
+		}
+		iBytesSent += iRes;
+	}
 
-	return iRes;
+	return true;
 }
 
 char* Connection::ReadLine(char* pBuffer, int iSize, int* pBytesRead)
@@ -278,21 +311,27 @@ char* Connection::ReadLine(char* pBuffer, int iSize, int* pBytesRead)
 	return res;
 }
 
-SOCKET Connection::Accept()
+Connection* Connection::Accept()
 {
 	debug("Accepting connection");
 
 	if (m_eStatus != csListening)
 	{
-		return INVALID_SOCKET;
+		return NULL;
 	}
 
 	SOCKET iRes = DoAccept();
+	if (iRes == INVALID_SOCKET)
+	{
+		return NULL;
+	}
+	
+	Connection* pCon = new Connection(iRes, m_bTLS);
 
-	return iRes;
+	return pCon;
 }
 
-int Connection::Recv(char* pBuffer, int iSize)
+int Connection::TryRecv(char* pBuffer, int iSize)
 {
 	debug("Receiving data");
 
@@ -308,7 +347,7 @@ int Connection::Recv(char* pBuffer, int iSize)
 	return iReceived;
 }
 
-bool Connection::RecvAll(char * pBuffer, int iSize)
+bool Connection::Recv(char * pBuffer, int iSize)
 {
 	debug("Receiving data (full buffer)");
 
@@ -357,17 +396,18 @@ bool Connection::DoConnect()
 	addr_hints.ai_family = AF_UNSPEC;    /* Allow IPv4 or IPv6 */
 	addr_hints.ai_socktype = SOCK_STREAM,
 
-	sprintf(iPortStr, "%d", m_pNetAddress->GetPort());
+	sprintf(iPortStr, "%d", m_iPort);
 
-	int res = getaddrinfo(m_pNetAddress->GetHost(), iPortStr, &addr_hints, &addr_list);
+	int res = getaddrinfo(m_szHost, iPortStr, &addr_hints, &addr_list);
 	if (res != 0)
 	{
-		ReportError("Could not resolve hostname %s", m_pNetAddress->GetHost(), true, 0);
+		ReportError("Could not resolve hostname %s", m_szHost, true, 0);
 		return false;
 	}
 
 	for (addr = addr_list; addr != NULL; addr = addr->ai_next)
 	{
+		bool bLastAddr = !addr->ai_next;
 		m_iSocket = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
 		if (m_iSocket != INVALID_SOCKET)
 		{
@@ -378,20 +418,33 @@ bool Connection::DoConnect()
 				break;
 			}
 			// Connection failed
+			if (bLastAddr)
+			{
+				ReportError("Connection to %s failed", m_szHost, true, 0);
+			}
 			closesocket(m_iSocket);
 			m_iSocket = INVALID_SOCKET;
 		}
+		else if (bLastAddr)
+		{
+			ReportError("Socket creation failed for %s", m_szHost, true, 0);
+		}
 	}
 
 	freeaddrinfo(addr_list);
 
+	if (m_iSocket == INVALID_SOCKET)
+	{
+		return false;
+	} 
+
 #else
-	
+
 	struct sockaddr_in	sSocketAddress;
 	memset(&sSocketAddress, 0, sizeof(sSocketAddress));
 	sSocketAddress.sin_family = AF_INET;
-	sSocketAddress.sin_port = htons(m_pNetAddress->GetPort());
-	sSocketAddress.sin_addr.s_addr = ResolveHostAddr(m_pNetAddress->GetHost());
+	sSocketAddress.sin_port = htons(m_iPort);
+	sSocketAddress.sin_addr.s_addr = ResolveHostAddr(m_szHost);
 	if (sSocketAddress.sin_addr.s_addr == (unsigned int)-1)
 	{
 		return false;
@@ -400,25 +453,20 @@ bool Connection::DoConnect()
 	m_iSocket = socket(PF_INET, SOCK_STREAM, 0);
 	if (m_iSocket == INVALID_SOCKET)
 	{
-		ReportError("Socket creation failed for %s", m_pNetAddress->GetHost(), true, 0);
+		ReportError("Socket creation failed for %s", m_szHost, true, 0);
 		return false;
 	}
 
 	int res = connect(m_iSocket , (struct sockaddr *) & sSocketAddress, sizeof(sSocketAddress));
 	if (res == -1)
 	{
-		// Connection failed
+		ReportError("Connection to %s failed", m_szHost, true, 0);
 		closesocket(m_iSocket);
 		m_iSocket = INVALID_SOCKET;
+		return false;
 	}
 #endif
 
-	if (m_iSocket == INVALID_SOCKET)
-	{
-		ReportError("Connection to %s failed", m_pNetAddress->GetHost(), true, 0);
-		return false;
-	} 
-
 #ifdef WIN32
 	int MSecVal = m_iTimeout * 1000;
 	int err = setsockopt(m_iSocket, SOL_SOCKET, SO_RCVTIMEO, (char*)&MSecVal, sizeof(MSecVal));
@@ -430,9 +478,16 @@ bool Connection::DoConnect()
 #endif
 	if (err != 0)
 	{
-		ReportError("setsockopt failed", NULL, true, 0);
+		ReportError("Socket initialization failed for %s", m_szHost, true, 0);
 	}
 
+#ifndef DISABLE_TLS
+	if (m_bTLS && !StartTLS(true, NULL, NULL))
+	{
+		return false;
+	}
+#endif
+
 	return true;
 }
 
@@ -442,14 +497,11 @@ bool Connection::DoDisconnect()
 
 	if (m_iSocket != INVALID_SOCKET)
 	{
+#ifndef DISABLE_TLS
+		CloseTLS();
+#endif
 		closesocket(m_iSocket);
 		m_iSocket = INVALID_SOCKET;
-#ifndef DISABLE_TLS
-		if (m_pTLS)
-		{
-			CloseTLS();
-		}
-#endif
 	}
 
 	m_eStatus = csDisconnected;
@@ -533,6 +585,15 @@ char* Connection::DoReadLine(char* pBuffer, int iSize, int* pBytesRead)
 	return pBuffer;
 }
 
+
+void Connection::ReadBuffer(char** pBuffer, int *iBufLen)
+{
+	*iBufLen = m_iBufAvail;
+	*pBuffer = m_szBufPtr;
+	m_iBufAvail = 0;
+ };
+
+
 int Connection::DoBind()
 {
 	debug("Do binding");
@@ -546,12 +607,12 @@ int Connection::DoBind()
 	addr_hints.ai_socktype = SOCK_STREAM,
 	addr_hints.ai_flags = AI_PASSIVE;    // For wildcard IP address
 
-	sprintf(iPortStr, "%d", m_pNetAddress->GetPort());
+	sprintf(iPortStr, "%d", m_iPort);
 
-	int res = getaddrinfo(m_pNetAddress->GetHost(), iPortStr, &addr_hints, &addr_list);
+	int res = getaddrinfo(m_szHost, iPortStr, &addr_hints, &addr_list);
 	if (res != 0)
 	{
-		error( "Could not resolve hostname %s", m_pNetAddress->GetHost() );
+		error("Could not resolve hostname %s", m_szHost);
 		return -1;
 	}
 
@@ -582,24 +643,24 @@ int Connection::DoBind()
 	struct sockaddr_in	sSocketAddress;
 	memset(&sSocketAddress, 0, sizeof(sSocketAddress));
 	sSocketAddress.sin_family = AF_INET;
-	if (!m_pNetAddress->GetHost() || strlen(m_pNetAddress->GetHost()) == 0)
+	if (!m_szHost || strlen(m_szHost) == 0)
 	{
 		sSocketAddress.sin_addr.s_addr = htonl(INADDR_ANY);
 	}
 	else
 	{
-		sSocketAddress.sin_addr.s_addr = ResolveHostAddr(m_pNetAddress->GetHost());
+		sSocketAddress.sin_addr.s_addr = ResolveHostAddr(m_szHost);
 		if (sSocketAddress.sin_addr.s_addr == (unsigned int)-1)
 		{
 			return -1;
 		}
 	}
-	sSocketAddress.sin_port = htons(m_pNetAddress->GetPort());
+	sSocketAddress.sin_port = htons(m_iPort);
 
 	m_iSocket = socket(PF_INET, SOCK_STREAM, 0);
 	if (m_iSocket == INVALID_SOCKET)
 	{
-		ReportError("Socket creation failed for %s", m_pNetAddress->GetHost(), true, 0);
+		ReportError("Socket creation failed for %s", m_szHost, true, 0);
 		return -1;
 	}
 	
@@ -617,13 +678,13 @@ int Connection::DoBind()
 
 	if (m_iSocket == INVALID_SOCKET)
 	{
-		ReportError("Binding socket failed for %s", m_pNetAddress->GetHost(), true, 0);
+		ReportError("Binding socket failed for %s", m_szHost, true, 0);
 		return -1;
 	}
 
-	if (listen(m_iSocket, 10) < 0)
+	if (listen(m_iSocket, 100) < 0)
 	{
-		ReportError("Listen on socket failed for %s", m_pNetAddress->GetHost(), true, 0);
+		ReportError("Listen on socket failed for %s", m_szHost, true, 0);
 		return -1;
 	}
 
@@ -632,7 +693,7 @@ int Connection::DoBind()
 
 SOCKET Connection::DoAccept()
 {
-	SOCKET iSocket = accept(GetSocket(), NULL, NULL);
+	SOCKET iSocket = accept(m_iSocket, NULL, NULL);
 
 	if (iSocket == INVALID_SOCKET && m_eStatus != csCancelled)
 	{
@@ -675,14 +736,9 @@ void Connection::ReportError(const char* szMsgPrefix, const char* szMsgArg, bool
 	{
 #ifdef WIN32
 		int ErrCode = WSAGetLastError();
-		if (m_bSuppressErrors)
-		{
-			debug("%s: ErrNo %i", szErrPrefix, ErrCode);
-		}
-		else
-		{
-			error("%s: ErrNo %i", szErrPrefix, ErrCode);
-		}
+		char szErrMsg[1024];
+		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, ErrCode, 0, szErrMsg, 1024, NULL);
+		szErrMsg[1024-1] = '\0';
 #else
 		const char *szErrMsg = NULL;
 		int ErrCode = herrno;
@@ -695,7 +751,7 @@ void Connection::ReportError(const char* szMsgPrefix, const char* szMsgArg, bool
 		{
 			szErrMsg = hstrerror(ErrCode);
 		}
-		
+#endif
 		if (m_bSuppressErrors)
 		{
 			debug("%s: ErrNo %i, %s", szErrPrefix, ErrCode, szErrMsg);
@@ -704,7 +760,6 @@ void Connection::ReportError(const char* szMsgPrefix, const char* szMsgArg, bool
 		{
 			error("%s: ErrNo %i, %s", szErrPrefix, ErrCode, szErrMsg);
 		}
-#endif
 	}
 	else
 	{
@@ -720,71 +775,40 @@ void Connection::ReportError(const char* szMsgPrefix, const char* szMsgArg, bool
 }
 
 #ifndef DISABLE_TLS
-bool Connection::CheckTLSResult(int iResultCode, char* szErrStr, const char* szErrMsgPrefix)
-{
-	bool bOK = iResultCode == TLS_EOK;
-	if (!bOK)
-	{
-		ReportError(szErrMsgPrefix, szErrStr ? szErrStr : "unknown error", false, 0);
-		if (szErrStr)
-		{
-			free(szErrStr);
-		}
-	}
-	return bOK;
-}
-
-bool Connection::StartTLS()
+bool Connection::StartTLS(bool bIsClient, const char* szCertFile, const char* szKeyFile)
 {
 	debug("Starting TLS");
 
-	if (m_pTLS)
+	if (m_pTLSSocket)
 	{
-		free(m_pTLS);
+		delete m_pTLSSocket;
 	}
 
-	m_pTLS = malloc(sizeof(tls_t));
-	tls_t* pTLS = (tls_t*)m_pTLS;
-	memset(pTLS, 0, sizeof(tls_t));
-	tls_clear(pTLS);
+	m_pTLSSocket = new TLSSocket(m_iSocket, bIsClient, szCertFile, szKeyFile, m_szCipher);
+	m_pTLSSocket->SetSuppressErrors(m_bSuppressErrors);
 
-	char* szErrStr;
-	int iRes;
-		
-	iRes = tls_init(pTLS, NULL, NULL, NULL, 0, &szErrStr);
-	if (!CheckTLSResult(iRes, szErrStr, "Could not initialize TLS-object: %s"))
-	{
-		return false;
-	}
-
-	debug("tls_start...");
-	iRes = tls_start(pTLS, (int)m_iSocket, NULL, 1, NULL, &szErrStr);
-	debug("tls_start...%i", iRes);
-	if (!CheckTLSResult(iRes, szErrStr, "Could not establish secure connection: %s"))
-	{
-		return false;
-	}
-
-	return true;
+	return m_pTLSSocket->Start();
 }
 
 void Connection::CloseTLS()
 {
-	tls_close((tls_t*)m_pTLS);
-	free(m_pTLS);
-	m_pTLS = NULL;
+	if (m_pTLSSocket)
+	{
+		m_pTLSSocket->Close();
+		delete m_pTLSSocket;
+		m_pTLSSocket = NULL;
+	}
 }
 
 int Connection::recv(SOCKET s, char* buf, int len, int flags)
 {
-	size_t iReceived = 0;
+	int iReceived = 0;
 	
-	if (m_pTLS)
+	if (m_pTLSSocket)
 	{
 		m_bTLSError = false;
-		char* szErrStr;
-		int iRes = tls_getbuf((tls_t*)m_pTLS, buf, len, &iReceived, &szErrStr);
-		if (!CheckTLSResult(iRes, szErrStr, "Could not read from TLS-socket: %s"))
+		iReceived = m_pTLSSocket->Recv(buf, len);
+		if (iReceived < 0)
 		{
 			m_bTLSError = true;
 			return -1;
@@ -799,22 +823,23 @@ int Connection::recv(SOCKET s, char* buf, int len, int flags)
 
 int Connection::send(SOCKET s, const char* buf, int len, int flags)
 {
-	if (m_pTLS)
+	int iSent = 0;
+
+	if (m_pTLSSocket)
 	{
 		m_bTLSError = false;
-		char* szErrStr;
-		int iRes = tls_putbuf((tls_t*)m_pTLS, buf, len, &szErrStr);
-		if (!CheckTLSResult(iRes, szErrStr, "Could not send to TLS-socket: %s"))
+		iSent = m_pTLSSocket->Send(buf, len);
+		if (iSent < 0)
 		{
 			m_bTLSError = true;
 			return -1;
 		}
-		return 0;
+		return iSent;
 	}
 	else
 	{
-		int iRet = ::send(s, buf, len, flags);
-		return iRet;
+		iSent = ::send(s, buf, len, flags);
+		return iSent;
 	}
 }
 #endif
@@ -868,3 +893,20 @@ unsigned int Connection::ResolveHostAddr(const char* szHost)
 	return uaddr;
 }
 #endif
+
+const char* Connection::GetRemoteAddr()
+{
+	struct sockaddr_in PeerName;
+	int iPeerNameLength = sizeof(PeerName);
+	if (getpeername(m_iSocket, (struct sockaddr*)&PeerName, (SOCKLEN_T*) &iPeerNameLength) >= 0)
+	{
+#ifdef WIN32
+		 strncpy(m_szRemoteAddr, inet_ntoa(PeerName.sin_addr), sizeof(m_szRemoteAddr));
+#else
+		inet_ntop(AF_INET, &PeerName.sin_addr, m_szRemoteAddr, sizeof(m_szRemoteAddr));
+#endif
+	}
+	m_szRemoteAddr[sizeof(m_szRemoteAddr)-1] = '\0';
+	
+	return m_szRemoteAddr;
+}
