@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2009 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,16 +15,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 353 $
- * $Date: 2009-09-26 17:21:11 +0200 (Sat, 26 Sep 2009) $
+ * $Revision: 576 $
+ * $Date: 2013-03-04 23:14:15 +0100 (Mon, 04 Mar 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -82,7 +82,7 @@ bool DiskState::SaveDownloadQueue(DownloadQueue* pDownloadQueue)
 		return false;
 	}
 
-	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 12);
+	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 21);
 
 	// save nzb-infos
 	SaveNZBList(pDownloadQueue, outfile);
@@ -93,6 +93,9 @@ bool DiskState::SaveDownloadQueue(DownloadQueue* pDownloadQueue)
 	// save post-queue
 	SavePostQueue(pDownloadQueue, outfile);
 
+	// save url-queue
+	SaveUrlQueue(pDownloadQueue, outfile);
+
 	// save history
 	SaveHistory(pDownloadQueue, outfile);
 
@@ -102,6 +105,7 @@ bool DiskState::SaveDownloadQueue(DownloadQueue* pDownloadQueue)
 	fclose(outfile);
 
 	if (pDownloadQueue->GetFileQueue()->empty() && 
+		pDownloadQueue->GetUrlQueue()->empty() &&
 		pDownloadQueue->GetPostQueue()->empty() &&
 		pDownloadQueue->GetHistoryList()->empty())
 	{
@@ -132,7 +136,7 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* pDownloadQueue)
 	char FileSignatur[128];
 	fgets(FileSignatur, sizeof(FileSignatur), infile);
 	int iFormatVersion = ParseFormatVersion(FileSignatur);
-	if (iFormatVersion < 3 || iFormatVersion > 12)
+	if (iFormatVersion < 3 || iFormatVersion > 21)
 	{
 		error("Could not load diskstate due to file version mismatch");
 		fclose(infile);
@@ -148,7 +152,7 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* pDownloadQueue)
 	if (iFormatVersion >= 7)
 	{
 		// load post-queue
-		if (!LoadPostQueue(pDownloadQueue, infile)) goto error;
+		if (!LoadPostQueue(pDownloadQueue, infile, iFormatVersion)) goto error;
 	}
 	else if (iFormatVersion < 7 && g_pOptions->GetReloadPostQueue())
 	{
@@ -156,10 +160,16 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* pDownloadQueue)
 		LoadOldPostQueue(pDownloadQueue);
 	}
 
+	if (iFormatVersion >= 15)
+	{
+		// load url-queue
+		if (!LoadUrlQueue(pDownloadQueue, infile, iFormatVersion)) goto error;
+	}
+
 	if (iFormatVersion >= 9)
 	{
 		// load history
-		if (!LoadHistory(pDownloadQueue, infile)) goto error;
+		if (!LoadHistory(pDownloadQueue, infile, iFormatVersion)) goto error;
 
 		// load parked file-infos
 		if (!LoadFileQueue(pDownloadQueue, pDownloadQueue->GetParkedFiles(), infile, iFormatVersion)) goto error;
@@ -191,10 +201,11 @@ void DiskState::SaveNZBList(DownloadQueue* pDownloadQueue, FILE* outfile)
 		fprintf(outfile, "%s\n", pNZBInfo->GetFilename());
 		fprintf(outfile, "%s\n", pNZBInfo->GetDestDir());
 		fprintf(outfile, "%s\n", pNZBInfo->GetQueuedFilename());
+		fprintf(outfile, "%s\n", pNZBInfo->GetName());
 		fprintf(outfile, "%s\n", pNZBInfo->GetCategory());
-		fprintf(outfile, "%i\n", pNZBInfo->GetPostProcess() ? 1 : 0);
-		fprintf(outfile, "%i\n", (int)pNZBInfo->GetParStatus());
-		fprintf(outfile, "%i\n", (int)pNZBInfo->GetScriptStatus());
+		fprintf(outfile, "%i\n", (int)pNZBInfo->GetPostProcess());
+		fprintf(outfile, "%i,%i,%i,%i,%i\n", (int)pNZBInfo->GetParStatus(), (int)pNZBInfo->GetUnpackStatus(), (int)pNZBInfo->GetScriptStatus(), (int)pNZBInfo->GetMoveStatus(), (int)pNZBInfo->GetRenameStatus());
+		fprintf(outfile, "%i\n", (int)pNZBInfo->GetUnpackCleanedUpDisk());
 		fprintf(outfile, "%i\n", pNZBInfo->GetFileCount());
 		fprintf(outfile, "%i\n", pNZBInfo->GetParkedFileCount());
 
@@ -269,6 +280,16 @@ bool DiskState::LoadNZBList(DownloadQueue* pDownloadQueue, FILE* infile, int iFo
 			pNZBInfo->SetQueuedFilename(buf);
 		}
 
+		if (iFormatVersion >= 13)
+		{
+			if (!fgets(buf, sizeof(buf), infile)) goto error;
+			if (buf[0] != 0) buf[strlen(buf)-1] = 0; // remove traling '\n'
+			if (strlen(buf) > 0)
+			{
+				pNZBInfo->SetName(buf);
+			}
+		}
+
 		if (iFormatVersion >= 4)
 		{
 			if (!fgets(buf, sizeof(buf), infile)) goto error;
@@ -277,23 +298,52 @@ bool DiskState::LoadNZBList(DownloadQueue* pDownloadQueue, FILE* infile, int iFo
 
 			int iPostProcess;
 			if (fscanf(infile, "%i\n", &iPostProcess) != 1) goto error;
-			pNZBInfo->SetPostProcess(iPostProcess == 1);
+			pNZBInfo->SetPostProcess((bool)iPostProcess);
 		}
 
-		if (iFormatVersion >= 8)
+		if (iFormatVersion >= 8 && iFormatVersion < 18)
 		{
 			int iParStatus;
 			if (fscanf(infile, "%i\n", &iParStatus) != 1) goto error;
 			pNZBInfo->SetParStatus((NZBInfo::EParStatus)iParStatus);
 		}
 
-		if (iFormatVersion >= 9)
+		if (iFormatVersion >= 9 && iFormatVersion < 18)
 		{
 			int iScriptStatus;
 			if (fscanf(infile, "%i\n", &iScriptStatus) != 1) goto error;
 			pNZBInfo->SetScriptStatus((NZBInfo::EScriptStatus)iScriptStatus);
 		}
 
+		if (iFormatVersion >= 18)
+		{
+			int iParStatus, iUnpackStatus, iScriptStatus, iMoveStatus = 0, iRenameStatus = 0;
+			if (iFormatVersion >= 21)
+			{
+				if (fscanf(infile, "%i,%i,%i,%i,%i\n", &iParStatus, &iUnpackStatus, &iScriptStatus, &iMoveStatus, &iRenameStatus) != 5) goto error;
+			}
+			else if (iFormatVersion >= 20)
+			{
+				if (fscanf(infile, "%i,%i,%i,%i\n", &iParStatus, &iUnpackStatus, &iScriptStatus, &iMoveStatus) != 4) goto error;
+			}
+			else
+			{
+				if (fscanf(infile, "%i,%i,%i\n", &iParStatus, &iUnpackStatus, &iScriptStatus) != 3) goto error;
+			}
+			pNZBInfo->SetParStatus((NZBInfo::EParStatus)iParStatus);
+			pNZBInfo->SetUnpackStatus((NZBInfo::EUnpackStatus)iUnpackStatus);
+			pNZBInfo->SetScriptStatus((NZBInfo::EScriptStatus)iScriptStatus);
+			pNZBInfo->SetMoveStatus((NZBInfo::EMoveStatus)iMoveStatus);
+			pNZBInfo->SetRenameStatus((NZBInfo::ERenameStatus)iRenameStatus);
+		}
+
+		if (iFormatVersion >= 19)
+		{
+			int iUnpackCleanedUpDisk;
+			if (fscanf(infile, "%i\n", &iUnpackCleanedUpDisk) != 1) goto error;
+			pNZBInfo->SetUnpackCleanedUpDisk((bool)iUnpackCleanedUpDisk);
+		}
+		
 		int iFileCount;
 		if (fscanf(infile, "%i\n", &iFileCount) != 1) goto error;
 		pNZBInfo->SetFileCount(iFileCount);
@@ -388,7 +438,8 @@ void DiskState::SaveFileQueue(DownloadQueue* pDownloadQueue, FileQueue* pFileQue
 		if (!pFileInfo->GetDeleted())
 		{
 			int iNZBIndex = FindNZBInfoIndex(pDownloadQueue, pFileInfo->GetNZBInfo());
-			fprintf(outfile, "%i,%i,%i,%i\n", pFileInfo->GetID(), iNZBIndex, (int)pFileInfo->GetPaused(), (int)pFileInfo->GetTime());
+			fprintf(outfile, "%i,%i,%i,%i,%i,%i\n", pFileInfo->GetID(), iNZBIndex, (int)pFileInfo->GetPaused(), 
+				(int)pFileInfo->GetTime(), pFileInfo->GetPriority(), (int)pFileInfo->GetExtraPriority());
 		}
 	}
 }
@@ -401,17 +452,26 @@ bool DiskState::LoadFileQueue(DownloadQueue* pDownloadQueue, FileQueue* pFileQue
 	if (fscanf(infile, "%i\n", &size) != 1) goto error;
 	for (int i = 0; i < size; i++)
 	{
-		unsigned int id, iNZBIndex, paused, iTime;
-		if (iFormatVersion >= 12)
+		unsigned int id, iNZBIndex, paused;
+		unsigned int iTime = 0;
+		int iPriority = 0, iExtraPriority = 0;
+		if (iFormatVersion >= 17)
+		{
+			if (fscanf(infile, "%i,%i,%i,%i,%i,%i\n", &id, &iNZBIndex, &paused, &iTime, &iPriority, &iExtraPriority) != 6) goto error;
+		}
+		else if (iFormatVersion >= 14)
+		{
+			if (fscanf(infile, "%i,%i,%i,%i,%i\n", &id, &iNZBIndex, &paused, &iTime, &iPriority) != 5) goto error;
+		}
+		else if (iFormatVersion >= 12)
 		{
 			if (fscanf(infile, "%i,%i,%i,%i\n", &id, &iNZBIndex, &paused, &iTime) != 4) goto error;
 		}
 		else
 		{
 			if (fscanf(infile, "%i,%i,%i\n", &id, &iNZBIndex, &paused) != 3) goto error;
-			iTime = 0;
 		}
-		if (iNZBIndex < 0 || iNZBIndex > pDownloadQueue->GetNZBInfoList()->size()) goto error;
+		if (iNZBIndex > pDownloadQueue->GetNZBInfoList()->size()) goto error;
 
 		char fileName[1024];
 		snprintf(fileName, 1024, "%s%i", g_pOptions->GetQueueDir(), id);
@@ -423,6 +483,8 @@ bool DiskState::LoadFileQueue(DownloadQueue* pDownloadQueue, FileQueue* pFileQue
 			pFileInfo->SetID(id);
 			pFileInfo->SetPaused(paused);
 			pFileInfo->SetTime(iTime);
+			pFileInfo->SetPriority(iPriority);
+			pFileInfo->SetExtraPriority(iExtraPriority != 0);
 			pFileInfo->SetNZBInfo(pDownloadQueue->GetNZBInfoList()->at(iNZBIndex - 1));
 			pFileQueue->push_back(pFileInfo);
 		}
@@ -570,14 +632,14 @@ void DiskState::SavePostQueue(DownloadQueue* pDownloadQueue, FILE* outfile)
 	{
 		PostInfo* pPostInfo = *it;
 		int iNZBIndex = FindNZBInfoIndex(pDownloadQueue, pPostInfo->GetNZBInfo());
-		fprintf(outfile, "%i,%i,%i,%i\n", iNZBIndex, (int)pPostInfo->GetParCheck(), 
-			(int)pPostInfo->GetParStatus(), (int)pPostInfo->GetStage());
+		fprintf(outfile, "%i,%i,%i,%i\n", iNZBIndex,
+			(int)pPostInfo->GetParStatus(), (int)pPostInfo->GetUnpackStatus(), (int)pPostInfo->GetStage());
 		fprintf(outfile, "%s\n", pPostInfo->GetInfoName());
 		fprintf(outfile, "%s\n", pPostInfo->GetParFilename());
 	}
 }
 
-bool DiskState::LoadPostQueue(DownloadQueue* pDownloadQueue, FILE* infile)
+bool DiskState::LoadPostQueue(DownloadQueue* pDownloadQueue, FILE* infile, int iFormatVersion)
 {
 	debug("Loading post-queue from disk");
 
@@ -590,15 +652,29 @@ bool DiskState::LoadPostQueue(DownloadQueue* pDownloadQueue, FILE* infile)
 	for (int i = 0; i < size; i++)
 	{
 		PostInfo* pPostInfo = NULL;
-		unsigned int iNZBIndex, iParCheck, iParStatus, iStage;
-		if (fscanf(infile, "%i,%i,%i,%i\n", &iNZBIndex, &iParCheck, &iParStatus, &iStage) != 4) goto error;
+		unsigned int iNZBIndex, iParCheck, iParStatus = 0, iUnpackStatus = 0, iStage;
+		if (iFormatVersion < 19)
+		{
+			if (fscanf(infile, "%i,%i,%i,%i\n", &iNZBIndex, &iParCheck, &iParStatus, &iStage) != 4) goto error;
+			if (!iParCheck)
+			{
+				iParStatus = PostInfo::psSkipped;
+			}
+		}
+		else
+		{
+			if (fscanf(infile, "%i,%i,%i,%i\n", &iNZBIndex, &iParStatus, &iUnpackStatus, &iStage) != 4) goto error;
+		}
+		if (iFormatVersion < 18 && iStage > (int)PostInfo::ptVerifyingRepaired) iStage++;
+		if (iFormatVersion < 21 && iStage > (int)PostInfo::ptVerifyingRepaired) iStage++;
+		if (iFormatVersion < 20 && iStage > (int)PostInfo::ptUnpacking) iStage++;
 
 		if (!bSkipPostQueue)
 		{
 			pPostInfo = new PostInfo();
 			pPostInfo->SetNZBInfo(pDownloadQueue->GetNZBInfoList()->at(iNZBIndex - 1));
-			pPostInfo->SetParCheck(iParCheck);
 			pPostInfo->SetParStatus((PostInfo::EParStatus)iParStatus);
+			pPostInfo->SetUnpackStatus((PostInfo::EUnpackStatus)iUnpackStatus);
 			pPostInfo->SetStage((PostInfo::EStage)iStage);
 		}
 
@@ -743,11 +819,11 @@ bool DiskState::LoadOldPostQueue(DownloadQueue* pDownloadQueue)
 			}
 		}
 
-		if (fscanf(infile, "%i\n", &iIntValue) != 1) goto error;
-		pPostInfo->SetParCheck(iIntValue);
+		int iParCheck;
+		if (fscanf(infile, "%i\n", &iParCheck) != 1) goto error; // ParCheck
 
 		if (fscanf(infile, "%i\n", &iIntValue) != 1) goto error;
-		pPostInfo->SetParStatus((PostInfo::EParStatus)iIntValue);
+		pPostInfo->SetParStatus(iParCheck ? (PostInfo::EParStatus)iIntValue : PostInfo::psSkipped);
 
 		if (iFormatVersion < 7)
 		{
@@ -792,6 +868,94 @@ error:
 	return false;
 }
 
+void DiskState::SaveUrlQueue(DownloadQueue* pDownloadQueue, FILE* outfile)
+{
+	debug("Saving url-queue to disk");
+
+	fprintf(outfile, "%i\n", pDownloadQueue->GetUrlQueue()->size());
+	for (UrlQueue::iterator it = pDownloadQueue->GetUrlQueue()->begin(); it != pDownloadQueue->GetUrlQueue()->end(); it++)
+	{
+		UrlInfo* pUrlInfo = *it;
+		SaveUrlInfo(pUrlInfo, outfile);
+	}
+}
+
+bool DiskState::LoadUrlQueue(DownloadQueue* pDownloadQueue, FILE* infile, int iFormatVersion)
+{
+	debug("Loading url-queue from disk");
+
+	bool bSkipUrlQueue = !g_pOptions->GetReloadUrlQueue();
+	int size;
+
+	// load url-infos
+	if (fscanf(infile, "%i\n", &size) != 1) goto error;
+	for (int i = 0; i < size; i++)
+	{
+		UrlInfo* pUrlInfo = NULL;
+		if (!bSkipUrlQueue)
+		{
+			pUrlInfo = new UrlInfo();
+		}
+
+		if (!LoadUrlInfo(pUrlInfo, infile, iFormatVersion)) goto error;
+
+		if (!bSkipUrlQueue)
+		{
+			pDownloadQueue->GetUrlQueue()->push_back(pUrlInfo);
+		}
+	}
+
+	return true;
+
+error:
+	error("Error reading diskstate for url-queue");
+	return false;
+}
+
+void DiskState::SaveUrlInfo(UrlInfo* pUrlInfo, FILE* outfile)
+{
+	fprintf(outfile, "%i,%i\n", (int)pUrlInfo->GetStatus(), pUrlInfo->GetPriority());
+	fprintf(outfile, "%i,%i\n", (int)pUrlInfo->GetAddTop(), pUrlInfo->GetAddPaused());
+	fprintf(outfile, "%s\n", pUrlInfo->GetURL());
+	fprintf(outfile, "%s\n", pUrlInfo->GetNZBFilename());
+	fprintf(outfile, "%s\n", pUrlInfo->GetCategory());
+}
+
+bool DiskState::LoadUrlInfo(UrlInfo* pUrlInfo, FILE* infile, int iFormatVersion)
+{
+	char buf[10240];
+
+	int iStatus, iPriority;
+	if (fscanf(infile, "%i,%i\n", &iStatus, &iPriority) != 2) goto error;
+	if (pUrlInfo) pUrlInfo->SetStatus((UrlInfo::EStatus)iStatus);
+	if (pUrlInfo) pUrlInfo->SetPriority(iPriority);
+
+	if (iFormatVersion >= 16)
+	{
+		int iAddTop, iAddPaused;
+		if (fscanf(infile, "%i,%i\n", &iAddTop, &iAddPaused) != 2) goto error;
+		if (pUrlInfo) pUrlInfo->SetAddTop(iAddTop);
+		if (pUrlInfo) pUrlInfo->SetAddPaused(iAddPaused);
+	}
+
+	if (!fgets(buf, sizeof(buf), infile)) goto error;
+	if (buf[0] != 0) buf[strlen(buf)-1] = 0; // remove traling '\n'
+	if (pUrlInfo) pUrlInfo->SetURL(buf);
+
+	if (!fgets(buf, sizeof(buf), infile)) goto error;
+	if (buf[0] != 0) buf[strlen(buf)-1] = 0; // remove traling '\n'
+	if (pUrlInfo) pUrlInfo->SetNZBFilename(buf);
+
+	if (!fgets(buf, sizeof(buf), infile)) goto error;
+	if (buf[0] != 0) buf[strlen(buf)-1] = 0; // remove traling '\n'
+	if (pUrlInfo) pUrlInfo->SetCategory(buf);
+
+	return true;
+
+error:
+	return false;
+}
+
 void DiskState::SaveHistory(DownloadQueue* pDownloadQueue, FILE* outfile)
 {
 	debug("Saving history to disk");
@@ -799,14 +963,25 @@ void DiskState::SaveHistory(DownloadQueue* pDownloadQueue, FILE* outfile)
 	fprintf(outfile, "%i\n", pDownloadQueue->GetHistoryList()->size());
 	for (HistoryList::iterator it = pDownloadQueue->GetHistoryList()->begin(); it != pDownloadQueue->GetHistoryList()->end(); it++)
 	{
-		NZBInfo* pNZBInfo = *it;
-		int iNZBIndex = FindNZBInfoIndex(pDownloadQueue, pNZBInfo);
-		fprintf(outfile, "%i\n", iNZBIndex);
-		fprintf(outfile, "%i\n", (int)pNZBInfo->GetHistoryTime());
+		HistoryInfo* pHistoryInfo = *it;
+
+		fprintf(outfile, "%i\n", (int)pHistoryInfo->GetKind());
+
+		if (pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo)
+		{
+			int iNZBIndex = FindNZBInfoIndex(pDownloadQueue, pHistoryInfo->GetNZBInfo());
+			fprintf(outfile, "%i\n", iNZBIndex);
+		}
+		else if (pHistoryInfo->GetKind() == HistoryInfo::hkUrlInfo)
+		{
+			SaveUrlInfo(pHistoryInfo->GetUrlInfo(), outfile);
+		}
+
+		fprintf(outfile, "%i\n", (int)pHistoryInfo->GetTime());
 	}
 }
 
-bool DiskState::LoadHistory(DownloadQueue* pDownloadQueue, FILE* infile)
+bool DiskState::LoadHistory(DownloadQueue* pDownloadQueue, FILE* infile, int iFormatVersion)
 {
 	debug("Loading history from disk");
 
@@ -814,23 +989,41 @@ bool DiskState::LoadHistory(DownloadQueue* pDownloadQueue, FILE* infile)
 	if (fscanf(infile, "%i\n", &size) != 1) goto error;
 	for (int i = 0; i < size; i++)
 	{
-		unsigned int iNZBIndex;
-		if (fscanf(infile, "%i\n", &iNZBIndex) != 1) goto error;
+		HistoryInfo* pHistoryInfo = NULL;
+		HistoryInfo::EKind eKind = HistoryInfo::hkNZBInfo;
 
-		NZBInfo* pNZBInfo = pDownloadQueue->GetNZBInfoList()->at(iNZBIndex - 1);
+		if (iFormatVersion >= 15)
+		{
+			int iKind = 0;
+			if (fscanf(infile, "%i\n", &iKind) != 1) goto error;
+			eKind = (HistoryInfo::EKind)iKind;
+		}
+
+		if (eKind == HistoryInfo::hkNZBInfo)
+		{
+			unsigned int iNZBIndex;
+			if (fscanf(infile, "%i\n", &iNZBIndex) != 1) goto error;
+			NZBInfo* pNZBInfo = pDownloadQueue->GetNZBInfoList()->at(iNZBIndex - 1);
+			pHistoryInfo = new HistoryInfo(pNZBInfo);
+		}
+		else if (eKind == HistoryInfo::hkUrlInfo)
+		{
+			UrlInfo* pUrlInfo = new UrlInfo();
+			if (!LoadUrlInfo(pUrlInfo, infile, iFormatVersion)) goto error;
+			pHistoryInfo = new HistoryInfo(pUrlInfo);
+		}
 
 		int iTime;
 		if (fscanf(infile, "%i\n", &iTime) != 1) goto error;
-		pNZBInfo->SetHistoryTime((time_t)iTime);
+		pHistoryInfo->SetTime((time_t)iTime);
 
-		pNZBInfo->AddReference();
-		pDownloadQueue->GetHistoryList()->push_back(pNZBInfo);
+		pDownloadQueue->GetHistoryList()->push_back(pHistoryInfo);
 	}
 
 	return true;
 
 error:
-	error("Error reading diskstate for post-processor queue");
+	error("Error reading diskstate for history");
 	return false;
 }
 
@@ -873,7 +1066,7 @@ bool DiskState::DiscardDownloadQueue()
 	char FileSignatur[128];
 	fgets(FileSignatur, sizeof(FileSignatur), infile);
 	int iFormatVersion = ParseFormatVersion(FileSignatur);
-	if (3 <= iFormatVersion && iFormatVersion <= 12)
+	if (3 <= iFormatVersion && iFormatVersion <= 21)
 	{
 		// skip nzb-infos
 		int size = 0;
@@ -887,19 +1080,31 @@ bool DiskState::DiscardDownloadQueue()
 			{
 				if (!fgets(buf, sizeof(buf), infile)) break; // localfile
 			}
+			if (iFormatVersion >= 13)
+			{
+				if (!fgets(buf, sizeof(buf), infile)) break; // name
+			}
 			if (iFormatVersion >= 4)
 			{
 				if (!fgets(buf, sizeof(buf), infile)) break; // category
 				if (!fgets(buf, sizeof(buf), infile)) break; // postprocess
 			}
-			if (iFormatVersion >= 8)
+			if (iFormatVersion >= 8 && iFormatVersion < 18)
 			{
 				if (!fgets(buf, sizeof(buf), infile)) break; // ParStatus
 			}
-			if (iFormatVersion >= 9)
+			if (iFormatVersion >= 9 && iFormatVersion < 18)
 			{
 				if (!fgets(buf, sizeof(buf), infile)) break; // ScriptStatus
 			}
+			if (iFormatVersion >= 18)
+			{
+				if (!fgets(buf, sizeof(buf), infile)) break; // ParStatus, UnpackStatus, ScriptStatus
+			}
+			if (iFormatVersion >= 19)
+			{
+				if (!fgets(buf, sizeof(buf), infile)) break; // UnpackCleanedUpDisk
+			}
 			if (!fgets(buf, sizeof(buf), infile)) break; // file count
 			if (iFormatVersion >= 10)
 			{
