@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2011 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 507 $
+ * $Date: 2012-11-12 22:11:42 +0100 (Mon, 12 Nov 2012) $
  *
  */
 
@@ -27,7 +27,7 @@
 #ifndef MESSAGEBASE_H
 #define MESSAGEBASE_H
 
-static const int32_t NZBMESSAGE_SIGNATURE = 0x6E7A6208; // = "nzb8" (protocol version)
+static const int32_t NZBMESSAGE_SIGNATURE = 0x6E7A6212; // = "nzb-XX" (protocol version)
 static const int NZBREQUESTFILENAMESIZE = 512;
 static const int NZBREQUESTPASSWORDSIZE = 32;
 
@@ -56,11 +56,14 @@ enum eRemoteRequest
 	eRemoteRequestEditQueue,
 	eRemoteRequestLog,
 	eRemoteRequestShutdown,
+	eRemoteRequestReload,
 	eRemoteRequestVersion,
 	eRemoteRequestPostQueue,
 	eRemoteRequestWriteLog,
 	eRemoteRequestScan,
-	eRemoteRequestHistory
+	eRemoteRequestHistory,
+	eRemoteRequestDownloadUrl,
+	eRemoteRequestUrlQueue
 };
 
 // Possible values for field "m_iAction" of struct "SNZBEditQueueRequest":
@@ -76,6 +79,8 @@ enum eRemoteEditAction
 	eRemoteEditActionFileDelete,			// delete files
 	eRemoteEditActionFilePauseAllPars,		// pause only (all) pars (does not affect other files)
 	eRemoteEditActionFilePauseExtraPars,	// pause only (almost all) pars, except main par-file (does not affect other files)
+	eRemoteEditActionFileSetPriority,		// set priority for files
+	eRemoteEditActionFileReorder,			// (not supported)
 	eRemoteEditActionGroupMoveOffset,		// move group to m_iOffset relative to the current position in download-queue
 	eRemoteEditActionGroupMoveTop,			// move group to the top of download-queue
 	eRemoteEditActionGroupMoveBottom,		// move group to the bottom of download-queue
@@ -84,9 +89,11 @@ enum eRemoteEditAction
 	eRemoteEditActionGroupDelete,			// delete group
 	eRemoteEditActionGroupPauseAllPars,		// pause only (all) pars (does not affect other files) in group
 	eRemoteEditActionGroupPauseExtraPars,	// pause only (almost all) pars in group, except main par-file (does not affect other files)
+	eRemoteEditActionGroupSetPriority,		// set priority for groups
 	eRemoteEditActionGroupSetCategory,		// set or change category for a group
 	eRemoteEditActionGroupMerge,			// merge group
 	eRemoteEditActionGroupSetParameter,		// set post-process parameter for group
+	eRemoteEditActionGroupSetName,			// set group name (rename group)
 	eRemoteEditActionPostMoveOffset = 51,	// move post-job to m_iOffset relative to the current position in post-queue
 	eRemoteEditActionPostMoveTop,			// move post-job to the top of post-queue
 	eRemoteEditActionPostMoveBottom,		// move post-job to the bottom of post-queue
@@ -105,6 +112,14 @@ enum eRemotePauseUnpauseAction
 	eRemotePauseUnpauseActionScan			// pause/unpause scan of incoming nzb-directory
 };
 
+// Possible values for field "m_iMatchMode" of struct "SNZBEditQueueRequest":
+enum eRemoteMatchMode
+{
+	eRemoteMatchModeID = 1,				// ID
+	eRemoteMatchModeName,				// Name
+	eRemoteMatchModeRegEx,				// RegEx
+};
+
 // The basic SNZBRequestBase struct, used in all requests
 struct SNZBRequestBase
 {
@@ -126,8 +141,10 @@ struct SNZBDownloadRequest
 {
 	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
 	char					m_szFilename[NZBREQUESTFILENAMESIZE];	// Name of nzb-file, may contain full path (local path on client) or only filename
-	char					m_szCategory[NZBREQUESTFILENAMESIZE];	// Category, be empty
+	char					m_szCategory[NZBREQUESTFILENAMESIZE];	// Category, can be empty
 	int32_t					m_bAddFirst;			// 1 - add file to the top of download queue
+	int32_t					m_bAddPaused;			// 1 - pause added files
+	int32_t					m_iPriority;			// Priority for files (0 - default)
 	int32_t					m_iTrailingDataLength;	// Length of nzb-file in bytes
 	//char					m_szContent[m_iTrailingDataLength];	// variable sized
 };
@@ -147,6 +164,9 @@ struct SNZBListRequest
 	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
 	int32_t					m_bFileList;			// 1 - return file list
 	int32_t					m_bServerState;			// 1 - return server state
+	int32_t					m_iMatchMode;			// File/Group match mode, see enum eRemoteMatchMode (only values eRemoteMatchModeID (no filter) and eRemoteMatchModeRegEx are allowed)
+	int32_t					m_bMatchGroup;			// 0 - match files; 1 - match nzbs (when m_iMatchMode == eRemoteMatchModeRegEx)
+	char					m_szPattern[NZBREQUESTFILENAMESIZE];	// RegEx Pattern (when m_iMatchMode == eRemoteMatchModeRegEx)
 };
 
 // A list response
@@ -169,6 +189,7 @@ struct SNZBListResponse
 	int32_t					m_iDownloadTimeSec;		// Server download time in seconds (up_time - standby_time)
 	int32_t					m_iDownloadedBytesLo;	// Amount of data downloaded since server start, Low 32-bits of 64-bit value
 	int32_t					m_iDownloadedBytesHi;	// Amount of data downloaded since server start, High 32-bits of 64-bit value
+	int32_t					m_bRegExValid;			// 0 - error in RegEx-pattern, 1 - RegEx-pattern is valid (only when Request has eRemoteMatchModeRegEx)
 	int32_t					m_iNrTrailingNZBEntries;	// Number of List-NZB-entries, following to this structure
 	int32_t					m_iNrTrailingPPPEntries;	// Number of List-PPP-entries, following to this structure
 	int32_t					m_iNrTrailingFileEntries;	// Number of List-File-entries, following to this structure
@@ -183,11 +204,14 @@ struct SNZBListResponseNZBEntry
 {
 	int32_t					m_iSizeLo;				// Size of all files in bytes, Low 32-bits of 64-bit value
 	int32_t					m_iSizeHi;				// Size of all files in bytes, High 32-bits of 64-bit value
+	int32_t					m_bMatch;				// 1 - group matches the pattern (only when Request has eRemoteMatchModeRegEx)
 	int32_t					m_iFilenameLen;			// Length of Filename-string (m_szFilename), following to this record
+	int32_t					m_iNameLen;				// Length of Name-string (m_szName), following to this record
 	int32_t					m_iDestDirLen;			// Length of DestDir-string (m_szDestDir), following to this record
 	int32_t					m_iCategoryLen;			// Length of Category-string (m_szCategory), following to this record
 	int32_t					m_iQueuedFilenameLen;	// Length of queued file name (m_szQueuedFilename), following to this record
 	//char					m_szFilename[m_iFilenameLen];				// variable sized
+	//char					m_szName[m_iNameLen];						// variable sized
 	//char					m_szDestDir[m_iDestDirLen];					// variable sized
 	//char					m_szCategory[m_iCategoryLen];				// variable sized
 	//char					m_szQueuedFilename[m_iQueuedFilenameLen];	// variable sized
@@ -214,6 +238,9 @@ struct SNZBListResponseFileEntry
 	int32_t					m_iRemainingSizeHi;		// Remaining size in bytes, High 32-bits of 64-bit value
 	int32_t					m_bPaused;				// 1 - file is paused
 	int32_t					m_bFilenameConfirmed;	// 1 - Filename confirmed (read from article body), 0 - Filename parsed from subject (can be changed after reading of article)
+	int32_t					m_iPriority;			// Download priority
+	int32_t					m_iActiveDownloads;		// Number of active downloads for this file
+	int32_t					m_bMatch;				// 1 - file matches the pattern (only when Request has eRemoteMatchModeRegEx)
 	int32_t					m_iSubjectLen;			// Length of Subject-string (m_szSubject), following to this record
 	int32_t					m_iFilenameLen;			// Length of Filename-string (m_szFilename), following to this record
 	//char					m_szSubject[m_iSubjectLen];			// variable sized
@@ -281,7 +308,7 @@ struct SNZBSetDownloadRateResponse
 	//char					m_szText[m_iTrailingDataLength];	// variable sized
 };
 
-// An edit queue request
+// edit queue request
 struct SNZBEditQueueRequest
 {
 	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
@@ -289,11 +316,15 @@ struct SNZBEditQueueRequest
 	int32_t					m_iOffset;				// Offset to move (for m_iAction = 0)
 	int32_t					m_bSmartOrder;			// For Move-Actions: 0 - execute action for each ID in order they are placed in array;
 													// 1 - smart execute to ensure that the relative order of all affected IDs are not changed.
-	int32_t					m_iNrTrailingEntries;	// Number of ID-entries, following to this structure
-	int32_t					m_iTextLen;				// Length of Text-string (m_szText), following to this record
-	int32_t					m_iTrailingDataLength;	// Length of Text-string and all ID-entries, following to this structure
-	//char					m_szText[m_iTextLen];	// variable sized
-	//int32_t				m_iIDs[m_iNrTrailingEntries];	// variable sized array of IDs. For File-Actions - ID of file, for Group-Actions - ID of any file belonging to group
+	int32_t					m_iMatchMode;				// File/Group match mode, see enum eRemoteMatchMode
+	int32_t					m_iNrTrailingIDEntries;		// Number of ID-entries, following to this structure
+	int32_t					m_iNrTrailingNameEntries;	// Number of Name-entries, following to this structure
+	int32_t					m_iTrailingNameEntriesLen;	// Length of all Name-entries, following to this structure
+	int32_t					m_iTextLen;					// Length of Text-string (m_szText), following to this record
+	int32_t					m_iTrailingDataLength;		// Length of Text-string and all ID-entries, following to this structure
+	//char					m_szText[m_iTextLen];		// variable sized
+	//int32_t				m_iIDs[m_iNrTrailingIDEntries];			// variable sized array of IDs. For File-Actions - ID of file, for Group-Actions - ID of any file belonging to group
+	//char*					m_szNames[m_iNrTrailingNameEntries];	// variable sized array of strings. For File-Actions - name of file incl. nzb-name as path, for Group-Actions - name of group
 };
 
 // An edit queue response
@@ -335,6 +366,21 @@ struct SNZBShutdownResponse
 	//char					m_szText[m_iTrailingDataLength];	// variable sized
 };
 
+// Reload server request
+struct SNZBReloadRequest
+{
+	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
+};
+
+// Reload server response
+struct SNZBReloadResponse
+{
+	SNZBResponseBase		m_MessageBase;			// Must be the first in the struct
+	int32_t					m_bSuccess;				// 0 - command failed, 1 - command executed successfully
+	int32_t					m_iTrailingDataLength;	// Length of Text-string (m_szText), following to this record
+	//char					m_szText[m_iTrailingDataLength];	// variable sized
+};
+
 // Server version request
 struct SNZBVersionRequest
 {
@@ -409,6 +455,7 @@ struct SNZBWriteLogResponse
 struct SNZBScanRequest
 {
 	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
+	int32_t					m_bSyncMode;			// 0 - asynchronous Scan (the command returns immediately), 1 - synchronous Scan (the command returns when the scan is completed)
 };
 
 // Scan nzb directory response
@@ -426,34 +473,80 @@ struct SNZBHistoryRequest
 	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
 };
 
-// A history response
+// history response
 struct SNZBHistoryResponse
 {
 	SNZBResponseBase		m_MessageBase;			// Must be the first in the struct
 	int32_t					m_iEntrySize;			// Size of the SNZBHistoryResponseEntry-struct
 	int32_t					m_iNrTrailingEntries;	// Number of History-entries, following to this structure
 	int32_t					m_iTrailingDataLength;	// Length of all History-entries, following to this structure
-	// SNZBHistoryResponseEntry m_NZBEntries[m_iNrTrailingNZBEntries]			// variable sized
+	// SNZBHistoryResponseEntry m_Entries[m_iNrTrailingEntries]			// variable sized
 };
 
-// A list response nzb entry
+// history entry
 struct SNZBHistoryResponseEntry
 {
-	int32_t					m_iID;					// NZBID
+	int32_t					m_iID;					// History-ID
+	int32_t					m_iKind;				// Kind of Item: 1 - Collection (NZB), 2 - URL
 	int32_t					m_tTime;				// When the item was added to history. time since the Epoch (00:00:00 UTC, January 1, 1970), measured in seconds.
+	int32_t					m_iNicenameLen;			// Length of Nicename-string (m_szNicename), following to this record
+	// for Collection items (m_iKind = 1)
 	int32_t					m_iSizeLo;				// Size of all files in bytes, Low 32-bits of 64-bit value
 	int32_t					m_iSizeHi;				// Size of all files in bytes, High 32-bits of 64-bit value
 	int32_t					m_iFileCount;			// Initial number of files included in NZB-file
 	int32_t					m_iParStatus;			// See NZBInfo::EParStatus
 	int32_t					m_iScriptStatus;		// See NZBInfo::EScriptStatus
-	int32_t					m_iFilenameLen;			// Length of Filename-string (m_szFilename), following to this record
-	int32_t					m_iDestDirLen;			// Length of DestDir-string (m_szDestDir), following to this record
-	int32_t					m_iCategoryLen;			// Length of Category-string (m_szCategory), following to this record
-	int32_t					m_iQueuedFilenameLen;	// Length of queued file name (m_szQueuedFilename), following to this record
-	//char					m_szFilename[m_iFilenameLen];				// variable sized
-	//char					m_szDestDir[m_iDestDirLen];					// variable sized
-	//char					m_szCategory[m_iCategoryLen];				// variable sized
-	//char					m_szQueuedFilename[m_iQueuedFilenameLen];	// variable sized
+	// for URL items (m_iKind = 2)
+	int32_t					m_iUrlStatus;			// See UrlInfo::EStatus
+	// trailing data
+	//char					m_szNicename[m_iNicenameLen];				// variable sized
+};
+
+// download url request
+struct SNZBDownloadUrlRequest
+{
+	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
+	char					m_szURL[NZBREQUESTFILENAMESIZE];		// url to nzb-file
+	char					m_szNZBFilename[NZBREQUESTFILENAMESIZE];// Name of nzb-file. Can be empty, then the filename is read from URL download response
+	char					m_szCategory[NZBREQUESTFILENAMESIZE];	// Category, can be empty
+	int32_t					m_bAddFirst;							// 1 - add url to the top of download queue
+	int32_t					m_bAddPaused;							// 1 - pause added files
+	int32_t					m_iPriority;							// Priority for files (0 - default)
+};
+
+// download url response
+struct SNZBDownloadUrlResponse
+{
+	SNZBResponseBase		m_MessageBase;			// Must be the first in the struct
+	int32_t					m_bSuccess;				// 0 - command failed, 1 - command executed successfully
+	int32_t					m_iTrailingDataLength;	// Length of Text-string (m_szText), following to this record
+	//char					m_szText[m_iTrailingDataLength];	// variable sized
+};
+
+// UrlQueue request
+struct SNZBUrlQueueRequest
+{
+	SNZBRequestBase			m_MessageBase;			// Must be the first in the struct
+};
+
+// UrlQueue response
+struct SNZBUrlQueueResponse
+{
+	SNZBResponseBase		m_MessageBase;			// Must be the first in the struct
+	int32_t					m_iEntrySize;			// Size of the SNZBUrlQueueResponseEntry-struct
+	int32_t					m_iNrTrailingEntries;	// Number of UrlQueue-entries, following to this structure
+	int32_t					m_iTrailingDataLength;	// Length of all UrlQueue-entries, following to this structure
+	// SNZBUrlQueueResponseEntry m_Entries[m_iNrTrailingEntries]		// variable sized
+};
+
+// UrlQueue response entry
+struct SNZBUrlQueueResponseEntry
+{
+	int32_t					m_iID;					// ID of Url-entry
+	int32_t					m_iURLLen;				// Length of URL-string (m_szURL), following to this record
+	int32_t					m_iNZBFilenameLen;		// Length of NZBFilename-string (m_szNZBFilename), following to this record
+	//char					m_szURL[m_iURLLen];					// variable sized
+	//char					m_szNZBFilename[m_iNZBFilenameLen];	// variable sized
 };
 
 #endif
