@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2011 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,16 +16,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 544 $
+ * $Date: 2013-01-18 22:36:17 +0100 (Fri, 18 Jan 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -613,10 +613,10 @@ void NCursesFrontend::PrintStatus()
     char timeString[100];
     timeString[0] = '\0';
 
-	float fCurrentDownloadSpeed = m_bStandBy ? 0 : m_fCurrentDownloadSpeed;
-    if (fCurrentDownloadSpeed > 0.0 && !(m_bPauseDownload || m_bPauseDownload2))
+	int iCurrentDownloadSpeed = m_bStandBy ? 0 : m_iCurrentDownloadSpeed;
+    if (iCurrentDownloadSpeed > 0 && !(m_bPauseDownload || m_bPauseDownload2))
     {
-        long long remain_sec = (long long)(m_lRemainingSize / (fCurrentDownloadSpeed * 1024));
+        long long remain_sec = (long long)(m_lRemainingSize / iCurrentDownloadSpeed);
 		int h = (int)(remain_sec / 3600);
 		int m = (int)((remain_sec % 3600) / 60);
 		int s = (int)(remain_sec % 60);
@@ -624,9 +624,9 @@ void NCursesFrontend::PrintStatus()
     }
 
     char szDownloadLimit[128];
-    if (m_fDownloadLimit > 0.0f)
+    if (m_iDownloadLimit > 0)
     {
-        sprintf(szDownloadLimit, ", Limit %.0f KB/s", m_fDownloadLimit);
+        sprintf(szDownloadLimit, ", Limit %.0f KB/s", (float)m_iDownloadLimit / 1024.0);
     }
     else
     {
@@ -646,7 +646,7 @@ void NCursesFrontend::PrintStatus()
 	float fAverageSpeed = (float)(Util::Int64ToFloat(m_iDnTimeSec > 0 ? m_iAllBytes / m_iDnTimeSec : 0) / 1024.0);
 
 	snprintf(tmp, MAX_SCREEN_WIDTH, " %d threads, %.*f KB/s, %.2f MB remaining%s%s%s%s%s, Avg. %.*f KB/s", 
-		m_iThreadCount, (fCurrentDownloadSpeed >= 10 ? 0 : 1), fCurrentDownloadSpeed, 
+		m_iThreadCount, (iCurrentDownloadSpeed >= 10*1024 ? 0 : 1), (float)iCurrentDownloadSpeed / 1024.0, 
 		(float)(Util::Int64ToFloat(m_lRemainingSize) / 1024.0 / 1024.0), timeString, szPostStatus, 
 		m_bPauseDownload || m_bPauseDownload2 ? (m_bStandBy ? ", Paused" : ", Pausing") : "",
 		m_bPauseDownload || m_bPauseDownload2 ? 
@@ -819,6 +819,19 @@ void NCursesFrontend::PrintFilename(FileInfo * pFileInfo, int iRow, bool bSelect
 		color = NCURSES_COLORPAIR_TEXT;
 	}
 
+	const char* szDownloading = "";
+	if (pFileInfo->GetActiveDownloads() > 0)
+	{
+		szDownloading = " *";
+	}
+
+	char szPriority[100];
+	szPriority[0] = '\0';
+	if (pFileInfo->GetPriority() != 0)
+	{
+		sprintf(szPriority, " [%+i]", pFileInfo->GetPriority());
+	}
+
 	char szCompleted[20];
 	szCompleted[0] = '\0';
 	if (pFileInfo->GetRemainingSize() < pFileInfo->GetSize())
@@ -829,7 +842,7 @@ void NCursesFrontend::PrintFilename(FileInfo * pFileInfo, int iRow, bool bSelect
 	char szNZBNiceName[1024];
 	if (m_bShowNZBname)
 	{
-		pFileInfo->GetNZBInfo()->GetNiceNZBName(szNZBNiceName, 1023);
+		strncpy(szNZBNiceName, pFileInfo->GetNZBInfo()->GetName(), 1023);
 		int len = strlen(szNZBNiceName);
 		szNZBNiceName[len] = PATH_SEPARATOR;
 		szNZBNiceName[len + 1] = '\0';
@@ -840,8 +853,9 @@ void NCursesFrontend::PrintFilename(FileInfo * pFileInfo, int iRow, bool bSelect
 	}
 
 	char szBuffer[MAX_SCREEN_WIDTH];
-	snprintf(szBuffer, MAX_SCREEN_WIDTH, "%s%i%s %s%s (%.2f MB%s)%s", Brace1, pFileInfo->GetID(),
-		Brace2, szNZBNiceName, pFileInfo->GetFilename(), (float)(Util::Int64ToFloat(pFileInfo->GetSize()) / 1024.0 / 1024.0),
+	snprintf(szBuffer, MAX_SCREEN_WIDTH, "%s%i%s%s%s %s%s (%.2f MB%s)%s", Brace1, pFileInfo->GetID(),
+		Brace2, szPriority, szDownloading, szNZBNiceName, pFileInfo->GetFilename(), 
+		(float)(Util::Int64ToFloat(pFileInfo->GetSize()) / 1024.0 / 1024.0),
 		szCompleted, pFileInfo->GetPaused() ? " (paused)" : "");
 	szBuffer[MAX_SCREEN_WIDTH - 1] = '\0';
 
@@ -912,6 +926,7 @@ void NCursesFrontend::PrintGroupQueue()
 {
 	int iLineNr = m_iQueueWinTop;
 
+    LockQueue();
 	GroupQueue* pGroupQueue = &m_groupQueue;
 	if (pGroupQueue->empty())
     {
@@ -966,6 +981,7 @@ void NCursesFrontend::PrintGroupQueue()
 		szBuffer[MAX_SCREEN_WIDTH - 1] = '\0';
 		PrintTopHeader(szBuffer, m_iQueueWinTop, false);
     }
+    UnlockQueue();
 }
 
 void NCursesFrontend::ResetColWidths()
@@ -977,21 +993,23 @@ void NCursesFrontend::ResetColWidths()
 
 void NCursesFrontend::PrintGroupname(GroupInfo * pGroupInfo, int iRow, bool bSelected, bool bCalcColWidth)
 {
-	int color = 0;
-	const char* Brace1 = "[";
-	const char* Brace2 = "]";
+	int color = NCURSES_COLORPAIR_TEXT;
+	char chBrace1 = '[';
+	char chBrace2 = ']';
 	if (m_eInputMode == eEditQueue && bSelected)
 	{
 		color = NCURSES_COLORPAIR_TEXTHIGHL;
 		if (!m_bUseColor)
 		{
-			Brace1 = "<";
-			Brace2 = ">";
+			chBrace1 = '<';
+			chBrace2 = '>';
 		}
 	}
-	else
+
+	const char* szDownloading = "";
+	if (pGroupInfo->GetActiveDownloads() > 0)
 	{
-		color = NCURSES_COLORPAIR_TEXT;
+		szDownloading = " *";
 	}
 
 	long long lUnpausedRemainingSize = pGroupInfo->GetRemainingSize() - pGroupInfo->GetPausedSize();
@@ -999,8 +1017,19 @@ void NCursesFrontend::PrintGroupname(GroupInfo * pGroupInfo, int iRow, bool bSel
 	char szRemaining[20];
 	Util::FormatFileSize(szRemaining, sizeof(szRemaining), lUnpausedRemainingSize);
 
-	char szNZBNiceName[1024];
-	pGroupInfo->GetNZBInfo()->GetNiceNZBName(szNZBNiceName, 1023);
+	char szPriority[100];
+	szPriority[0] = '\0';
+	if (pGroupInfo->GetMinPriority() != 0 || pGroupInfo->GetMaxPriority() != 0)
+	{
+		if (pGroupInfo->GetMinPriority() == pGroupInfo->GetMaxPriority())
+		{
+			sprintf(szPriority, " [%+i]", pGroupInfo->GetMinPriority());
+		}
+		else
+		{
+			sprintf(szPriority, " [%+i..%+i]", pGroupInfo->GetMinPriority(), pGroupInfo->GetMaxPriority());
+		}
+	}
 
 	char szBuffer[MAX_SCREEN_WIDTH];
 
@@ -1030,20 +1059,21 @@ void NCursesFrontend::PrintGroupname(GroupInfo * pGroupInfo, int iRow, bool bSel
 		Util::FormatFileSize(szTotal, sizeof(szTotal), pGroupInfo->GetNZBInfo()->GetSize());
 
 		char szNameWithIds[1024];
-		snprintf(szNameWithIds, 1024, "%s%i-%i%s %s", Brace1, pGroupInfo->GetFirstID(), pGroupInfo->GetLastID(), Brace2, szNZBNiceName);
+		snprintf(szNameWithIds, 1024, "%c%i-%i%c%s%s %s", chBrace1, pGroupInfo->GetFirstID(), pGroupInfo->GetLastID(), chBrace2, 
+			szPriority, szDownloading, pGroupInfo->GetNZBInfo()->GetName());
 		szNameWithIds[iNameLen] = '\0';
 
 		char szTime[100];
 		szTime[0] = '\0';
-		float fCurrentDownloadSpeed = m_bStandBy ? 0 : m_fCurrentDownloadSpeed;
+		int iCurrentDownloadSpeed = m_bStandBy ? 0 : m_iCurrentDownloadSpeed;
 		if (pGroupInfo->GetPausedSize() > 0 && lUnpausedRemainingSize == 0)
 		{
 			snprintf(szTime, 100, "[paused]");
 			Util::FormatFileSize(szRemaining, sizeof(szRemaining), pGroupInfo->GetRemainingSize());
 		}
-		else if (fCurrentDownloadSpeed > 0.0 && !(m_bPauseDownload || m_bPauseDownload2))
+		else if (iCurrentDownloadSpeed > 0 && !(m_bPauseDownload || m_bPauseDownload2))
 		{
-			long long remain_sec = (long long)(lUnpausedRemainingSize / (fCurrentDownloadSpeed * 1024));
+			long long remain_sec = (long long)(lUnpausedRemainingSize / iCurrentDownloadSpeed);
 			int h = (int)(remain_sec / 3600);
 			int m = (int)((remain_sec % 3600) / 60);
 			int s = (int)(remain_sec % 60);
@@ -1075,7 +1105,8 @@ void NCursesFrontend::PrintGroupname(GroupInfo * pGroupInfo, int iRow, bool bSel
 	}
 	else
 	{
-		snprintf(szBuffer, MAX_SCREEN_WIDTH, "%s%i-%i%s %s", Brace1, pGroupInfo->GetFirstID(), pGroupInfo->GetLastID(), Brace2, szNZBNiceName);
+		snprintf(szBuffer, MAX_SCREEN_WIDTH, "%c%i-%i%c%s %s", chBrace1, pGroupInfo->GetFirstID(), 
+			pGroupInfo->GetLastID(), chBrace2, szDownloading, pGroupInfo->GetNZBInfo()->GetName());
 	}
 
 	szBuffer[MAX_SCREEN_WIDTH - 1] = '\0';
@@ -1417,7 +1448,7 @@ void NCursesFrontend::UpdateInput(int initialKey)
 			// Enter
 			else if (iKey == 10 || iKey == 13)
 			{
-				ServerSetDownloadRate((float)m_iInputValue);
+				ServerSetDownloadRate(m_iInputValue * 1024);
 				m_eInputMode = eNormal;
 				return;
 			}
