@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,16 +16,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 599 $
+ * $Date: 2013-03-14 23:30:59 +0100 (Thu, 14 Mar 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -127,7 +127,7 @@ void QueueCoordinator::Run()
 	{
 		if (!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
 		{
-			NNTPConnection* pConnection = g_pServerPool->GetConnection(0);
+			NNTPConnection* pConnection = g_pServerPool->GetConnection(0, NULL, NULL);
 			if (pConnection)
 			{
 				// start download for next article
@@ -168,9 +168,12 @@ void QueueCoordinator::Run()
 		int iSleepInterval = bStandBy ? 100 : 5;
 		usleep(iSleepInterval * 1000);
 
-		AddSpeedReading(0);
+		if (!bStandBy)
+		{
+			AddSpeedReading(0);
+		}
 
-		iResetCounter+= iSleepInterval;
+		iResetCounter += iSleepInterval;
 		if (iResetCounter >= 1000)
 		{
 			// this code should not be called too often, once per second is OK
@@ -294,75 +297,99 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 /*
  * NOTE: see note to "AddSpeedReading"
  */
-float QueueCoordinator::CalcCurrentDownloadSpeed()
+int QueueCoordinator::CalcCurrentDownloadSpeed()
 {
 	if (m_bStandBy)
 	{
 		return 0;
 	}
 
-    int iTimeDiff = (int)time(NULL) - m_iSpeedStartTime * SPEEDMETER_SLOTSIZE;
-    if (iTimeDiff == 0)
-    {
-    	return 0;
-    }
-	float fSpeed = m_iSpeedTotalBytes / 1024.0f / iTimeDiff;
-	return fSpeed;
+	int iTimeDiff = (int)time(NULL) - m_iSpeedStartTime * SPEEDMETER_SLOTSIZE;
+	if (iTimeDiff == 0)
+	{
+		return 0;
+	}
+
+	return m_iSpeedTotalBytes / iTimeDiff;
 }
 
-/*
- * NOTE: we should use mutex by access to m_iSpeedBytes and m_iSpeedBytesIndex,
- * but this would results in a big performance loss (the function 
- * "AddSpeedReading" is called extremly often), so we better agree with calculation 
- * errors possible because of simultaneuos access from several threads.
- * The used algorithm is able to recover after few seconds.
- * In any case the calculation errors can not result in fatal system 
- * errors (segmentation faults).
- */
 void QueueCoordinator::AddSpeedReading(int iBytes)
 {
-    int iNowSlot = (int)time(NULL) / SPEEDMETER_SLOTSIZE;
+	time_t tCurTime = time(NULL);
+	int iNowSlot = (int)tCurTime / SPEEDMETER_SLOTSIZE;
 
-    if (iNowSlot > m_iSpeedTime[m_iSpeedBytesIndex])
-    {
-        //record bytes in next slot
-        m_iSpeedBytesIndex++;
-        if (m_iSpeedBytesIndex >= SPEEDMETER_SLOTS)
-        {
-            m_iSpeedBytesIndex = 0;
-        }
-        //Adjust counters with outging information.
-        m_iSpeedTotalBytes -= m_iSpeedBytes[m_iSpeedBytesIndex];
+	if (g_pOptions->GetAccurateRate())
+	{
+#ifdef HAVE_SPINLOCK
+		m_spinlockSpeed.Lock();
+#else
+		m_mutexSpeed.Lock();
+#endif
+	}
 
-        //Note we should really use the start time of the next slot
-        //but its easier to just use the outgoing slot time. This 
-        //will result in a small error.
-        m_iSpeedStartTime = m_iSpeedTime[m_iSpeedBytesIndex];
+	while (iNowSlot > m_iSpeedTime[m_iSpeedBytesIndex])
+	{
+		//record bytes in next slot
+		m_iSpeedBytesIndex++;
+		if (m_iSpeedBytesIndex >= SPEEDMETER_SLOTS)
+		{
+			m_iSpeedBytesIndex = 0;
+		}
+		//Adjust counters with outgoing information.
+		m_iSpeedTotalBytes -= m_iSpeedBytes[m_iSpeedBytesIndex];
 
-        //Now reset.
-        m_iSpeedBytes[m_iSpeedBytesIndex] = 0;
-        m_iSpeedTime[m_iSpeedBytesIndex] = iNowSlot;
-    } 
+		//Note we should really use the start time of the next slot
+		//but its easier to just use the outgoing slot time. This
+		//will result in a small error.
+		m_iSpeedStartTime = m_iSpeedTime[m_iSpeedBytesIndex];
 
-    if (m_iSpeedTotalBytes == 0)
-    {
-        m_iSpeedStartTime = iNowSlot;
-    }
-    m_iSpeedBytes[m_iSpeedBytesIndex] += iBytes;
-    m_iSpeedTotalBytes += iBytes;
+		//Now reset.
+		m_iSpeedBytes[m_iSpeedBytesIndex] = 0;
+		m_iSpeedTime[m_iSpeedBytesIndex] = iNowSlot;
+	}
+
+	// Once per second recalculate summary field "m_iSpeedTotalBytes" to recover from possible synchronisation errors
+	if (tCurTime > m_tSpeedCorrection)
+	{
+		int iSpeedTotalBytes = 0;
+		for (int i = 0; i < SPEEDMETER_SLOTS; i++)
+		{
+			iSpeedTotalBytes += m_iSpeedBytes[i];
+		}
+		m_iSpeedTotalBytes = iSpeedTotalBytes;
+		m_tSpeedCorrection = tCurTime;
+	}
+
+	if (m_iSpeedTotalBytes == 0)
+	{
+		m_iSpeedStartTime = iNowSlot;
+	}
+	m_iSpeedBytes[m_iSpeedBytesIndex] += iBytes;
+	m_iSpeedTotalBytes += iBytes;
 	m_iAllBytes += iBytes;
+
+	if (g_pOptions->GetAccurateRate())
+	{
+#ifdef HAVE_SPINLOCK
+		m_spinlockSpeed.Unlock();
+#else
+		m_mutexSpeed.Unlock();
+#endif
+	}
 }
 
 void QueueCoordinator::ResetSpeedStat()
 {
-    m_iSpeedStartTime = (int)time(NULL) / SPEEDMETER_SLOTSIZE;
+	time_t tCurTime = time(NULL);
+	m_iSpeedStartTime = (int)tCurTime / SPEEDMETER_SLOTSIZE;
 	for (int i = 0; i < SPEEDMETER_SLOTS; i++)
 	{
 		m_iSpeedBytes[i] = 0;
-        m_iSpeedTime[i] = m_iSpeedStartTime;
+		m_iSpeedTime[i] = m_iSpeedStartTime;
 	}
 	m_iSpeedBytesIndex = 0;
-    m_iSpeedTotalBytes = 0;
+	m_iSpeedTotalBytes = 0;
+	m_tSpeedCorrection = tCurTime;
 }
 
 long long QueueCoordinator::CalcRemainingSize()
@@ -425,31 +452,90 @@ void QueueCoordinator::Stop()
 	debug("ArticleDownloads are notified");
 }
 
+/*
+ * Returns next article for download.
+ */
 bool QueueCoordinator::GetNextArticle(FileInfo* &pFileInfo, ArticleInfo* &pArticleInfo)
 {
+	// find an unpaused file with the highest priority, then take the next article from the file.
+	// if the file doesn't have any articles left for download, we store that fact and search again,
+	// ignoring all files which were previously marked as not having any articles.
+
+	// special case: if the file has ExtraPriority-flag set, it has the highest priority and the
+	// Paused-flag is ignored.
+
 	//debug("QueueCoordinator::GetNextArticle()");
 
-	for (FileQueue::iterator it = m_DownloadQueue.GetFileQueue()->begin(); it != m_DownloadQueue.GetFileQueue()->end(); it++)
+	bool bOK = false;
+
+	// pCheckedFiles stores
+	bool* pCheckedFiles = NULL;
+
+	while (!bOK) 
 	{
-		pFileInfo = *it;
-		if (!pFileInfo->GetPaused() && !pFileInfo->GetDeleted())
+		//debug("QueueCoordinator::GetNextArticle() - in loop");
+
+		pFileInfo = NULL;
+		int iNum = 0;
+		int iFileNum = 0;
+
+		for (FileQueue::iterator it = m_DownloadQueue.GetFileQueue()->begin(); it != m_DownloadQueue.GetFileQueue()->end(); it++)
 		{
-			if (pFileInfo->GetArticles()->empty() && g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+			FileInfo* pFileInfo1 = *it;
+			if ((!pCheckedFiles || !pCheckedFiles[iNum]) && 
+				!pFileInfo1->GetPaused() && !pFileInfo1->GetDeleted() &&
+				(!pFileInfo ||
+				 (pFileInfo1->GetExtraPriority() == pFileInfo->GetExtraPriority() &&
+				  pFileInfo1->GetPriority() > pFileInfo->GetPriority()) ||
+				 (pFileInfo1->GetExtraPriority() > pFileInfo->GetExtraPriority())))
 			{
-				g_pDiskState->LoadArticles(pFileInfo);
+				pFileInfo = pFileInfo1;
+				iFileNum = iNum;
 			}
-			for (FileInfo::Articles::iterator at = pFileInfo->GetArticles()->begin(); at != pFileInfo->GetArticles()->end(); at++)
+			iNum++;
+		}
+
+		if (!pFileInfo)
+		{
+			// there are no more files for download
+			break;
+		}
+
+		if (pFileInfo->GetArticles()->empty() && g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+		{
+			g_pDiskState->LoadArticles(pFileInfo);
+		}
+
+		// check if the file has any articles left for download
+		for (FileInfo::Articles::iterator at = pFileInfo->GetArticles()->begin(); at != pFileInfo->GetArticles()->end(); at++)
+		{
+			pArticleInfo = *at;
+			if (pArticleInfo->GetStatus() == 0)
 			{
-				pArticleInfo = *at;
-				if (pArticleInfo->GetStatus() == 0)
-				{
-					return true;
-				}
+				bOK = true;
+				break;
 			}
 		}
+
+		if (!bOK)
+		{
+			// the file doesn't have any articles left for download, we mark the file as such
+			if (!pCheckedFiles)
+			{
+				int iArrSize = sizeof(bool) * m_DownloadQueue.GetFileQueue()->size();
+				pCheckedFiles = (bool*)malloc(iArrSize);
+				memset(pCheckedFiles, false, iArrSize);
+			}
+			pCheckedFiles[iFileNum] = true;
+		}
 	}
 
-	return false;
+	if (pCheckedFiles)
+	{
+		free(pCheckedFiles);
+	}
+
+	return bOK;
 }
 
 void QueueCoordinator::StartArticleDownload(FileInfo* pFileInfo, ArticleInfo* pArticleInfo, NNTPConnection* pConnection)
@@ -465,6 +551,7 @@ void QueueCoordinator::StartArticleDownload(FileInfo* pFileInfo, ArticleInfo* pA
 	BuildArticleFilename(pArticleDownloader, pFileInfo, pArticleInfo);
 
 	pArticleInfo->SetStatus(ArticleInfo::aiRunning);
+	pFileInfo->SetActiveDownloads(pFileInfo->GetActiveDownloads() + 1);
 
 	m_ActiveDownloads.push_back(pArticleDownloader);
 	pArticleDownloader->Start();
@@ -483,17 +570,28 @@ void QueueCoordinator::BuildArticleFilename(ArticleDownloader* pArticleDownloade
 	tmpname[1024-1] = '\0';
 	pArticleDownloader->SetTempFilename(tmpname);
 
-	char szNZBNiceName[1024];
-	pFileInfo->GetNZBInfo()->GetNiceNZBName(szNZBNiceName, 1024);
-	
-	snprintf(name, 1024, "%s%c%s [%i/%i]", szNZBNiceName, (int)PATH_SEPARATOR, pFileInfo->GetFilename(), pArticleInfo->GetPartNumber(), pFileInfo->GetArticles()->size());
+	snprintf(name, 1024, "%s%c%s [%i/%i]", pFileInfo->GetNZBInfo()->GetName(), (int)PATH_SEPARATOR, pFileInfo->GetFilename(), pArticleInfo->GetPartNumber(), pFileInfo->GetArticles()->size());
 	name[1024-1] = '\0';
 	pArticleDownloader->SetInfoName(name);
 
 	if (g_pOptions->GetDirectWrite())
 	{
-		snprintf(name, 1024, "%s%i.out", g_pOptions->GetTempDir(), pFileInfo->GetID());
-		name[1024-1] = '\0';
+		pFileInfo->LockOutputFile();
+
+		if (pFileInfo->GetOutputFilename())
+		{
+			strncpy(name, pFileInfo->GetOutputFilename(), 1024);
+			name[1024-1] = '\0';
+		}
+		else
+		{
+			snprintf(name, 1024, "%s%c%i.out.tmp", pFileInfo->GetNZBInfo()->GetDestDir(), (int)PATH_SEPARATOR, pFileInfo->GetID());
+			name[1024-1] = '\0';
+			pFileInfo->SetOutputFilename(name);
+		}
+
+		pFileInfo->UnlockOutputFile();
+
 		pArticleDownloader->SetOutputFilename(name);
 	}
 }
@@ -607,6 +705,8 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 		}
 	}
 
+	pFileInfo->SetActiveDownloads(pFileInfo->GetActiveDownloads() - 1);
+
 	if (deleteFileObj)
 	{
 		bool fileDeleted = pFileInfo->GetDeleted();
@@ -667,12 +767,9 @@ void QueueCoordinator::DiscardDiskFile(FileInfo* pFileInfo)
 		}
 	}
 
-	if (g_pOptions->GetDirectWrite())
+	if (g_pOptions->GetDirectWrite() && pFileInfo->GetOutputFilename())
 	{
-		char name[1024];
-		snprintf(name, 1024, "%s%i.out", g_pOptions->GetTempDir(), pFileInfo->GetID());
-		name[1024-1] = '\0';
-		remove(name);
+		remove(pFileInfo->GetOutputFilename());
 	}
 }
 
@@ -704,9 +801,25 @@ bool QueueCoordinator::IsDupe(FileInfo* pFileInfo)
 void QueueCoordinator::LogDebugInfo()
 {
 	debug("--------------------------------------------");
-	debug("Dumping debug info to log");
+	debug("Dumping debug debug to log");
 	debug("--------------------------------------------");
 
+	debug("   SpeedMeter");
+	debug("   ----------");
+	float fSpeed = (float)(CalcCurrentDownloadSpeed() / 1024.0);
+	int iTimeDiff = (int)time(NULL) - m_iSpeedStartTime * SPEEDMETER_SLOTSIZE;
+	debug("      Speed: %f", fSpeed);
+	debug("      SpeedStartTime: %i", m_iSpeedStartTime);
+	debug("      SpeedTotalBytes: %i", m_iSpeedTotalBytes);
+	debug("      SpeedBytesIndex: %i", m_iSpeedBytesIndex);
+	debug("      AllBytes: %i", m_iAllBytes);
+	debug("      Time: %i", (int)time(NULL));
+	debug("      TimeDiff: %i", iTimeDiff);
+	for (int i=0; i < SPEEDMETER_SLOTS; i++)
+	{
+		debug("      Bytes[%i]: %i, Time[%i]: %i", i, m_iSpeedBytes[i], i, m_iSpeedTime[i]);
+	}
+
 	debug("   QueueCoordinator");
 	debug("   ----------------");
 
@@ -726,8 +839,7 @@ void QueueCoordinator::LogDebugInfo()
 
 void QueueCoordinator::ResetHangingDownloads()
 {
-	const int TimeOut = g_pOptions->GetTerminateTimeout();
-	if (TimeOut == 0)
+	if (g_pOptions->GetTerminateTimeout() == 0 && g_pOptions->GetConnectionTimeout() == 0)
 	{
 		return;
 	}
@@ -738,7 +850,15 @@ void QueueCoordinator::ResetHangingDownloads()
 	for (ActiveDownloads::iterator it = m_ActiveDownloads.begin(); it != m_ActiveDownloads.end();)
 	{
 		ArticleDownloader* pArticleDownloader = *it;
-		if (tm - pArticleDownloader->GetLastUpdateTime() > TimeOut &&
+
+		if (tm - pArticleDownloader->GetLastUpdateTime() > g_pOptions->GetConnectionTimeout() + 1 &&
+		   pArticleDownloader->GetStatus() == ArticleDownloader::adRunning)
+		{
+			error("Cancelling hanging download %s", pArticleDownloader->GetInfoName());
+			pArticleDownloader->Stop();
+		}
+		
+		if (tm - pArticleDownloader->GetLastUpdateTime() > g_pOptions->GetTerminateTimeout() &&
 		   pArticleDownloader->GetStatus() == ArticleDownloader::adRunning)
 		{
 			ArticleInfo* pArticleInfo = pArticleDownloader->GetArticleInfo();
@@ -753,6 +873,7 @@ void QueueCoordinator::ResetHangingDownloads()
 				error("Could not terminate hanging download %s", Util::BaseFileName(pArticleInfo->GetResultFilename()));
 			}
 			m_ActiveDownloads.erase(it);
+			pArticleDownloader->GetFileInfo()->SetActiveDownloads(pArticleDownloader->GetFileInfo()->GetActiveDownloads() - 1);
 			// it's not safe to destroy pArticleDownloader, because the state of object is unknown
 			delete pArticleDownloader;
 			it = m_ActiveDownloads.begin();
@@ -834,9 +955,7 @@ bool QueueCoordinator::SetQueueEntryNZBCategory(NZBInfo* pNZBInfo, const char* s
 {
 	if (pNZBInfo->GetPostProcess())
 	{
-		char szNZBNiceName[1024];
-		pNZBInfo->GetNiceNZBName(szNZBNiceName, 1024);
-		error("Could not change category for %s. File in post-process-stage", szNZBNiceName);
+		error("Could not change category for %s. File in post-process-stage", pNZBInfo->GetName());
 		return false;
 	}
 
@@ -853,6 +972,36 @@ bool QueueCoordinator::SetQueueEntryNZBCategory(NZBInfo* pNZBInfo, const char* s
 	return bOK;
 }
 
+bool QueueCoordinator::SetQueueEntryNZBName(NZBInfo* pNZBInfo, const char* szName)
+{
+	if (pNZBInfo->GetPostProcess())
+	{
+		error("Could not rename %s. File in post-process-stage", pNZBInfo->GetName());
+		return false;
+	}
+
+	if (strlen(szName) == 0)
+	{
+		error("Could not rename %s. The new name cannot be empty", pNZBInfo->GetName());
+		return false;
+	}
+
+	char szOldDestDir[1024];
+	strncpy(szOldDestDir, pNZBInfo->GetDestDir(), 1024);
+	szOldDestDir[1024-1] = '\0';
+
+	char szNZBNicename[1024];
+	NZBInfo::MakeNiceNZBName(szName, szNZBNicename, sizeof(szNZBNicename), false);
+	pNZBInfo->SetName(szNZBNicename);
+
+	pNZBInfo->BuildDestDirName();
+
+	bool bDirUnchanged = !strcmp(pNZBInfo->GetDestDir(), szOldDestDir);
+	bool bOK = bDirUnchanged || ArticleDownloader::MoveCompletedFiles(pNZBInfo, szOldDestDir);
+
+	return bOK;
+}
+
 /*
  * NOTE: DownloadQueue must be locked prior to call of this function
  */
@@ -860,11 +1009,7 @@ bool QueueCoordinator::MergeQueueEntries(NZBInfo* pDestNZBInfo, NZBInfo* pSrcNZB
 {
 	if (pDestNZBInfo->GetPostProcess() || pSrcNZBInfo->GetPostProcess())
 	{
-		char szDestNZBNiceName[1024];
-		pDestNZBInfo->GetNiceNZBName(szDestNZBNiceName, 1024);
-		char szSrcNZBNiceName[1024];
-		pSrcNZBInfo->GetNiceNZBName(szSrcNZBNiceName, 1024);
-		error("Could not merge %s and %s. File in post-process-stage", szDestNZBNiceName, szSrcNZBNiceName);
+		error("Could not merge %s and %s. File in post-process-stage", pDestNZBInfo->GetName(), pSrcNZBInfo->GetName());
 		return false;
 	}
 
