@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2009 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2011 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,16 +15,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 320 $
- * $Date: 2009-05-24 16:26:53 +0200 (Sun, 24 May 2009) $
+ * $Revision: 487 $
+ * $Date: 2012-10-20 13:06:45 +0200 (Sat, 20 Oct 2012) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -36,6 +36,7 @@
 #include <cctype>
 #include <cstdio>
 #include <sys/stat.h>
+#include <set>
 #ifndef WIN32
 #include <unistd.h>
 #include <sys/time.h>
@@ -110,7 +111,7 @@ void QueueEditor::PauseUnpauseEntry(FileInfo* pFileInfo, bool bPause)
 }
 
 /*
- * Removes entry with index iEntry
+ * Removes entry
  * returns true if successful, false if operation is not possible
  */
 void QueueEditor::DeleteEntry(FileInfo* pFileInfo)
@@ -120,7 +121,7 @@ void QueueEditor::DeleteEntry(FileInfo* pFileInfo)
 }
 
 /*
- * Moves entry identified with iID in the queue
+ * Moves entry in the queue
  * returns true if successful, false if operation is not possible
  */
 void QueueEditor::MoveEntry(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo, int iOffset)
@@ -148,12 +149,23 @@ void QueueEditor::MoveEntry(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo,
 	}
 }
 
+/*
+ * Set priority for entry
+ * returns true if successful, false if operation is not possible
+ */
+void QueueEditor::SetPriorityEntry(FileInfo* pFileInfo, const char* szPriority)
+{
+	debug("Setting priority %s for file %s", szPriority, pFileInfo->GetFilename());
+	int iPriority = atoi(szPriority);
+	pFileInfo->SetPriority(iPriority);
+}
+
 bool QueueEditor::EditEntry(int ID, bool bSmartOrder, EEditAction eAction, int iOffset, const char* szText)
 {
 	IDList cIDList;
 	cIDList.clear();
 	cIDList.push_back(ID);
-	return EditList(&cIDList, bSmartOrder, eAction, iOffset, szText);
+	return EditList(&cIDList, NULL, mmID, bSmartOrder, eAction, iOffset, szText);
 }
 
 bool QueueEditor::LockedEditEntry(DownloadQueue* pDownloadQueue, int ID, bool bSmartOrder, EEditAction eAction, int iOffset, const char* szText)
@@ -164,11 +176,20 @@ bool QueueEditor::LockedEditEntry(DownloadQueue* pDownloadQueue, int ID, bool bS
 	return InternEditList(pDownloadQueue, &cIDList, bSmartOrder, eAction, iOffset, szText);
 }
 
-bool QueueEditor::EditList(IDList* pIDList, bool bSmartOrder, EEditAction eAction, int iOffset, const char* szText)
+bool QueueEditor::EditList(IDList* pIDList, NameList* pNameList, EMatchMode eMatchMode, bool bSmartOrder, 
+	EEditAction eAction, int iOffset, const char* szText)
 {
 	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
 
-	bool bOK = InternEditList(pDownloadQueue, pIDList, bSmartOrder, eAction, iOffset, szText);
+	bool bOK = true;
+
+	if (pNameList)
+	{
+		pIDList = new IDList();
+		bOK = BuildIDListFromNameList(pDownloadQueue, pIDList, pNameList, eMatchMode, eAction);
+	}
+
+	bOK = bOK && (InternEditList(pDownloadQueue, pIDList, bSmartOrder, eAction, iOffset, szText) || eMatchMode == mmRegEx);
 
 	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
 	{
@@ -177,6 +198,11 @@ bool QueueEditor::EditList(IDList* pIDList, bool bSmartOrder, EEditAction eActio
 
 	g_pQueueCoordinator->UnlockQueue();
 
+	if (pNameList)
+	{
+		delete pIDList;
+	}
+
 	return bOK;
 }
 
@@ -203,6 +229,10 @@ bool QueueEditor::InternEditList(DownloadQueue* pDownloadQueue, IDList* pIDList,
 	{
 		MergeGroups(pDownloadQueue, &cItemList);
 	}
+	else if (eAction == eaFileReorder)
+	{
+		ReorderFiles(pDownloadQueue, &cItemList);
+	}
 	else
 	{
 		for (ItemList::iterator it = cItemList.begin(); it != cItemList.end(); it++)
@@ -228,10 +258,18 @@ bool QueueEditor::InternEditList(DownloadQueue* pDownloadQueue, IDList* pIDList,
 					DeleteEntry(pItem->m_pFileInfo);
 					break;
 
+				case eaFileSetPriority:
+					SetPriorityEntry(pItem->m_pFileInfo, szText);
+					break;
+
 				case eaGroupSetCategory:
 					SetNZBCategory(pItem->m_pFileInfo->GetNZBInfo(), szText);
 					break;
 
+				case eaGroupSetName:
+					SetNZBName(pItem->m_pFileInfo->GetNZBInfo(), szText);
+					break;
+
 				case eaGroupSetParameter:
 					SetNZBParameter(pItem->m_pFileInfo->GetNZBInfo(), szText);
 					break;
@@ -244,12 +282,14 @@ bool QueueEditor::InternEditList(DownloadQueue* pDownloadQueue, IDList* pIDList,
 				case eaGroupMoveOffset:
 				case eaGroupPauseAllPars:
 				case eaGroupPauseExtraPars:
-					EditGroup(pDownloadQueue, pItem->m_pFileInfo, eAction, iOffset);
+				case eaGroupSetPriority:
+					EditGroup(pDownloadQueue, pItem->m_pFileInfo, eAction, iOffset, szText);
 					break;
 
 				case eaFilePauseAllPars:
 				case eaFilePauseExtraPars:
 				case eaGroupMerge:
+				case eaFileReorder:
 					// remove compiler warning "enumeration not handled in switch"
 					break;
 			}
@@ -365,7 +405,79 @@ void QueueEditor::PrepareList(DownloadQueue* pDownloadQueue, ItemList* pItemList
 	}
 }
 
-bool QueueEditor::EditGroup(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo, EEditAction eAction, int iOffset)
+bool QueueEditor::BuildIDListFromNameList(DownloadQueue* pDownloadQueue, IDList* pIDList, NameList* pNameList, EMatchMode eMatchMode, EEditAction eAction)
+{
+#ifndef HAVE_REGEX_H
+	if (eMatchMode == mmRegEx)
+	{
+		return false;
+	}
+#endif
+
+	std::set<int> uniqueIDs;
+
+	for (NameList::iterator it = pNameList->begin(); it != pNameList->end(); it++)
+	{
+		const char* szName = *it;
+
+		RegEx *pRegEx = NULL;
+		if (eMatchMode == mmRegEx)
+		{
+			pRegEx = new RegEx(szName);
+			if (!pRegEx->IsValid())
+			{
+				delete pRegEx;
+				return false;
+			}
+		}
+
+		bool bFound = false;
+
+		for (FileQueue::iterator it2 = pDownloadQueue->GetFileQueue()->begin(); it2 != pDownloadQueue->GetFileQueue()->end(); it2++)
+		{
+			FileInfo* pFileInfo = *it2;
+			if (eAction < eaGroupMoveOffset)
+			{
+				// file action
+				char szFilename[MAX_PATH];
+				snprintf(szFilename, sizeof(szFilename) - 1, "%s/%s", pFileInfo->GetNZBInfo()->GetName(), Util::BaseFileName(pFileInfo->GetFilename()));
+				if (((!pRegEx && !strcmp(szFilename, szName)) || (pRegEx && pRegEx->Match(szFilename))) &&
+					(uniqueIDs.find(pFileInfo->GetID()) == uniqueIDs.end()))
+				{
+					uniqueIDs.insert(pFileInfo->GetID());
+					pIDList->push_back(pFileInfo->GetID());
+					bFound = true;
+				}
+			}
+			else
+			{
+				// group action
+				const char *szFilename = pFileInfo->GetNZBInfo()->GetName();
+				if (((!pRegEx && !strcmp(szFilename, szName)) || (pRegEx && pRegEx->Match(szFilename))) &&
+					(uniqueIDs.find(pFileInfo->GetNZBInfo()->GetID()) == uniqueIDs.end()))
+				{
+					uniqueIDs.insert(pFileInfo->GetNZBInfo()->GetID());
+					pIDList->push_back(pFileInfo->GetID());
+					bFound = true;
+				}
+			}
+		}
+
+		if (pRegEx)
+		{
+			delete pRegEx;
+		}
+
+		if (!bFound && (eMatchMode == mmName))
+		{
+			return false;
+		}
+	}
+
+	return true;
+}
+
+bool QueueEditor::EditGroup(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo, EEditAction eAction, int iOffset, const char* szText)
 {
 	IDList cIDList;
 	cIDList.clear();
@@ -431,10 +543,13 @@ bool QueueEditor::EditGroup(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo,
 		pFileInfo->GetNZBInfo()->SetCleanupDisk(CanCleanupDisk(pDownloadQueue, pFileInfo->GetNZBInfo()));
 	}
 
-	EEditAction GroupToFileMap[] = { (EEditAction)0, eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, eaFilePause, eaFileResume, eaFileDelete, eaFilePauseAllPars, eaFilePauseExtraPars,
-		eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, eaFilePause, eaFileResume, eaFileDelete, eaFilePauseAllPars, eaFilePauseExtraPars, (EEditAction)0, (EEditAction)0, (EEditAction)0 };
+	EEditAction GroupToFileMap[] = { (EEditAction)0, eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, 
+		eaFilePause, eaFileResume, eaFileDelete, eaFilePauseAllPars, eaFilePauseExtraPars, eaFileSetPriority, eaFileReorder,
+		eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, eaFilePause, eaFileResume, eaFileDelete, 
+		eaFilePauseAllPars, eaFilePauseExtraPars, eaFileSetPriority,
+		(EEditAction)0, (EEditAction)0, (EEditAction)0 };
 
-	return InternEditList(pDownloadQueue, &cIDList, true, GroupToFileMap[eAction], iOffset, NULL);
+	return InternEditList(pDownloadQueue, &cIDList, true, GroupToFileMap[eAction], iOffset, szText);
 }
 
 void QueueEditor::BuildGroupList(DownloadQueue* pDownloadQueue, FileList* pGroupList)
@@ -533,11 +648,11 @@ void QueueEditor::AlignAffectedGroups(DownloadQueue* pDownloadQueue, IDList* pID
 	for (FileList::iterator it = cAffectedGroupList.begin(); it != cAffectedGroupList.end(); it++)
 	{
 		FileInfo* pFileInfo = *it;
-		AlignGroup(pDownloadQueue, pFileInfo);
+		AlignGroup(pDownloadQueue, pFileInfo->GetNZBInfo());
 	}
 }
 
-void QueueEditor::AlignGroup(DownloadQueue* pDownloadQueue, FileInfo* pFirstFileInfo)
+void QueueEditor::AlignGroup(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 {
 	FileInfo* pLastFileInfo = NULL;
 	unsigned int iLastNum = 0;
@@ -545,7 +660,7 @@ void QueueEditor::AlignGroup(DownloadQueue* pDownloadQueue, FileInfo* pFirstFile
 	while (iNum < pDownloadQueue->GetFileQueue()->size())
 	{
 		FileInfo* pFileInfo = pDownloadQueue->GetFileQueue()->at(iNum);
-		if (pFirstFileInfo->GetNZBInfo() == pFileInfo->GetNZBInfo())
+		if (pFileInfo->GetNZBInfo() == pNZBInfo)
 		{
 			if (pLastFileInfo && iNum - iLastNum > 1)
 			{
@@ -687,6 +802,13 @@ void QueueEditor::SetNZBCategory(NZBInfo* pNZBInfo, const char* szCategory)
 	g_pQueueCoordinator->SetQueueEntryNZBCategory(pNZBInfo, szCategory);
 }
 
+void QueueEditor::SetNZBName(NZBInfo* pNZBInfo, const char* szName)
+{
+	debug("QueueEditor: renaming '%s' to '%s'", Util::BaseFileName(pNZBInfo->GetFilename()), szName);
+
+	g_pQueueCoordinator->SetQueueEntryNZBName(pNZBInfo, szName);
+}
+
 /**
 * Check if deletion of already downloaded files is possible (when nzb id deleted from queue).
 * The deletion is most always possible, except the case if all remaining files in queue 
@@ -732,20 +854,57 @@ void QueueEditor::MergeGroups(DownloadQueue* pDownloadQueue, ItemList* pItemList
 		delete pItem;
 	}
 
-	// align group ("AlignGroup" needs the first file item as parameter)
-	for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
-	{
-		FileInfo* pFileInfo = *it;
-		if (pFileInfo->GetNZBInfo() == pDestItem->m_pFileInfo->GetNZBInfo())
-		{
-			AlignGroup(pDownloadQueue, pFileInfo);
-			break;
-		}
-	}
+	// align group
+	AlignGroup(pDownloadQueue, pDestItem->m_pFileInfo->GetNZBInfo());
 
 	delete pDestItem;
 }
 
+void QueueEditor::ReorderFiles(DownloadQueue* pDownloadQueue, ItemList* pItemList)
+{
+	if (pItemList->size() == 0)
+	{
+		return;
+	}
+
+	EditItem* pFirstItem = pItemList->front();
+	NZBInfo* pNZBInfo = pFirstItem->m_pFileInfo->GetNZBInfo();
+	unsigned int iInsertPos = 0;
+
+	// find first file of the group
+    for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
+    {
+        FileInfo* pFileInfo = *it;
+		if (pFileInfo->GetNZBInfo() == pNZBInfo)
+		{
+			break;
+		}
+		iInsertPos++;
+	}
+
+	// now can reorder
+	for (ItemList::iterator it = pItemList->begin(); it != pItemList->end(); it++)
+	{
+		EditItem* pItem = *it;
+		FileInfo* pFileInfo = pItem->m_pFileInfo;
+
+		// move file item
+		for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
+		{
+			FileInfo* pFileInfo1 = *it;
+			if (pFileInfo1 == pFileInfo)
+			{
+				pDownloadQueue->GetFileQueue()->erase(it);
+				pDownloadQueue->GetFileQueue()->insert(pDownloadQueue->GetFileQueue()->begin() + iInsertPos, pFileInfo);
+				iInsertPos++;				
+				break;
+			}
+		}
+
+		delete pItem;
+	}
+}
+
 void QueueEditor::SetNZBParameter(NZBInfo* pNZBInfo, const char* szParamString)
 {
 	debug("QueueEditor: setting nzb parameter '%s' for '%s'", szParamString, Util::BaseFileName(pNZBInfo->GetFilename()));
@@ -761,7 +920,7 @@ void QueueEditor::SetNZBParameter(NZBInfo* pNZBInfo, const char* szParamString)
 	}
 	else
 	{
-		error("Could not set nzb parameter for %s: invalid argument: %s", Util::BaseFileName(pNZBInfo->GetFilename()), szParamString);
+		error("Could not set nzb parameter for %s: invalid argument: %s", pNZBInfo->GetName(), szParamString);
 	}
 
 	free(szStr);
