@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2011 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 555 $
+ * $Date: 2013-02-06 23:04:50 +0100 (Wed, 06 Feb 2013) $
  *
  */
 
@@ -46,6 +46,7 @@
 
 #include "nzbget.h"
 #include "RemoteClient.h"
+#include "DownloadInfo.h"
 #include "Options.h"
 #include "Log.h"
 #include "Util.h"
@@ -55,7 +56,6 @@ extern Options* g_pOptions;
 RemoteClient::RemoteClient()
 {
 	m_pConnection	= NULL;
-	m_pNetAddress	= NULL;
 	m_bVerbose		= true;
 
 	/*
@@ -80,11 +80,6 @@ RemoteClient::~RemoteClient()
 	{
 		delete m_pConnection;
 	}
-
-	if (m_pNetAddress)
-	{
-		delete m_pNetAddress;
-	}
 }
 
 void RemoteClient::printf(const char * msg,...)
@@ -109,13 +104,19 @@ void RemoteClient::perror(const char * msg)
 bool RemoteClient::InitConnection()
 {
 	// Create a connection to the server
-	m_pNetAddress	= new NetAddress(g_pOptions->GetServerIP(), g_pOptions->GetServerPort());
-	m_pConnection	= new Connection(m_pNetAddress);
+
+	const char *szControlIP = g_pOptions->GetControlIP();
+	if (!strcmp(szControlIP, "0.0.0.0"))
+	{
+		szControlIP = "127.0.0.1";
+	}
+
+	m_pConnection = new Connection(szControlIP, g_pOptions->GetControlPort(), false);
 
 	bool OK = m_pConnection->Connect();
 	if (!OK)
 	{
-		printf("Unable to send request to nzbserver at %s (port %i)\n", g_pOptions->GetServerIP(), g_pOptions->GetServerPort());
+		printf("Unable to send request to nzbserver at %s (port %i)\n", szControlIP, g_pOptions->GetControlPort());
 	}
 	return OK;
 }
@@ -125,7 +126,7 @@ void RemoteClient::InitMessageBase(SNZBRequestBase* pMessageBase, int iRequest,
 	pMessageBase->m_iSignature	= htonl(NZBMESSAGE_SIGNATURE);
 	pMessageBase->m_iType = htonl(iRequest);
 	pMessageBase->m_iStructSize = htonl(iSize);
-	strncpy(pMessageBase->m_szPassword, g_pOptions->GetServerPassword(), NZBREQUESTPASSWORDSIZE - 1);
+	strncpy(pMessageBase->m_szPassword, g_pOptions->GetControlPassword(), NZBREQUESTPASSWORDSIZE - 1);
 	pMessageBase->m_szPassword[NZBREQUESTPASSWORDSIZE - 1] = '\0';
 }
 
@@ -137,35 +138,22 @@ bool RemoteClient::ReceiveBoolResponse()
 	SNZBDownloadResponse BoolResponse;
 	memset(&BoolResponse, 0, sizeof(BoolResponse));
 
-	int iResponseLen = m_pConnection->Recv((char*)&BoolResponse, sizeof(BoolResponse));
-	if (iResponseLen != sizeof(BoolResponse) || 
+	bool bRead = m_pConnection->Recv((char*)&BoolResponse, sizeof(BoolResponse));
+	if (!bRead || 
 		(int)ntohl(BoolResponse.m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE ||
 		ntohl(BoolResponse.m_MessageBase.m_iStructSize) != sizeof(BoolResponse))
 	{
-		if (iResponseLen < 0)
-		{
-			printf("No response received (timeout)\n");
-		}
-		else
-		{
-			printf("Invalid response received: either not nzbget-server or wrong server version\n");
-		}
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
 		return false;
 	}
 
 	int iTextLen = ntohl(BoolResponse.m_iTrailingDataLength);
 	char* buf = (char*)malloc(iTextLen);
-	iResponseLen = m_pConnection->Recv(buf, iTextLen);
-	if (iResponseLen != iTextLen)
+	bRead = m_pConnection->Recv(buf, iTextLen);
+	if (!bRead)
 	{
-		if (iResponseLen < 0)
-		{
-			printf("No response received (timeout)\n");
-		}
-		else
-		{
-			printf("Invalid response received: either not nzbget-server or wrong server version\n");
-		}
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
+		free(buf);
 		return false;
 	}
 
@@ -177,7 +165,7 @@ bool RemoteClient::ReceiveBoolResponse()
 /*
  * Sends a message to the running nzbget process.
  */
-bool RemoteClient::RequestServerDownload(const char* szFilename, const char* szCategory, bool bAddFirst)
+bool RemoteClient::RequestServerDownload(const char* szFilename, const char* szCategory, bool bAddFirst, bool bAddPaused, int iPriority)
 {
 	// Read the file into the buffer
 	char* szBuffer	= NULL;
@@ -194,6 +182,8 @@ bool RemoteClient::RequestServerDownload(const char* szFilename, const char* szC
 		SNZBDownloadRequest DownloadRequest;
 		InitMessageBase(&DownloadRequest.m_MessageBase, eRemoteRequestDownload, sizeof(DownloadRequest));
 		DownloadRequest.m_bAddFirst = htonl(bAddFirst);
+		DownloadRequest.m_bAddPaused = htonl(bAddPaused);
+		DownloadRequest.m_iPriority = htonl(iPriority);
 		DownloadRequest.m_iTrailingDataLength = htonl(iLength);
 
 		strncpy(DownloadRequest.m_szFilename, szFilename, NZBREQUESTFILENAMESIZE - 1);
@@ -205,7 +195,7 @@ bool RemoteClient::RequestServerDownload(const char* szFilename, const char* szC
 		}
 		DownloadRequest.m_szCategory[NZBREQUESTFILENAMESIZE-1] = '\0';
 
-		if (m_pConnection->Send((char*)(&DownloadRequest), sizeof(DownloadRequest)) < 0)
+		if (!m_pConnection->Send((char*)(&DownloadRequest), sizeof(DownloadRequest)))
 		{
 			perror("m_pConnection->Send");
 			OK = false;
@@ -239,23 +229,29 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 			SNZBListResponseNZBEntry* pListAnswer = (SNZBListResponseNZBEntry*) pBufPtr;
 
 			const char* szFileName = pBufPtr + sizeof(SNZBListResponseNZBEntry);
-			const char* szDestDir = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen);
-			const char* szCategory = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) + ntohl(pListAnswer->m_iDestDirLen);
-			const char* m_szQueuedFilename = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) + ntohl(pListAnswer->m_iDestDirLen) + ntohl(pListAnswer->m_iCategoryLen);
+			const char* szName = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen);
+			const char* szDestDir = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) + 
+				ntohl(pListAnswer->m_iNameLen);
+			const char* szCategory = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) + 
+				ntohl(pListAnswer->m_iNameLen) + ntohl(pListAnswer->m_iDestDirLen);
+			const char* m_szQueuedFilename = pBufPtr + sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) + 
+				ntohl(pListAnswer->m_iNameLen) + ntohl(pListAnswer->m_iDestDirLen) + ntohl(pListAnswer->m_iCategoryLen);
 			
-			NZBInfo* pNZBInfo = new NZBInfo();
+			MatchedNZBInfo* pNZBInfo = new MatchedNZBInfo();
 			pNZBInfo->SetSize(Util::JoinInt64(ntohl(pListAnswer->m_iSizeHi), ntohl(pListAnswer->m_iSizeLo)));
 			pNZBInfo->SetFilename(szFileName);
+			pNZBInfo->SetName(szName);
 			pNZBInfo->SetDestDir(szDestDir);
 			pNZBInfo->SetCategory(szCategory);
 			pNZBInfo->SetQueuedFilename(m_szQueuedFilename);
+			pNZBInfo->m_bMatch = ntohl(pListAnswer->m_bMatch);
 
 			pNZBInfo->AddReference();
 			pDownloadQueue->GetNZBInfoList()->Add(pNZBInfo);
 
 			pBufPtr += sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) +
-				ntohl(pListAnswer->m_iDestDirLen) + ntohl(pListAnswer->m_iCategoryLen) + 
-				ntohl(pListAnswer->m_iQueuedFilenameLen);
+				ntohl(pListAnswer->m_iNameLen) + ntohl(pListAnswer->m_iDestDirLen) + 
+				ntohl(pListAnswer->m_iCategoryLen) + ntohl(pListAnswer->m_iQueuedFilenameLen);
 		}
 
 		//read ppp entries
@@ -281,7 +277,7 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 			const char* szSubject = pBufPtr + sizeof(SNZBListResponseFileEntry);
 			const char* szFileName = pBufPtr + sizeof(SNZBListResponseFileEntry) + ntohl(pListAnswer->m_iSubjectLen);
 			
-			FileInfo* pFileInfo = new FileInfo();
+			MatchedFileInfo* pFileInfo = new MatchedFileInfo();
 			pFileInfo->SetID(ntohl(pListAnswer->m_iID));
 			pFileInfo->SetSize(Util::JoinInt64(ntohl(pListAnswer->m_iFileSizeHi), ntohl(pListAnswer->m_iFileSizeLo)));
 			pFileInfo->SetRemainingSize(Util::JoinInt64(ntohl(pListAnswer->m_iRemainingSizeHi), ntohl(pListAnswer->m_iRemainingSizeLo)));
@@ -289,6 +285,9 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 			pFileInfo->SetSubject(szSubject);
 			pFileInfo->SetFilename(szFileName);
 			pFileInfo->SetFilenameConfirmed(ntohl(pListAnswer->m_bFilenameConfirmed));
+			pFileInfo->SetActiveDownloads(ntohl(pListAnswer->m_iActiveDownloads));
+			pFileInfo->SetPriority(ntohl(pListAnswer->m_iPriority));
+			pFileInfo->m_bMatch = ntohl(pListAnswer->m_bMatch);
 
 			NZBInfo* pNZBInfo = pDownloadQueue->GetNZBInfoList()->at(ntohl(pListAnswer->m_iNZBIndex) - 1);
 
@@ -304,7 +303,7 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 	pDownloadQueue->GetNZBInfoList()->ReleaseAll();
 }
 
-bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
+bool RemoteClient::RequestServerList(bool bFiles, bool bGroups, const char* szPattern)
 {
 	if (!InitConnection()) return false;
 
@@ -312,8 +311,15 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 	InitMessageBase(&ListRequest.m_MessageBase, eRemoteRequestList, sizeof(ListRequest));
 	ListRequest.m_bFileList = htonl(true);
 	ListRequest.m_bServerState = htonl(true);
+	ListRequest.m_iMatchMode = htonl(szPattern ? eRemoteMatchModeRegEx : eRemoteMatchModeID);
+	ListRequest.m_bMatchGroup = htonl(bGroups);
+	if (szPattern)
+	{
+		strncpy(ListRequest.m_szPattern, szPattern, NZBREQUESTFILENAMESIZE - 1);
+		ListRequest.m_szPattern[NZBREQUESTFILENAMESIZE-1] = '\0';
+	}
 
-	if (m_pConnection->Send((char*)(&ListRequest), sizeof(ListRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&ListRequest), sizeof(ListRequest)))
 	{
 		perror("m_pConnection->Send");
 		return false;
@@ -323,19 +329,12 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 
 	// Now listen for the returned list
 	SNZBListResponse ListResponse;
-	int iResponseLen = m_pConnection->Recv((char*) &ListResponse, sizeof(ListResponse));
-	if (iResponseLen != sizeof(ListResponse) || 
+	bool bRead = m_pConnection->Recv((char*) &ListResponse, sizeof(ListResponse));
+	if (!bRead || 
 		(int)ntohl(ListResponse.m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE ||
 		ntohl(ListResponse.m_MessageBase.m_iStructSize) != sizeof(ListResponse))
 	{
-		if (iResponseLen < 0)
-		{
-			printf("No response received (timeout)\n");
-		}
-		else
-		{
-			printf("Invalid response received: either not nzbget-server or wrong server version\n");
-		}
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
 		return false;
 	}
 
@@ -343,7 +342,7 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 	if (ntohl(ListResponse.m_iTrailingDataLength) > 0)
 	{
 		pBuf = (char*)malloc(ntohl(ListResponse.m_iTrailingDataLength));
-		if (!m_pConnection->RecvAll(pBuf, ntohl(ListResponse.m_iTrailingDataLength)))
+		if (!m_pConnection->Recv(pBuf, ntohl(ListResponse.m_iTrailingDataLength)))
 		{
 			free(pBuf);
 			return false;
@@ -352,6 +351,13 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 
 	m_pConnection->Disconnect();
 
+	if (szPattern && !ListResponse.m_bRegExValid)
+	{
+		printf("Error in regular expression\n");
+		free(pBuf);
+		return false;
+	}
+
 	if (bFiles)
 	{
 		if (ntohl(ListResponse.m_iTrailingDataLength) == 0)
@@ -368,17 +374,33 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 
 			long long lRemaining = 0;
 			long long lPaused = 0;
+			int iMatches = 0;
 
 			for (FileQueue::iterator it = cRemoteQueue.GetFileQueue()->begin(); it != cRemoteQueue.GetFileQueue()->end(); it++)
 			{
 				FileInfo* pFileInfo = *it;
 
+				char szPriority[100];
+				szPriority[0] = '\0';
+				if (pFileInfo->GetPriority() != 0)
+				{
+					sprintf(szPriority, "[%+i] ", pFileInfo->GetPriority());
+				}
+
 				char szCompleted[100];
 				szCompleted[0] = '\0';
 				if (pFileInfo->GetRemainingSize() < pFileInfo->GetSize())
 				{
 					sprintf(szCompleted, ", %i%s", (int)(100 - Util::Int64ToFloat(pFileInfo->GetRemainingSize()) * 100.0 / Util::Int64ToFloat(pFileInfo->GetSize())), "%");
 				}
+
+				char szThreads[100];
+				szThreads[0] = '\0';
+				if (pFileInfo->GetActiveDownloads() > 0)
+				{
+					sprintf(szThreads, ", %i thread%s", pFileInfo->GetActiveDownloads(), (pFileInfo->GetActiveDownloads() > 1 ? "s" : ""));
+				}
+
 				char szStatus[100];
 				if (pFileInfo->GetPaused())
 				{
@@ -390,18 +412,30 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 					szStatus[0] = '\0';
 					lRemaining += pFileInfo->GetRemainingSize();
 				}
-				
-				char szNZBNiceName[1024];
-				pFileInfo->GetNZBInfo()->GetNiceNZBName(szNZBNiceName, 1024);
-				
-				printf("[%i] %s%c%s (%.2f MB%s)%s\n", pFileInfo->GetID(), szNZBNiceName, (int)PATH_SEPARATOR, pFileInfo->GetFilename(),
-					(float)(Util::Int64ToFloat(pFileInfo->GetSize()) / 1024.0 / 1024.0), szCompleted, szStatus);
+
+				if (!szPattern || ((MatchedFileInfo*)pFileInfo)->m_bMatch)
+				{
+					printf("[%i] %s%s/%s (%.2f MB%s%s)%s\n", pFileInfo->GetID(), szPriority, pFileInfo->GetNZBInfo()->GetName(),
+						pFileInfo->GetFilename(),
+						(float)(Util::Int64ToFloat(pFileInfo->GetSize()) / 1024.0 / 1024.0), 
+						szCompleted, szThreads, szStatus);
+					iMatches++;
+				}
 
 				delete pFileInfo;
 			}
 
+			if (iMatches == 0)
+			{
+				printf("No matches founds\n");
+			}
+
 			printf("-----------------------------------\n");
 			printf("Files: %i\n", cRemoteQueue.GetFileQueue()->size());
+			if (szPattern)
+			{
+				printf("Matches: %i\n", iMatches);
+			}
 			if (lPaused > 0)
 			{
 				printf("Remaining size: %.2f MB (+%.2f MB paused)\n", (float)(Util::Int64ToFloat(lRemaining) / 1024.0 / 1024.0), 
@@ -433,6 +467,7 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 
 			long long lRemaining = 0;
 			long long lPaused = 0;
+			int iMatches = 0;
 
 			for (GroupQueue::iterator it = cGroupQueue.begin(); it != cGroupQueue.end(); it++)
 			{
@@ -444,6 +479,20 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 				char szRemaining[20];
 				Util::FormatFileSize(szRemaining, sizeof(szRemaining), lUnpausedRemainingSize);
 
+				char szPriority[100];
+				szPriority[0] = '\0';
+				if (pGroupInfo->GetMinPriority() != 0 || pGroupInfo->GetMaxPriority() != 0)
+				{
+					if (pGroupInfo->GetMinPriority() == pGroupInfo->GetMaxPriority())
+					{
+						sprintf(szPriority, "[%+i] ", pGroupInfo->GetMinPriority());
+					}
+					else
+					{
+						sprintf(szPriority, "[%+i..%+i] ", pGroupInfo->GetMinPriority(), pGroupInfo->GetMaxPriority());
+					}
+				}
+
 				char szPaused[20];
 				szPaused[0] = '\0';
 				if (pGroupInfo->GetPausedSize() > 0)
@@ -454,9 +503,6 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 					lPaused += pGroupInfo->GetPausedSize();
 				}
 
-				char szNZBNiceName[1024];
-				pGroupInfo->GetNZBInfo()->GetNiceNZBName(szNZBNiceName, 1023);
-
 				char szCategory[1024];
 				szCategory[0] = '\0';
 				if (pGroupInfo->GetNZBInfo()->GetCategory() && strlen(pGroupInfo->GetNZBInfo()->GetCategory()) > 0)
@@ -464,6 +510,13 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 					sprintf(szCategory, " (%s)", pGroupInfo->GetNZBInfo()->GetCategory());
 				}
 
+				char szThreads[100];
+				szThreads[0] = '\0';
+				if (pGroupInfo->GetActiveDownloads() > 0)
+				{
+					sprintf(szThreads, ", %i thread%s", pGroupInfo->GetActiveDownloads(), (pGroupInfo->GetActiveDownloads() > 1 ? "s" : ""));
+				}
+
 				char szParameters[1024];
 				szParameters[0] = '\0';
 				for (NZBParameterList::iterator it = pGroupInfo->GetNZBInfo()->GetParameters()->begin(); it != pGroupInfo->GetNZBInfo()->GetParameters()->end(); it++)
@@ -486,9 +539,14 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 					strncat(szParameters, ")", 1024);
 				}
 
-				printf("[%i-%i] %s (%i file%s, %s%s)%s%s\n", pGroupInfo->GetFirstID(), pGroupInfo->GetLastID(), szNZBNiceName, 
-					pGroupInfo->GetRemainingFileCount(), pGroupInfo->GetRemainingFileCount() > 1 ? "s" : "", szRemaining, 
-					szPaused, szCategory, szParameters);
+				if (!szPattern || ((MatchedNZBInfo*)pGroupInfo->GetNZBInfo())->m_bMatch)
+				{
+					printf("[%i-%i] %s%s (%i file%s, %s%s%s)%s%s\n", pGroupInfo->GetFirstID(), pGroupInfo->GetLastID(), szPriority, 
+						pGroupInfo->GetNZBInfo()->GetName(), pGroupInfo->GetRemainingFileCount(),
+						pGroupInfo->GetRemainingFileCount() > 1 ? "s" : "", szRemaining, 
+						szPaused, szThreads, szCategory, szParameters);
+					iMatches++;
+				}
 
 				delete pGroupInfo;
 			}
@@ -498,8 +556,17 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups)
 				delete *it;
 			}
 
+			if (iMatches == 0)
+			{
+				printf("No matches founds\n");
+			}
+
 			printf("-----------------------------------\n");
 			printf("Groups: %i\n", cGroupQueue.size());
+			if (szPattern)
+			{
+				printf("Matches: %i\n", iMatches);
+			}
 			printf("Files: %i\n", cRemoteQueue.GetFileQueue()->size());
 			if (lPaused > 0)
 			{
@@ -613,7 +680,7 @@ bool RemoteClient::RequestServerLog(int iLines)
 	LogRequest.m_iLines = htonl(iLines);
 	LogRequest.m_iIDFrom = 0;
 
-	if (m_pConnection->Send((char*)(&LogRequest), sizeof(LogRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&LogRequest), sizeof(LogRequest)))
 	{
 		perror("m_pConnection->Send");
 		return false;
@@ -623,19 +690,12 @@ bool RemoteClient::RequestServerLog(int iLines)
 
 	// Now listen for the returned log
 	SNZBLogResponse LogResponse;
-	int iResponseLen = m_pConnection->Recv((char*) &LogResponse, sizeof(LogResponse));
-	if (iResponseLen != sizeof(LogResponse) || 
+	bool bRead = m_pConnection->Recv((char*) &LogResponse, sizeof(LogResponse));
+	if (!bRead || 
 		(int)ntohl(LogResponse.m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE ||
 		ntohl(LogResponse.m_MessageBase.m_iStructSize) != sizeof(LogResponse))
 	{
-		if (iResponseLen < 0)
-		{
-			printf("No response received (timeout)\n");
-		}
-		else
-		{
-			printf("Invalid response received: either not nzbget-server or wrong server version\n");
-		}
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
 		return false;
 	}
 
@@ -643,7 +703,7 @@ bool RemoteClient::RequestServerLog(int iLines)
 	if (ntohl(LogResponse.m_iTrailingDataLength) > 0)
 	{
 		pBuf = (char*)malloc(ntohl(LogResponse.m_iTrailingDataLength));
-		if (!m_pConnection->RecvAll(pBuf, ntohl(LogResponse.m_iTrailingDataLength)))
+		if (!m_pConnection->Recv(pBuf, ntohl(LogResponse.m_iTrailingDataLength)))
 		{
 			free(pBuf);
 			return false;
@@ -706,7 +766,7 @@ bool RemoteClient::RequestServerPauseUnpause(bool bPause, eRemotePauseUnpauseAct
 	PauseUnpauseRequest.m_bPause = htonl(bPause);
 	PauseUnpauseRequest.m_iAction = htonl(iAction);
 
-	if (m_pConnection->Send((char*)(&PauseUnpauseRequest), sizeof(PauseUnpauseRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&PauseUnpauseRequest), sizeof(PauseUnpauseRequest)))
 	{
 		perror("m_pConnection->Send");
 		m_pConnection->Disconnect();
@@ -719,15 +779,15 @@ bool RemoteClient::RequestServerPauseUnpause(bool bPause, eRemotePauseUnpauseAct
 	return OK;
 }
 
-bool RemoteClient::RequestServerSetDownloadRate(float fRate)
+bool RemoteClient::RequestServerSetDownloadRate(int iRate)
 {
 	if (!InitConnection()) return false;
 
 	SNZBSetDownloadRateRequest SetDownloadRateRequest;
 	InitMessageBase(&SetDownloadRateRequest.m_MessageBase, eRemoteRequestSetDownloadRate, sizeof(SetDownloadRateRequest));
-	SetDownloadRateRequest.m_iDownloadRate = htonl((unsigned int)(fRate * 1024));
+	SetDownloadRateRequest.m_iDownloadRate = htonl(iRate);
 
-	if (m_pConnection->Send((char*)(&SetDownloadRateRequest), sizeof(SetDownloadRateRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&SetDownloadRateRequest), sizeof(SetDownloadRateRequest)))
 	{
 		perror("m_pConnection->Send");
 		m_pConnection->Disconnect();
@@ -747,7 +807,7 @@ bool RemoteClient::RequestServerDumpDebug()
 	SNZBDumpDebugRequest DumpDebugInfo;
 	InitMessageBase(&DumpDebugInfo.m_MessageBase, eRemoteRequestDumpDebug, sizeof(DumpDebugInfo));
 
-	if (m_pConnection->Send((char*)(&DumpDebugInfo), sizeof(DumpDebugInfo)) < 0)
+	if (!m_pConnection->Send((char*)(&DumpDebugInfo), sizeof(DumpDebugInfo)))
 	{
 		perror("m_pConnection->Send");
 		m_pConnection->Disconnect();
@@ -760,9 +820,10 @@ bool RemoteClient::RequestServerDumpDebug()
 	return OK;
 }
 
-bool RemoteClient::RequestServerEditQueue(eRemoteEditAction iAction, int iOffset, const char* szText, int* pIDList, int iIDCount, bool bSmartOrder)
+bool RemoteClient::RequestServerEditQueue(eRemoteEditAction iAction, int iOffset, const char* szText, 
+	int* pIDList, int iIDCount, NameList* pNameList, eRemoteMatchMode iMatchMode, bool bSmartOrder)
 {
-	if (iIDCount <= 0 || pIDList == NULL)
+	if ((iIDCount <= 0 || pIDList == NULL) && (pNameList == NULL || pNameList->size() == 0))
 	{
 		printf("File(s) not specified\n");
 		return false;
@@ -770,18 +831,38 @@ bool RemoteClient::RequestServerEditQueue(eRemoteEditAction iAction, int iOffset
 
 	if (!InitConnection()) return false;
 
+	int iIDLength = sizeof(int32_t) * iIDCount;
+
+	int iNameCount = 0;
+	int iNameLength = 0;
+	if (pNameList && pNameList->size() > 0)
+	{
+		for (NameList::iterator it = pNameList->begin(); it != pNameList->end(); it++)
+		{
+			const char *szName = *it;
+			iNameLength += strlen(szName) + 1;
+			iNameCount++;
+		}
+		// align size to 4-bytes, needed by ARM-processor (and may be others)
+		iNameLength += iNameLength % 4 > 0 ? 4 - iNameLength % 4 : 0;
+	}
+
 	int iTextLen = szText ? strlen(szText) + 1 : 0;
 	// align size to 4-bytes, needed by ARM-processor (and may be others)
 	iTextLen += iTextLen % 4 > 0 ? 4 - iTextLen % 4 : 0;
-	int iLength = sizeof(int32_t) * iIDCount + iTextLen;
+
+	int iLength = iTextLen + iIDLength + iNameLength;
 
 	SNZBEditQueueRequest EditQueueRequest;
 	InitMessageBase(&EditQueueRequest.m_MessageBase, eRemoteRequestEditQueue, sizeof(EditQueueRequest));
 	EditQueueRequest.m_iAction = htonl(iAction);
+	EditQueueRequest.m_iMatchMode = htonl(iMatchMode);
 	EditQueueRequest.m_iOffset = htonl((int)iOffset);
 	EditQueueRequest.m_bSmartOrder = htonl(bSmartOrder);
 	EditQueueRequest.m_iTextLen = htonl(iTextLen);
-	EditQueueRequest.m_iNrTrailingEntries = htonl(iIDCount);
+	EditQueueRequest.m_iNrTrailingIDEntries = htonl(iIDCount);
+	EditQueueRequest.m_iNrTrailingNameEntries = htonl(iNameCount);
+	EditQueueRequest.m_iTrailingNameEntriesLen = htonl(iNameLength);
 	EditQueueRequest.m_iTrailingDataLength = htonl(iLength);
 
 	char* pTrailingData = (char*)malloc(iLength);
@@ -797,9 +878,21 @@ bool RemoteClient::RequestServerEditQueue(eRemoteEditAction iAction, int iOffset
 	{
 		pIDs[i] = htonl(pIDList[i]);
 	}
-			
+	
+	if (iNameCount > 0)
+	{
+		char *pNames = pTrailingData + iTextLen + iIDLength;
+		for (NameList::iterator it = pNameList->begin(); it != pNameList->end(); it++)
+		{
+			const char *szName = *it;
+			int iLen = strlen(szName);
+			strncpy(pNames, szName, iLen + 1);
+			pNames += iLen + 1;
+		}
+	}
+
 	bool OK = false;
-	if (m_pConnection->Send((char*)(&EditQueueRequest), sizeof(EditQueueRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&EditQueueRequest), sizeof(EditQueueRequest)))
 	{
 		perror("m_pConnection->Send");
 	}
@@ -823,7 +916,28 @@ bool RemoteClient::RequestServerShutdown()
 	SNZBShutdownRequest ShutdownRequest;
 	InitMessageBase(&ShutdownRequest.m_MessageBase, eRemoteRequestShutdown, sizeof(ShutdownRequest));
 
-	bool OK = m_pConnection->Send((char*)(&ShutdownRequest), sizeof(ShutdownRequest)) >= 0;
+	bool OK = m_pConnection->Send((char*)(&ShutdownRequest), sizeof(ShutdownRequest));
+	if (OK)
+	{
+		OK = ReceiveBoolResponse();
+	}
+	else
+	{
+		perror("m_pConnection->Send");
+	}
+
+	m_pConnection->Disconnect();
+	return OK;
+}
+
+bool RemoteClient::RequestServerReload()
+{
+	if (!InitConnection()) return false;
+
+	SNZBReloadRequest ReloadRequest;
+	InitMessageBase(&ReloadRequest.m_MessageBase, eRemoteRequestReload, sizeof(ReloadRequest));
+
+	bool OK = m_pConnection->Send((char*)(&ReloadRequest), sizeof(ReloadRequest));
 	if (OK)
 	{
 		OK = ReceiveBoolResponse();
@@ -844,7 +958,7 @@ bool RemoteClient::RequestServerVersion()
 	SNZBVersionRequest VersionRequest;
 	InitMessageBase(&VersionRequest.m_MessageBase, eRemoteRequestVersion, sizeof(VersionRequest));
 
-	bool OK = m_pConnection->Send((char*)(&VersionRequest), sizeof(VersionRequest)) >= 0;
+	bool OK = m_pConnection->Send((char*)(&VersionRequest), sizeof(VersionRequest));
 	if (OK)
 	{
 		OK = ReceiveBoolResponse();
@@ -865,7 +979,7 @@ bool RemoteClient::RequestPostQueue()
 	SNZBPostQueueRequest PostQueueRequest;
 	InitMessageBase(&PostQueueRequest.m_MessageBase, eRemoteRequestPostQueue, sizeof(PostQueueRequest));
 
-	if (m_pConnection->Send((char*)(&PostQueueRequest), sizeof(PostQueueRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&PostQueueRequest), sizeof(PostQueueRequest)))
 	{
 		perror("m_pConnection->Send");
 		return false;
@@ -875,19 +989,12 @@ bool RemoteClient::RequestPostQueue()
 
 	// Now listen for the returned list
 	SNZBPostQueueResponse PostQueueResponse;
-	int iResponseLen = m_pConnection->Recv((char*) &PostQueueResponse, sizeof(PostQueueResponse));
-	if (iResponseLen != sizeof(PostQueueResponse) || 
+	bool bRead = m_pConnection->Recv((char*) &PostQueueResponse, sizeof(PostQueueResponse));
+	if (!bRead || 
 		(int)ntohl(PostQueueResponse.m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE ||
 		ntohl(PostQueueResponse.m_MessageBase.m_iStructSize) != sizeof(PostQueueResponse))
 	{
-		if (iResponseLen < 0)
-		{
-			printf("No response received (timeout)\n");
-		}
-		else
-		{
-			printf("Invalid response received: either not nzbget-server or wrong server version\n");
-		}
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
 		return false;
 	}
 
@@ -895,7 +1002,7 @@ bool RemoteClient::RequestPostQueue()
 	if (ntohl(PostQueueResponse.m_iTrailingDataLength) > 0)
 	{
 		pBuf = (char*)malloc(ntohl(PostQueueResponse.m_iTrailingDataLength));
-		if (!m_pConnection->RecvAll(pBuf, ntohl(PostQueueResponse.m_iTrailingDataLength)))
+		if (!m_pConnection->Recv(pBuf, ntohl(PostQueueResponse.m_iTrailingDataLength)))
 		{
 			free(pBuf);
 			return false;
@@ -920,15 +1027,14 @@ bool RemoteClient::RequestPostQueue()
 
 			int iStageProgress = ntohl(pPostQueueAnswer->m_iStageProgress);
 
-			static const int EXECUTING_SCRIPT = 5;
 			char szCompleted[100];
 			szCompleted[0] = '\0';
-			if (iStageProgress > 0 && (int)ntohl(pPostQueueAnswer->m_iStage) != EXECUTING_SCRIPT)
+			if (iStageProgress > 0 && (int)ntohl(pPostQueueAnswer->m_iStage) != (int)PostInfo::ptExecutingScript)
 			{
 				sprintf(szCompleted, ", %i%s", (int)(iStageProgress / 10), "%");
 			}
 
-			const char* szPostStageName[] = { "", ", Loading Pars", ", Verifying source files", ", Repairing", ", Verifying repaired files", ", Executing postprocess-script", "" };
+			const char* szPostStageName[] = { "", ", Loading Pars", ", Verifying source files", ", Repairing", ", Verifying repaired files", ", Unpacking", ", Executing postprocess-script", "" };
 			char* szInfoName = pBufPtr + sizeof(SNZBPostQueueResponseEntry) + ntohl(pPostQueueAnswer->m_iNZBFilenameLen) + ntohl(pPostQueueAnswer->m_iParFilename);
 			
 			printf("[%i] %s%s%s\n", ntohl(pPostQueueAnswer->m_iID), szInfoName, szPostStageName[ntohl(pPostQueueAnswer->m_iStage)], szCompleted);
@@ -956,7 +1062,7 @@ bool RemoteClient::RequestWriteLog(int iKind, const char* szText)
 	int iLength = strlen(szText) + 1;
 	WriteLogRequest.m_iTrailingDataLength = htonl(iLength);
 
-	if (m_pConnection->Send((char*)(&WriteLogRequest), sizeof(WriteLogRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&WriteLogRequest), sizeof(WriteLogRequest)))
 	{
 		perror("m_pConnection->Send");
 		return false;
@@ -968,14 +1074,16 @@ bool RemoteClient::RequestWriteLog(int iKind, const char* szText)
 	return OK;
 }
 
-bool RemoteClient::RequestScan()
+bool RemoteClient::RequestScan(bool bSyncMode)
 {
 	if (!InitConnection()) return false;
 
 	SNZBScanRequest ScanRequest;
 	InitMessageBase(&ScanRequest.m_MessageBase, eRemoteRequestScan, sizeof(ScanRequest));
 
-	bool OK = m_pConnection->Send((char*)(&ScanRequest), sizeof(ScanRequest)) >= 0;
+	ScanRequest.m_bSyncMode = htonl(bSyncMode);
+
+	bool OK = m_pConnection->Send((char*)(&ScanRequest), sizeof(ScanRequest));
 	if (OK)
 	{
 		OK = ReceiveBoolResponse();
@@ -996,7 +1104,7 @@ bool RemoteClient::RequestHistory()
 	SNZBHistoryRequest HistoryRequest;
 	InitMessageBase(&HistoryRequest.m_MessageBase, eRemoteRequestHistory, sizeof(HistoryRequest));
 
-	if (m_pConnection->Send((char*)(&HistoryRequest), sizeof(HistoryRequest)) < 0)
+	if (!m_pConnection->Send((char*)(&HistoryRequest), sizeof(HistoryRequest)))
 	{
 		perror("m_pConnection->Send");
 		return false;
@@ -1006,19 +1114,12 @@ bool RemoteClient::RequestHistory()
 
 	// Now listen for the returned list
 	SNZBHistoryResponse HistoryResponse;
-	int iResponseLen = m_pConnection->Recv((char*) &HistoryResponse, sizeof(HistoryResponse));
-	if (iResponseLen != sizeof(HistoryResponse) || 
+	bool bRead = m_pConnection->Recv((char*) &HistoryResponse, sizeof(HistoryResponse));
+	if (!bRead || 
 		(int)ntohl(HistoryResponse.m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE ||
 		ntohl(HistoryResponse.m_MessageBase.m_iStructSize) != sizeof(HistoryResponse))
 	{
-		if (iResponseLen < 0)
-		{
-			printf("No response received (timeout)\n");
-		}
-		else
-		{
-			printf("Invalid response received: either not nzbget-server or wrong server version\n");
-		}
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
 		return false;
 	}
 
@@ -1026,7 +1127,7 @@ bool RemoteClient::RequestHistory()
 	if (ntohl(HistoryResponse.m_iTrailingDataLength) > 0)
 	{
 		pBuf = (char*)malloc(ntohl(HistoryResponse.m_iTrailingDataLength));
-		if (!m_pConnection->RecvAll(pBuf, ntohl(HistoryResponse.m_iTrailingDataLength)))
+		if (!m_pConnection->Recv(pBuf, ntohl(HistoryResponse.m_iTrailingDataLength)))
 		{
 			free(pBuf);
 			return false;
@@ -1049,27 +1150,33 @@ bool RemoteClient::RequestHistory()
 		{
 			SNZBHistoryResponseEntry* pListAnswer = (SNZBHistoryResponseEntry*) pBufPtr;
 
-			const char* szFileName = pBufPtr + sizeof(SNZBHistoryResponseEntry);
+			HistoryInfo::EKind eKind = (HistoryInfo::EKind)ntohl(pListAnswer->m_iKind);
+			const char* szNicename = pBufPtr + sizeof(SNZBHistoryResponseEntry);
 
-			long long lSize = Util::JoinInt64(ntohl(pListAnswer->m_iSizeHi), ntohl(pListAnswer->m_iSizeLo));
+			if (eKind == HistoryInfo::hkNZBInfo)
+			{
+				long long lSize = Util::JoinInt64(ntohl(pListAnswer->m_iSizeHi), ntohl(pListAnswer->m_iSizeLo));
 
-			char szNZBNiceName[1024];
-			NZBInfo::MakeNiceNZBName(szFileName, szNZBNiceName, 1024);
+				char szSize[20];
+				Util::FormatFileSize(szSize, sizeof(szSize), lSize);
 
-			char szSize[20];
-			Util::FormatFileSize(szSize, sizeof(szSize), lSize);
+				const char* szParStatusText[] = { "", ", Par failed", ", Par possible", ", Par successful" };
+				const char* szScriptStatusText[] = { "", ", Script status unknown", ", Script failed", ", Script successful" };
 
-			const char* szParStatusText[] = { "", ", Par failed", ", Par possible", ", Par successful" };
-			const char* szScriptStatusText[] = { "", ", Script status unknown", ", Script failed", ", Script successful" };
+				printf("[%i] %s (%i files, %s%s%s)\n", ntohl(pListAnswer->m_iID), szNicename, 
+					ntohl(pListAnswer->m_iFileCount), szSize, 
+					szParStatusText[ntohl(pListAnswer->m_iParStatus)], 
+					szScriptStatusText[ntohl(pListAnswer->m_iScriptStatus)]);
+			}
+			else if (eKind == HistoryInfo::hkUrlInfo)
+			{
+				const char* szUrlStatusText[] = { "", "", "Url download successful", "Url download failed", "" };
 
-			printf("[%i] %s (%i files, %s%s%s)\n", ntohl(pListAnswer->m_iID), szNZBNiceName, 
-				ntohl(pListAnswer->m_iFileCount), szSize, 
-				szParStatusText[ntohl(pListAnswer->m_iParStatus)], 
-				szScriptStatusText[ntohl(pListAnswer->m_iScriptStatus)]);
+				printf("[%i] %s (%s)\n", ntohl(pListAnswer->m_iID), szNicename, 
+					szUrlStatusText[ntohl(pListAnswer->m_iUrlStatus)]);
+			}
 
-			pBufPtr += sizeof(SNZBHistoryResponseEntry) + ntohl(pListAnswer->m_iFilenameLen) +
-				ntohl(pListAnswer->m_iDestDirLen) + ntohl(pListAnswer->m_iCategoryLen) + 
-				ntohl(pListAnswer->m_iQueuedFilenameLen);
+			pBufPtr += sizeof(SNZBHistoryResponseEntry) + ntohl(pListAnswer->m_iNicenameLen);
 		}
 
 		printf("-----------------------------------\n");
@@ -1080,3 +1187,117 @@ bool RemoteClient::RequestHistory()
 
 	return true;
 }
+
+bool RemoteClient::RequestServerDownloadUrl(const char* szURL, const char* szNZBFilename, const char* szCategory, bool bAddFirst, bool bAddPaused, int iPriority)
+{
+	if (!InitConnection()) return false;
+
+	SNZBDownloadUrlRequest DownloadUrlRequest;
+	InitMessageBase(&DownloadUrlRequest.m_MessageBase, eRemoteRequestDownloadUrl, sizeof(DownloadUrlRequest));
+	DownloadUrlRequest.m_bAddFirst = htonl(bAddFirst);
+	DownloadUrlRequest.m_bAddPaused = htonl(bAddPaused);
+	DownloadUrlRequest.m_iPriority = htonl(iPriority);
+
+	strncpy(DownloadUrlRequest.m_szURL, szURL, NZBREQUESTFILENAMESIZE - 1);
+	DownloadUrlRequest.m_szURL[NZBREQUESTFILENAMESIZE-1] = '\0';
+
+	DownloadUrlRequest.m_szCategory[0] = '\0';
+	if (szCategory)
+	{
+		strncpy(DownloadUrlRequest.m_szCategory, szCategory, NZBREQUESTFILENAMESIZE - 1);
+	}
+	DownloadUrlRequest.m_szCategory[NZBREQUESTFILENAMESIZE-1] = '\0';
+
+	DownloadUrlRequest.m_szNZBFilename[0] = '\0';
+	if (szNZBFilename)
+	{
+		strncpy(DownloadUrlRequest.m_szNZBFilename, szNZBFilename, NZBREQUESTFILENAMESIZE - 1);
+	}
+	DownloadUrlRequest.m_szNZBFilename[NZBREQUESTFILENAMESIZE-1] = '\0';
+
+	bool OK = m_pConnection->Send((char*)(&DownloadUrlRequest), sizeof(DownloadUrlRequest));
+	if (OK)
+	{
+		OK = ReceiveBoolResponse();
+	}
+	else
+	{
+		perror("m_pConnection->Send");
+	}
+
+	m_pConnection->Disconnect();
+	return OK;
+}
+
+bool RemoteClient::RequestUrlQueue()
+{
+	if (!InitConnection()) return false;
+
+	SNZBUrlQueueRequest UrlQueueRequest;
+	InitMessageBase(&UrlQueueRequest.m_MessageBase, eRemoteRequestUrlQueue, sizeof(UrlQueueRequest));
+
+	if (!m_pConnection->Send((char*)(&UrlQueueRequest), sizeof(UrlQueueRequest)))
+	{
+		perror("m_pConnection->Send");
+		return false;
+	}
+
+	printf("Request sent\n");
+
+	// Now listen for the returned list
+	SNZBUrlQueueResponse UrlQueueResponse;
+	bool bRead = m_pConnection->Recv((char*) &UrlQueueResponse, sizeof(UrlQueueResponse));
+	if (!bRead || 
+		(int)ntohl(UrlQueueResponse.m_MessageBase.m_iSignature) != (int)NZBMESSAGE_SIGNATURE ||
+		ntohl(UrlQueueResponse.m_MessageBase.m_iStructSize) != sizeof(UrlQueueResponse))
+	{
+		printf("No response or invalid response (timeout, not nzbget-server or wrong nzbget-server version)\n");
+		return false;
+	}
+
+	char* pBuf = NULL;
+	if (ntohl(UrlQueueResponse.m_iTrailingDataLength) > 0)
+	{
+		pBuf = (char*)malloc(ntohl(UrlQueueResponse.m_iTrailingDataLength));
+		if (!m_pConnection->Recv(pBuf, ntohl(UrlQueueResponse.m_iTrailingDataLength)))
+		{
+			free(pBuf);
+			return false;
+		}
+	}
+
+	m_pConnection->Disconnect();
+
+	if (ntohl(UrlQueueResponse.m_iTrailingDataLength) == 0)
+	{
+		printf("Server has no urls queued for download\n");
+	}
+	else
+	{
+		printf("Url-Queue\n");
+		printf("-----------------------------------\n");
+
+		char* pBufPtr = (char*)pBuf;
+		for (unsigned int i = 0; i < ntohl(UrlQueueResponse.m_iNrTrailingEntries); i++)
+		{
+			SNZBUrlQueueResponseEntry* pUrlQueueAnswer = (SNZBUrlQueueResponseEntry*) pBufPtr;
+
+			const char* szURL = pBufPtr + sizeof(SNZBUrlQueueResponseEntry);
+			const char* szTitle = pBufPtr + sizeof(SNZBUrlQueueResponseEntry) + ntohl(pUrlQueueAnswer->m_iURLLen);
+
+			char szNiceName[1024];
+			UrlInfo::MakeNiceName(szURL, szTitle, szNiceName, 1024);
+
+			printf("[%i] %s\n", ntohl(pUrlQueueAnswer->m_iID), szNiceName);
+
+			pBufPtr += sizeof(SNZBUrlQueueResponseEntry) + ntohl(pUrlQueueAnswer->m_iURLLen) + 
+				ntohl(pUrlQueueAnswer->m_iNZBFilenameLen);
+		}
+
+		free(pBuf);
+
+		printf("-----------------------------------\n");
+	}
+
+	return true;
+}
