@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,16 +15,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 381 $
- * $Date: 2010-02-08 17:08:10 +0100 (Mon, 08 Feb 2010) $
+ * $Revision: 590 $
+ * $Date: 2013-03-10 23:19:46 +0100 (Sun, 10 Mar 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -43,14 +43,18 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <stdarg.h>
 
 #include "nzbget.h"
 #include "ScriptController.h"
 #include "Log.h"
 #include "Util.h"
 
+// System global variable holding environments variables
+extern char** environ;
+
 extern Options* g_pOptions;
-extern char* (*szEnvironmentVariables)[];
+extern char* (*g_szEnvironmentVariables)[];
 extern DownloadQueueHolder* g_pDownloadQueueHolder;
 
 static const int POSTPROCESS_PARCHECK_CURRENT = 91;
@@ -120,9 +124,9 @@ EnvironmentStrings::~EnvironmentStrings()
 
 void EnvironmentStrings::InitFromCurrentProcess()
 {
-	for (int i = 0; (*szEnvironmentVariables)[i]; i++)
+	for (int i = 0; (*g_szEnvironmentVariables)[i]; i++)
 	{
-		char* szVar = (*szEnvironmentVariables)[i];
+		char* szVar = (*g_szEnvironmentVariables)[i];
 		Append(strdup(szVar));
 	}
 }
@@ -214,7 +218,7 @@ void ScriptController::SetEnvVar(const char* szName, const char* szValue)
 	m_environmentStrings.Append(szVar);
 }
 
-void ScriptController::PrepareEnvironmentStrings()
+void ScriptController::PrepareEnvOptions()
 {
 	Options::OptEntries* pOptEntries = g_pOptions->LockOptEntries();
 
@@ -227,10 +231,7 @@ void ScriptController::PrepareEnvironmentStrings()
 		// convert to upper case; replace "." with "_".
 		for (char* szPtr = szVarname; *szPtr; szPtr++)
 		{
-			if (*szPtr == '.')
-			{
-				*szPtr = '_';
-			}
+			if (*szPtr == '.') *szPtr = '_';
 			*szPtr = toupper(*szPtr);
 		}
 
@@ -241,15 +242,40 @@ void ScriptController::PrepareEnvironmentStrings()
 	g_pOptions->UnlockOptEntries();
 }
 
+void ScriptController::PrepareEnvParameters(NZBInfo* pNZBInfo)
+{
+	for (NZBParameterList::iterator it = pNZBInfo->GetParameters()->begin(); it != pNZBInfo->GetParameters()->end(); it++)
+	{
+		NZBParameter* pParameter = *it;
+		char szVarname[1024];
+		snprintf(szVarname, sizeof(szVarname), "NZBPR_%s", pParameter->GetName());
+		szVarname[1024-1] = '\0';
+
+		// Original name
+		SetEnvVar(szVarname, pParameter->GetValue());
+
+		char szNormVarname[1024];
+		strncpy(szNormVarname, szVarname, sizeof(szVarname));
+		szNormVarname[1024-1] = '\0';
+
+		// replace ".*:"  with "_".
+		for (char* szPtr = szNormVarname; *szPtr; szPtr++)
+		{
+			if (*szPtr == '.' || *szPtr == ':' || *szPtr == '*') *szPtr = '_';
+			*szPtr = toupper(*szPtr);
+		}
+
+		// Another env var with normalized name (replaced special chars and converted to upper case)
+		if (strcmp(szVarname, szNormVarname))
+		{
+			SetEnvVar(szNormVarname, pParameter->GetValue());
+		}
+	}
+}
+
 int ScriptController::Execute()
 {
-	if (!Util::FileExists(m_szScript))
-	{
-		error("Could not start %s: could not find file %s", m_szInfoName, m_szScript);
-		return -1;
-	}
-
-	PrepareEnvironmentStrings();
+	PrepareEnvOptions();
 
 	int iExitCode = 0;
 	int pipein;
@@ -299,8 +325,7 @@ int ScriptController::Execute()
 		DWORD dwErrCode = GetLastError();
 		char szErrMsg[255];
 		szErrMsg[255-1] = '\0';
-		if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM || FORMAT_MESSAGE_IGNORE_INSERTS || FORMAT_MESSAGE_ARGUMENT_ARRAY, 
-			NULL, dwErrCode, 0, szErrMsg, 255, NULL))
+		if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwErrCode, 0, szErrMsg, 255, NULL))
 		{
 			error("Could not start %s: %s", m_szInfoName, szErrMsg);
 		}
@@ -308,6 +333,10 @@ int ScriptController::Execute()
 		{
 			error("Could not start %s: error %i", m_szInfoName, dwErrCode);
 		}
+		if (!Util::FileExists(m_szScript))
+		{
+			error("Could not find file %s", m_szScript);
+		}
 		free(szEnvironmentStrings);
 		return -1;
 	}
@@ -370,10 +399,14 @@ int ScriptController::Execute()
 		fflush(stdout);
 #endif
 
-		execve(m_szScript, (char* const*)m_szArgs, (char* const*)pEnvironmentStrings);
-		fprintf(stdout, "[ERROR] Could not start script: %s", strerror(errno));
+		chdir(m_szWorkingDir);
+		environ = pEnvironmentStrings;
+		execvp(m_szScript, (char* const*)m_szArgs);
+		// NOTE: the text "[ERROR] Could not start " is checked later,
+		// by changing adjust the dependent code below.
+		fprintf(stdout, "[ERROR] Could not start %s: %s", m_szScript, strerror(errno));
 		fflush(stdout);
-		_exit(-1);
+		_exit(254);
 	}
 
 	// continue the first instance
@@ -407,9 +440,11 @@ int ScriptController::Execute()
 	char* buf = (char*)malloc(10240);
 
 	debug("Entering pipe-loop");
+	bool bFirstLine = true;
+	bool bStartError = false;
 	while (!feof(readpipe) && !m_bTerminated)
 	{
-		if (fgets(buf, 10240, readpipe))
+		if (ReadLine(buf, 10240, readpipe))
 		{
 #ifdef CHILD_WATCHDOG
 			if (!bChildConfirmed)
@@ -417,9 +452,15 @@ int ScriptController::Execute()
 				bChildConfirmed = true;
 				pWatchDog->Stop();
 				debug("Child confirmed");
+				continue;
 			}
 #endif
+			if (bFirstLine && !strncmp(buf, "[ERROR] Could not start ", 24))
+			{
+				bStartError = true;
+			}
 			ProcessOutput(buf);
+			bFirstLine = false;
 		}
 	}
 	debug("Exited pipe-loop");
@@ -458,6 +499,10 @@ int ScriptController::Execute()
 	if (WIFEXITED(iStatus))
 	{
 		iExitCode = WEXITSTATUS(iStatus);
+		if (iExitCode == 254 && bStartError)
+		{
+			iExitCode = -1;
+		}
 	}
 #endif
 	
@@ -503,6 +548,11 @@ void ScriptController::Terminate()
 	debug("Stopped %s", m_szInfoName);
 }
 
+bool ScriptController::ReadLine(char* szBuf, int iBufSize, FILE* pStream)
+{
+	return fgets(szBuf, iBufSize, pStream);
+}
+
 void ScriptController::ProcessOutput(char* szText)
 {
 	debug("Processing output received from script");
@@ -517,23 +567,23 @@ void ScriptController::ProcessOutput(char* szText)
 
 	if (!strncmp(szText, "[INFO] ", 7))
 	{
-		AddMessage(Message::mkInfo, false, g_pOptions->GetInfoTarget(), szText + 7);
+		AddMessage(Message::mkInfo, false, szText + 7);
 	}
 	else if (!strncmp(szText, "[WARNING] ", 10))
 	{
-		AddMessage(Message::mkWarning, false, g_pOptions->GetWarningTarget(), szText + 10);
+		AddMessage(Message::mkWarning, false, szText + 10);
 	}
 	else if (!strncmp(szText, "[ERROR] ", 8))
 	{
-		AddMessage(Message::mkError, false, g_pOptions->GetErrorTarget(), szText + 8);
+		AddMessage(Message::mkError, false, szText + 8);
 	}
 	else if (!strncmp(szText, "[DETAIL] ", 9))
 	{
-		AddMessage(Message::mkDetail, false, g_pOptions->GetDetailTarget(), szText + 9);
+		AddMessage(Message::mkDetail, false, szText + 9);
 	}
 	else if (!strncmp(szText, "[DEBUG] ", 8))
 	{
-		AddMessage(Message::mkDebug, false, g_pOptions->GetDebugTarget(), szText + 8);
+		AddMessage(Message::mkDebug, false, szText + 8);
 	}
 	else 
 	{	
@@ -543,23 +593,23 @@ void ScriptController::ProcessOutput(char* szText)
 				break;
 
 			case Options::slDetail:
-				AddMessage(Message::mkDetail, true, g_pOptions->GetDetailTarget(), szText);
+				AddMessage(Message::mkDetail, true, szText);
 				break;
 
 			case Options::slInfo:
-				AddMessage(Message::mkInfo, true, g_pOptions->GetInfoTarget(), szText);
+				AddMessage(Message::mkInfo, true, szText);
 				break;
 
 			case Options::slWarning:
-				AddMessage(Message::mkWarning, true, g_pOptions->GetWarningTarget(), szText);
+				AddMessage(Message::mkWarning, true, szText);
 				break;
 
 			case Options::slError:
-				AddMessage(Message::mkError, true, g_pOptions->GetErrorTarget(), szText);
+				AddMessage(Message::mkError, true, szText);
 				break;
 
 			case Options::slDebug:
-				AddMessage(Message::mkDebug, true, g_pOptions->GetDebugTarget(), szText);
+				AddMessage(Message::mkDebug, true, szText);
 				break;
 		}
 	}
@@ -567,7 +617,7 @@ void ScriptController::ProcessOutput(char* szText)
 	debug("Processing output received from script - completed");
 }
 
-void ScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, Options::EMessageTarget eMessageTarget, const char* szText)
+void ScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, const char* szText)
 {
 	switch (eKind)
 	{
@@ -593,32 +643,53 @@ void ScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, Optio
 	}
 }
 
-void PostScriptController::StartScriptJob(PostInfo* pPostInfo, const char* szScript, bool bNZBFileCompleted, bool bHasFailedParJobs)
+void ScriptController::PrintMessage(Message::EKind eKind, const char* szFormat, ...)
 {
-	info("Executing post-process-script for %s", pPostInfo->GetInfoName());
+	char tmp2[1024];
 
+	va_list ap;
+	va_start(ap, szFormat);
+	vsnprintf(tmp2, 1024, szFormat, ap);
+	tmp2[1024-1] = '\0';
+	va_end(ap);
+	
+	AddMessage(eKind, false, tmp2);
+}
+
+void PostScriptController::StartScriptJob(PostInfo* pPostInfo, bool bNZBFileCompleted, bool bHasFailedParJobs)
+{
 	PostScriptController* pScriptController = new PostScriptController();
 	pScriptController->m_pPostInfo = pPostInfo;
-	pScriptController->SetScript(szScript);
+	pScriptController->SetScript(g_pOptions->GetPostProcess());
 	pScriptController->SetWorkingDir(g_pOptions->GetDestDir());
 	pScriptController->m_bNZBFileCompleted = bNZBFileCompleted;
 	pScriptController->m_bHasFailedParJobs = bHasFailedParJobs;
 	pScriptController->SetAutoDestroy(false);
 
-	pPostInfo->SetScriptThread(pScriptController);
+	pPostInfo->SetPostThread(pScriptController);
 
 	pScriptController->Start();
 }
 
 void PostScriptController::Run()
 {
-	// the locking is needed for accessing the memebers of NZBInfo
+	// the locking is needed for accessing the members of NZBInfo
 	g_pDownloadQueueHolder->LockQueue();
 
+	char szNZBName[1024];
+	strncpy(szNZBName, m_pPostInfo->GetNZBInfo()->GetName(), 1024);
+	szNZBName[1024-1] = '\0';
+
+	int iParStatus[] = { 0, 0, 1, 2, 3 };
 	char szParStatus[10];
-	snprintf(szParStatus, 10, "%i", m_pPostInfo->GetParStatus());
+	snprintf(szParStatus, 10, "%i", iParStatus[g_pOptions->GetAllowReProcess() ? (int)m_pPostInfo->GetParStatus() : (int)m_pPostInfo->GetNZBInfo()->GetParStatus()]);
 	szParStatus[10-1] = '\0';
 
+	int iUnpackStatus[] = { 0, 0, 1, 2 };
+	char szUnpackStatus[10];
+	snprintf(szUnpackStatus, 10, "%i", g_pOptions->GetAllowReProcess() ? 0 : iUnpackStatus[m_pPostInfo->GetNZBInfo()->GetUnpackStatus()]);
+	szUnpackStatus[10-1] = '\0';
+
 	char szCollectionCompleted[10];
 	snprintf(szCollectionCompleted, 10, "%i", (int)m_bNZBFileCompleted);
 	szCollectionCompleted[10-1] = '\0';
@@ -644,7 +715,7 @@ void PostScriptController::Run()
 	szCategory[1024-1] = '\0';
 
 	char szInfoName[1024];
-	snprintf(szInfoName, 1024, "post-process-script for %s", m_pPostInfo->GetInfoName());
+	snprintf(szInfoName, 1024, "post-process-script for %s", g_pOptions->GetAllowReProcess() ? m_pPostInfo->GetInfoName() : m_pPostInfo->GetNZBInfo()->GetName());
 	szInfoName[1024-1] = '\0';
 	SetInfoName(szInfoName);
 
@@ -663,35 +734,35 @@ void PostScriptController::Run()
 	szArgs[8] = NULL;
 	SetArgs(szArgs, false);
 
+	SetEnvVar("NZBPP_NZBNAME", szNZBName);
 	SetEnvVar("NZBPP_DIRECTORY", szDestDir);
 	SetEnvVar("NZBPP_NZBFILENAME", szNZBFilename);
 	SetEnvVar("NZBPP_PARFILENAME", szParFilename);
 	SetEnvVar("NZBPP_PARSTATUS", szParStatus);
+	SetEnvVar("NZBPP_UNPACKSTATUS", szUnpackStatus);
 	SetEnvVar("NZBPP_NZBCOMPLETED", szCollectionCompleted);
 	SetEnvVar("NZBPP_PARFAILED", szHasFailedParJobs);
 	SetEnvVar("NZBPP_CATEGORY", szCategory);
 
-	for (NZBParameterList::iterator it = m_pPostInfo->GetNZBInfo()->GetParameters()->begin(); it != m_pPostInfo->GetNZBInfo()->GetParameters()->end(); it++)
-	{
-		NZBParameter* pParameter = *it;
-		char szVarname[1024];
-		snprintf(szVarname, sizeof(szVarname), "NZBPR_%s", pParameter->GetName());
-		szVarname[1024-1] = '\0';
-		SetEnvVar(szVarname, pParameter->GetValue());
-	}
+	PrepareEnvParameters(m_pPostInfo->GetNZBInfo());
 
 	g_pDownloadQueueHolder->UnlockQueue();
 
+	info("Executing post-process-script for %s", g_pOptions->GetAllowReProcess() ? m_pPostInfo->GetInfoName() : szNZBName);
+
 	int iResult = Execute();
 
+	szInfoName[0] = 'P'; // uppercase
+
 	switch (iResult)
 	{
 		case POSTPROCESS_SUCCESS:
-			info("%s sucessful", szInfoName);
+			info("%s successful", szInfoName);
 			m_pPostInfo->SetScriptStatus(PostInfo::srSuccess);
 			break;
 
 		case POSTPROCESS_ERROR:
+		case -1: // Execute() returns -1 if the process could not be started (file not found or other problem)
 			info("%s failed", szInfoName);
 			m_pPostInfo->SetScriptStatus(PostInfo::srFailure);
 			break;
@@ -703,7 +774,7 @@ void PostScriptController::Run()
 
 #ifndef DISABLE_PARCHECK
 		case POSTPROCESS_PARCHECK_ALL:
-			if (m_pPostInfo->GetParCheck())
+			if (m_pPostInfo->GetParStatus() > PostInfo::psSkipped)
 			{
 				error("%s requested par-check/repair for all collections, but they were already checked", szInfoName);
 				m_pPostInfo->SetScriptStatus(PostInfo::srFailure);
@@ -722,7 +793,7 @@ void PostScriptController::Run()
 			break;
 
 		case POSTPROCESS_PARCHECK_CURRENT:
-			if (m_pPostInfo->GetParCheck())
+			if (m_pPostInfo->GetParStatus() > PostInfo::psSkipped)
 			{
 				error("%s requested par-check/repair for current collection, but it was already checked", szInfoName);
 				m_pPostInfo->SetScriptStatus(PostInfo::srFailure);
@@ -750,26 +821,18 @@ void PostScriptController::Run()
 	m_pPostInfo->SetWorking(false);
 }
 
-void PostScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, Options::EMessageTarget eMessageTarget, const char* szText)
+void PostScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, const char* szText)
 {
 	if (!strncmp(szText, "[HISTORY] ", 10))
 	{
-		if (eMessageTarget == Options::mtScreen || eMessageTarget == Options::mtBoth)
-		{
-			m_pPostInfo->GetNZBInfo()->AppendMessage(eKind, 0, szText + 10);
-		}
+		m_pPostInfo->GetNZBInfo()->AppendMessage(eKind, 0, szText + 10);
 	}
 	else
 	{
-		ScriptController::AddMessage(eKind, bDefaultKind, eMessageTarget, szText);
-
-		if (eMessageTarget == Options::mtScreen || eMessageTarget == Options::mtBoth)
-		{
-			m_pPostInfo->AppendMessage(eKind, szText);
-		}
+		ScriptController::AddMessage(eKind, bDefaultKind, szText);
+		m_pPostInfo->AppendMessage(eKind, szText);
 	}
 
-
 	if (g_pOptions->GetPausePostProcess())
 	{
 		time_t tStageTime = m_pPostInfo->GetStageTime();
@@ -806,7 +869,7 @@ void PostScriptController::Stop()
 }
 
 void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBFilename, 
-	const char* szDirectory, char** pCategory, NZBParameterList* pParameterList)
+	const char* szDirectory, char** pCategory, int* iPriority, NZBParameterList* pParameterList)
 {
 	info("Executing nzb-process-script for %s", Util::BaseFileName(szNZBFilename));
 
@@ -814,6 +877,7 @@ void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBF
 	pScriptController->SetScript(szScript);
 	pScriptController->m_pCategory = pCategory;
 	pScriptController->m_pParameterList = pParameterList;
+	pScriptController->m_iPriority = iPriority;
 
 	char szInfoName[1024];
 	snprintf(szInfoName, 1024, "nzb-process-script for %s", Util::BaseFileName(szNZBFilename));
@@ -848,7 +912,7 @@ void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBF
 	delete pScriptController;
 }
 
-void NZBScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, Options::EMessageTarget eMessageTarget, const char* szText)
+void NZBScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, const char* szText)
 {
 	if (!strncmp(szText, "[NZB] ", 6))
 	{
@@ -873,6 +937,10 @@ void NZBScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, Op
 			}
 			free(szParam);
 		}
+		else if (!strncmp(szText + 6, "PRIORITY=", 9))
+		{
+			*m_iPriority = atoi(szText + 6 + 9);
+		}
 		else
 		{
 			error("Invalid command \"%s\" received from %s", szText, GetInfoName());
@@ -880,10 +948,72 @@ void NZBScriptController::AddMessage(Message::EKind eKind, bool bDefaultKind, Op
 	}
 	else
 	{
-		ScriptController::AddMessage(eKind, bDefaultKind, eMessageTarget, szText);
+		ScriptController::AddMessage(eKind, bDefaultKind, szText);
 	}
 }
 
+void NZBAddedScriptController::StartScript(DownloadQueue* pDownloadQueue, NZBInfo *pNZBInfo, const char* szScript)
+{
+	NZBAddedScriptController* pScriptController = new NZBAddedScriptController();
+	pScriptController->SetScript(szScript);
+	pScriptController->m_szNZBName = strdup(pNZBInfo->GetName());
+	pScriptController->SetEnvVar("NZBNA_NAME", pNZBInfo->GetName());
+	pScriptController->SetEnvVar("NZBNA_FILENAME", pNZBInfo->GetFilename());
+	pScriptController->SetEnvVar("NZBNA_CATEGORY", pNZBInfo->GetCategory());
+
+	int iLastID = 0;
+	int iMaxPriority = 0;
+
+	for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
+    {
+        FileInfo* pFileInfo = *it;
+		if (pFileInfo->GetNZBInfo() == pNZBInfo && ( pFileInfo->GetPriority() > iMaxPriority || iLastID == 0))
+		{
+			iMaxPriority = pFileInfo->GetPriority();
+		}
+		if (pFileInfo->GetNZBInfo() == pNZBInfo && pFileInfo->GetID() > iLastID)
+		{
+			iLastID = pFileInfo->GetID();
+		}
+	}
+
+	char buf[100];
+
+	snprintf(buf, 100, "%i", iLastID);
+	pScriptController->SetEnvVar("NZBNA_LASTID", buf);
+
+	snprintf(buf, 100, "%i", iMaxPriority);
+	pScriptController->SetEnvVar("NZBNA_PRIORITY", buf);
+
+	pScriptController->PrepareEnvParameters(pNZBInfo);
+
+	pScriptController->SetAutoDestroy(true);
+
+	pScriptController->Start();
+}
+
+void NZBAddedScriptController::Run()
+{
+	char szInfoName[1024];
+	snprintf(szInfoName, 1024, "nzb-added process-script for %s", m_szNZBName);
+	szInfoName[1024-1] = '\0';
+	SetInfoName(szInfoName);
+
+	info("Executing %s", szInfoName);
+
+	SetDefaultKindPrefix("NZB-Added Process: ");
+	SetDefaultLogKind(g_pOptions->GetProcessLogKind());
+
+	const char* szArgs[2];
+	szArgs[0] = GetScript();
+	szArgs[1] = NULL;
+	SetArgs(szArgs, false);
+
+	Execute();
+
+	free(m_szNZBName);
+}
+
 void SchedulerScriptController::StartScript(const char* szCommandLine)
 {
 	char** argv = NULL;
