@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2012 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,16 +16,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 380 $
- * $Date: 2010-01-30 15:43:58 +0100 (Sat, 30 Jan 2010) $
+ * $Revision: 542 $
+ * $Date: 2013-01-17 20:07:13 +0100 (Thu, 17 Jan 2013) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef WIN32
@@ -66,6 +66,7 @@
 #include "ColoredFrontend.h"
 #include "NCursesFrontend.h"
 #include "QueueCoordinator.h"
+#include "UrlCoordinator.h"
 #include "RemoteServer.h"
 #include "RemoteClient.h"
 #include "MessageBase.h"
@@ -79,7 +80,9 @@
 #endif
 
 // Prototypes
-void Run();
+void RunMain();
+void Run(bool bReload);
+void Reload();
 void Cleanup();
 void ProcessClientRequest();
 #ifndef WIN32
@@ -101,14 +104,19 @@ Thread* g_pFrontend = NULL;
 Options* g_pOptions = NULL;
 ServerPool* g_pServerPool = NULL;
 QueueCoordinator* g_pQueueCoordinator = NULL;
+UrlCoordinator* g_pUrlCoordinator = NULL;
 RemoteServer* g_pRemoteServer = NULL;
+RemoteServer* g_pRemoteSecureServer = NULL;
 DownloadSpeedMeter* g_pDownloadSpeedMeter = NULL;
 DownloadQueueHolder* g_pDownloadQueueHolder = NULL;
 Log* g_pLog = NULL;
 PrePostProcessor* g_pPrePostProcessor = NULL;
 DiskState* g_pDiskState = NULL;
 Scheduler* g_pScheduler = NULL;
-char* (*szEnvironmentVariables)[] = NULL;
+int g_iArgumentCount;
+char* (*g_szEnvironmentVariables)[] = NULL;
+char* (*g_szArguments)[] = NULL;
+bool g_bReloading = true;
 
 /*
  * Main loop
@@ -137,17 +145,67 @@ int main(int argc, char *argv[], char *argp[])
 	DisableCout();
 #endif
 
+	g_iArgumentCount = argc;
+	g_szArguments = (char*(*)[])argv;
+	g_szEnvironmentVariables = (char*(*)[])argp;
+
+#ifdef WIN32
+	for (int i=0; i < argc; i++)
+	{
+		if (!strcmp(argv[i], "-D"))
+		{
+			StartService(RunMain);
+			return 0;
+		}
+	}
+#endif
+
+	RunMain();
+
+#ifdef WIN32
+#ifdef _DEBUG
+	_CrtDumpMemoryLeaks();
+#endif
+#endif
+
+	return 0;
+}
+
+void RunMain()
+{
+	// we need to save and later restore current directory each time
+	// the program is reloaded (RPC-Method "reload") in order for
+	// config to properly load in a case relative paths are used 
+	// in command line
+	char szCurDir[MAX_PATH + 1];
+	Util::GetCurrentDirectory(szCurDir, sizeof(szCurDir));
+	
+	bool bReload = false;
+	while (g_bReloading)
+	{
+		g_bReloading = false;
+		Util::SetCurrentDirectory(szCurDir);
+		Run(bReload);
+		bReload = true;
+	}
+}
+
+void Run(bool bReload)
+{
 	g_pLog = new Log();
 
 	debug("nzbget %s", Util::VersionRevision());
 
+	if (!bReload)
+	{
+		Thread::Init();
+	}
+
 	g_pServerPool = new ServerPool();
 	g_pScheduler = new Scheduler();
-	Thread::Init();
 
 	debug("Reading options");
-	g_pOptions = new Options(argc, argv);
-	szEnvironmentVariables = (char*(*)[])argp;
+	g_pOptions = new Options(g_iArgumentCount, *g_szArguments);
 
 #ifndef WIN32
 	if (g_pOptions->GetUMask() < 01000)
@@ -165,7 +223,7 @@ int main(int argc, char *argv[], char *argp[])
 
 	g_pLog->InitOptions();
 
-	if (g_pOptions->GetDaemonMode())
+	if (g_pOptions->GetDaemonMode() && !bReload)
 	{
 #ifdef WIN32
 		info("nzbget %s service-mode", Util::VersionRevision());
@@ -174,7 +232,7 @@ int main(int argc, char *argv[], char *argp[])
 		info("nzbget %s daemon-mode", Util::VersionRevision());
 #endif
 	}
-	else if (g_pOptions->GetServerMode())
+	else if (g_pOptions->GetServerMode() && !bReload)
 	{
 		info("nzbget %s server-mode", Util::VersionRevision());
 	}
@@ -183,6 +241,11 @@ int main(int argc, char *argv[], char *argp[])
 		info("nzbget %s remote-mode", Util::VersionRevision());
 	}
 
+	if (!bReload)
+	{
+		Connection::Init();
+	}
+
 	if (!g_pOptions->GetRemoteClientMode())
 	{
 		g_pServerPool->InitConnections();
@@ -191,13 +254,7 @@ int main(int argc, char *argv[], char *argp[])
 #endif
 	}
 
-#ifdef WIN32
-	if (g_pOptions->GetDaemonMode())
-	{
-		StartService(Run);
-		return 0;
-	}
-#else
+#ifndef WIN32
 #ifdef HAVE_SYS_PRCTL_H
 	if (g_pOptions->GetDumpCore())
 	{
@@ -206,19 +263,6 @@ int main(int argc, char *argv[], char *argp[])
 #endif
 #endif
 
-	Run();
-
-#ifdef WIN32
-#ifdef _DEBUG
-	_CrtDumpMemoryLeaks();
-#endif
-#endif
-
-	return 0;
-}
-
-void Run()
-{
 #ifndef WIN32
 	InstallSignalHandlers();
 #ifdef DEBUG
@@ -228,9 +272,6 @@ void Run()
 	}
 #endif
 #endif
-	
-	Connection::Init(g_pOptions->GetTLS() && !g_pOptions->GetRemoteClientMode() &&
-		(g_pOptions->GetClientOperation() == Options::opClientNoOperation));
 
 	// client request
 	if (g_pOptions->GetClientOperation() != Options::opClientNoOperation)
@@ -246,13 +287,21 @@ void Run()
 		g_pQueueCoordinator = new QueueCoordinator();
 		g_pDownloadSpeedMeter = g_pQueueCoordinator;
 		g_pDownloadQueueHolder = g_pQueueCoordinator;
+
+		g_pUrlCoordinator = new UrlCoordinator();
 	}
 
 	// Setup the network-server
 	if (g_pOptions->GetServerMode())
 	{
-		g_pRemoteServer = new RemoteServer();
+		g_pRemoteServer = new RemoteServer(false);
 		g_pRemoteServer->Start();
+
+		if (g_pOptions->GetSecureControl())
+		{
+			g_pRemoteSecureServer = new RemoteServer(true);
+			g_pRemoteSecureServer->Start();
+		}
 	}
 
 	// Creating PrePostProcessor
@@ -292,7 +341,7 @@ void Run()
 		// Standalone-mode
 		if (!g_pOptions->GetServerMode())
 		{
-			NZBFile* pNZBFile = NZBFile::CreateFromFile(g_pOptions->GetArgFilename(), g_pOptions->GetCategory() ? g_pOptions->GetCategory() : "");
+			NZBFile* pNZBFile = NZBFile::CreateFromFile(g_pOptions->GetArgFilename(), g_pOptions->GetAddCategory() ? g_pOptions->GetAddCategory() : "");
 			if (!pNZBFile)
 			{
 				abort("FATAL ERROR: Parsing NZB-document %s failed\n\n", g_pOptions->GetArgFilename() ? g_pOptions->GetArgFilename() : "N/A");
@@ -308,18 +357,28 @@ void Run()
 		}
 
 		g_pQueueCoordinator->Start();
+		g_pUrlCoordinator->Start();
 		g_pPrePostProcessor->Start();
 
 		// enter main program-loop
-		while (g_pQueueCoordinator->IsRunning() || g_pPrePostProcessor->IsRunning())
+		while (g_pQueueCoordinator->IsRunning() || 
+			g_pUrlCoordinator->IsRunning() || 
+			g_pPrePostProcessor->IsRunning())
 		{
-			if (!g_pOptions->GetServerMode() && !g_pQueueCoordinator->HasMoreJobs() && !g_pPrePostProcessor->HasMoreJobs())
+			if (!g_pOptions->GetServerMode() && 
+				!g_pQueueCoordinator->HasMoreJobs() && 
+				!g_pUrlCoordinator->HasMoreJobs() && 
+				!g_pPrePostProcessor->HasMoreJobs())
 			{
 				// Standalone-mode: download completed
 				if (!g_pQueueCoordinator->IsStopped())
 				{
 					g_pQueueCoordinator->Stop();
 				}
+				if (!g_pUrlCoordinator->IsStopped())
+				{
+					g_pUrlCoordinator->Stop();
+				}
 				if (!g_pPrePostProcessor->IsStopped())
 				{
 					g_pPrePostProcessor->Stop();
@@ -330,6 +389,7 @@ void Run()
 
 		// main program-loop is terminated
 		debug("QueueCoordinator stopped");
+		debug("UrlCoordinator stopped");
 		debug("PrePostProcessor stopped");
 	}
 
@@ -351,7 +411,25 @@ void Run()
 		}
 		debug("RemoteServer stopped");
 	}
-	
+
+	if (g_pRemoteSecureServer)
+	{
+		debug("stopping RemoteSecureServer");
+		g_pRemoteSecureServer->Stop();
+		int iMaxWaitMSec = 1000;
+		while (g_pRemoteSecureServer->IsRunning() && iMaxWaitMSec > 0)
+		{
+			usleep(100 * 1000);
+			iMaxWaitMSec -= 100;
+		}
+		if (g_pRemoteSecureServer->IsRunning())
+		{
+			debug("Killing RemoteSecureServer");
+			g_pRemoteSecureServer->Kill();
+		}
+		debug("RemoteSecureServer stopped");
+	}
+
 	// Stop Frontend
 	if (g_pFrontend)
 	{
@@ -377,15 +455,15 @@ void ProcessClientRequest()
 	switch (g_pOptions->GetClientOperation())
 	{
 		case Options::opClientRequestListFiles:
-			Client->RequestServerList(true, false);
+			Client->RequestServerList(true, false, g_pOptions->GetMatchMode() == Options::mmRegEx ? g_pOptions->GetEditQueueText() : NULL);
 			break;
 
 		case Options::opClientRequestListGroups:
-			Client->RequestServerList(false, true);
+			Client->RequestServerList(false, true, g_pOptions->GetMatchMode() == Options::mmRegEx ? g_pOptions->GetEditQueueText() : NULL);
 			break;
 
 		case Options::opClientRequestListStatus:
-			Client->RequestServerList(false, false);
+			Client->RequestServerList(false, false, NULL);
 			break;
 
 		case Options::opClientRequestDownloadPause:
@@ -414,7 +492,8 @@ void ProcessClientRequest()
 
 		case Options::opClientRequestEditQueue:
 			Client->RequestServerEditQueue((eRemoteEditAction)g_pOptions->GetEditQueueAction(), g_pOptions->GetEditQueueOffset(),
-				g_pOptions->GetEditQueueText(), g_pOptions->GetEditQueueIDList(), g_pOptions->GetEditQueueIDCount(), true);
+				g_pOptions->GetEditQueueText(), g_pOptions->GetEditQueueIDList(), g_pOptions->GetEditQueueIDCount(), 
+				g_pOptions->GetEditQueueNameList(), (eRemoteMatchMode)g_pOptions->GetMatchMode(), true);
 			break;
 
 		case Options::opClientRequestLog:
@@ -425,8 +504,12 @@ void ProcessClientRequest()
 			Client->RequestServerShutdown();
 			break;
 
+		case Options::opClientRequestReload:
+			Client->RequestServerReload();
+			break;
+
 		case Options::opClientRequestDownload:
-			Client->RequestServerDownload(g_pOptions->GetArgFilename(), g_pOptions->GetCategory(), g_pOptions->GetAddTop());
+			Client->RequestServerDownload(g_pOptions->GetArgFilename(), g_pOptions->GetAddCategory(), g_pOptions->GetAddTop(), g_pOptions->GetAddPaused(), g_pOptions->GetAddPriority());
 			break;
 
 		case Options::opClientRequestVersion:
@@ -441,8 +524,12 @@ void ProcessClientRequest()
 			Client->RequestWriteLog(g_pOptions->GetWriteLogKind(), g_pOptions->GetLastArg());
 			break;
 
-		case Options::opClientRequestScan:
-			Client->RequestScan();
+		case Options::opClientRequestScanAsync:
+			Client->RequestScan(false);
+			break;
+
+		case Options::opClientRequestScanSync:
+			Client->RequestScan(true);
 			break;
 
 		case Options::opClientRequestPostPause:
@@ -465,6 +552,14 @@ void ProcessClientRequest()
 			Client->RequestHistory();	 
 			break;
 
+		case Options::opClientRequestDownloadUrl:
+			Client->RequestServerDownloadUrl(g_pOptions->GetLastArg(), g_pOptions->GetAddNZBFilename(), g_pOptions->GetAddCategory(), g_pOptions->GetAddTop(), g_pOptions->GetAddPaused(), g_pOptions->GetAddPriority());
+			break;
+
+		case Options::opClientRequestUrlQueue:
+			Client->RequestUrlQueue();
+			break;
+
 		case Options::opClientNoOperation:
 			break;
 	}
@@ -474,7 +569,10 @@ void ProcessClientRequest()
 
 void ExitProc()
 {
-	info("Stopping, please wait...");
+	if (!g_bReloading)
+	{
+		info("Stopping, please wait...");
+	}
 	if (g_pOptions->GetRemoteClientMode())
 	{
 		if (g_pFrontend)
@@ -489,11 +587,19 @@ void ExitProc()
 		{
 			debug("Stopping QueueCoordinator");
 			g_pQueueCoordinator->Stop();
+			g_pUrlCoordinator->Stop();
 			g_pPrePostProcessor->Stop();
 		}
 	}
 }
 
+void Reload()
+{
+	g_bReloading = true;
+	info("Reloading...");
+	ExitProc();
+}
+
 #ifndef WIN32
 #ifdef DEBUG
 typedef void(*sighandler)(int);
@@ -522,22 +628,10 @@ void SignalProc(int iSignal)
 			break;
 
 #ifdef DEBUG
-		case SIGPIPE:
-			// ignoring
-			break;
-			
 		case SIGSEGV:
 			signal(SIGSEGV, SIG_DFL);   // Reset the signal handler
 			PrintBacktrace();
 			break;
-		
-		default:
-			// printf("Signal %i received\n", iSignal);
-			if (SignalProcList[iSignal - 1])
-			{
-				SignalProcList[iSignal - 1](iSignal);
-			}
-			break;
 #endif
 	}
 }
@@ -546,15 +640,14 @@ void InstallSignalHandlers()
 {
 	signal(SIGINT, SignalProc);
 	signal(SIGTERM, SignalProc);
-	signal(SIGCHLD, SignalProc);
 	signal(SIGPIPE, SIG_IGN);
 #ifdef DEBUG
-	SignalProcList.clear();
-	for (int i = 1; i <= 32; i++)
-	{
-		SignalProcList.push_back((sighandler)signal(i, SignalProc));
-	}
-	signal(SIGWINCH, SIG_DFL);
+	signal(SIGSEGV, SignalProc);
+#endif
+#ifdef SIGCHLD_HANDLER
+    // it could be necessary on some systems to activate a handler for SIGCHLD
+    // however it make troubles on other systems and is deactivated by default
+	signal(SIGCHLD, SignalProc);
 #endif
 }
 
@@ -619,13 +712,13 @@ void Cleanup()
 {
 	debug("Cleaning up global objects");
 
-	debug("Deleting QueueCoordinator");
-	if (g_pQueueCoordinator)
+	debug("Deleting UrlCoordinator");
+	if (g_pUrlCoordinator)
 	{
-		delete g_pQueueCoordinator;
-		g_pQueueCoordinator = NULL;
+		delete g_pUrlCoordinator;
+		g_pUrlCoordinator = NULL;
 	}
-	debug("QueueCoordinator deleted");
+	debug("UrlCoordinator deleted");
 
 	debug("Deleting RemoteServer");
 	if (g_pRemoteServer)
@@ -635,6 +728,14 @@ void Cleanup()
 	}
 	debug("RemoteServer deleted");
 
+	debug("Deleting RemoteSecureServer");
+	if (g_pRemoteSecureServer)
+	{
+		delete g_pRemoteSecureServer;
+		g_pRemoteSecureServer = NULL;
+	}
+	debug("RemoteSecureServer deleted");
+
 	debug("Deleting PrePostProcessor");
 	if (g_pPrePostProcessor)
 	{
@@ -651,6 +752,14 @@ void Cleanup()
 	}
 	debug("Frontend deleted");
 
+	debug("Deleting QueueCoordinator");
+	if (g_pQueueCoordinator)
+	{
+		delete g_pQueueCoordinator;
+		g_pQueueCoordinator = NULL;
+	}
+	debug("QueueCoordinator deleted");
+
 	debug("Deleting DiskState");
 	if (g_pDiskState)
 	{
@@ -662,7 +771,7 @@ void Cleanup()
 	debug("Deleting Options");
 	if (g_pOptions)
 	{
-		if (g_pOptions->GetDaemonMode())
+		if (g_pOptions->GetDaemonMode() && !g_bReloading)
 		{
 			info("Deleting lock file");
 			remove(g_pOptions->GetLockFile());
@@ -688,9 +797,11 @@ void Cleanup()
 	}
 	debug("Scheduler deleted");
 
-	Connection::Final();
-
-	Thread::Final();
+	if (!g_bReloading)
+	{
+		Connection::Final();
+		Thread::Final();
+	}
 
 	debug("Global objects cleaned up");
 
